package com.qmth.sop.business.bean.params;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.sop.common.enums.InOutTypeEnum;
import io.swagger.annotations.ApiModelProperty;
import org.hibernate.validator.constraints.Range;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

/**
 * @Description: 打卡参数
 * @Author: CaoZixuan
 * @Date: 2023-09-05
 */
public class DingSaveParam {

    @ApiModelProperty("服务单元id")
    @NotNull(message = "缺少服务单元")
    @Range(min = 1L, message = "缺少服务单元")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long serviceId;

    @ApiModelProperty("sop号")
    private String sopNo;

    @ApiModelProperty("签到时间")
    @NotNull(message = "缺少签到时间")
    @Range(min = 1L, message = "缺少签到时间")
    private Long signTime;

    @ApiModelProperty("签到类型")
    @NotNull(message = "缺少签到类型")
    private InOutTypeEnum signType;

    @ApiModelProperty("签到地址")
    @NotNull(message = "缺少签到地址")
    @NotBlank(message = "缺少签到地址")
    private String signAddress;

    @ApiModelProperty("人脸对比照片地址")
    private String facePhotoPath;

    @ApiModelProperty("x坐标")
    @NotNull(message = "缺少x坐标")
    @NotBlank(message = "缺少x坐标")
    private String axisX;

    @ApiModelProperty("y坐标")
    @NotNull(message = "缺少y坐标")
    @NotBlank(message = "缺少y坐标")
    private String axisY;

    public Long getServiceId() {
        return serviceId;
    }

    public void setServiceId(Long serviceId) {
        this.serviceId = serviceId;
    }

    public String getSopNo() {
        return sopNo;
    }

    public void setSopNo(String sopNo) {
        this.sopNo = sopNo;
    }

    public Long getSignTime() {
        return signTime;
    }

    public void setSignTime(Long signTime) {
        this.signTime = signTime;
    }

    public InOutTypeEnum getSignType() {
        return signType;
    }

    public void setSignType(InOutTypeEnum signType) {
        this.signType = signType;
    }

    public String getSignAddress() {
        return signAddress;
    }

    public void setSignAddress(String signAddress) {
        this.signAddress = signAddress;
    }

    public String getFacePhotoPath() {
        return facePhotoPath;
    }

    public void setFacePhotoPath(String facePhotoPath) {
        this.facePhotoPath = facePhotoPath;
    }

    public String getAxisX() {
        return axisX;
    }

    public void setAxisX(String axisX) {
        this.axisX = axisX;
    }

    public String getAxisY() {
        return axisY;
    }

    public void setAxisY(String axisY) {
        this.axisY = axisY;
    }
}
