package com.qmth.sop.business.bean.params;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import com.qmth.sop.common.enums.SopQuickSearchTypeEnum;
import com.qmth.sop.common.enums.TFCustomTypeEnum;
import io.swagger.annotations.ApiModelProperty;
import org.hibernate.validator.constraints.Length;

import javax.validation.constraints.NotNull;
import java.io.Serializable;

/**
 * @Description: sop快捷搜索参数
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2024/5/21
 */
public class SopQuickSearchParam implements Serializable {

    private static final long serialVersionUID = 1L;

    @ApiModelProperty(value = "主键")
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    @ApiModelProperty(value = "流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程")
    @NotNull(message = "流程类型不能为空")
    private TFCustomTypeEnum type;

    @ApiModelProperty(value = "内容名称")
    @NotNull(message = "内容名称不能为空")
    @Length(message = "内容名称不能超过{max}个字符", max = 50)
    private String name;

    @ApiModelProperty(value = "内容")
    @NotNull(message = "内容不能为空")
    private String content;

    @ApiModelProperty(value = "内容类型，VIEW：显示列表，SEARCH：快捷搜索")
    @NotNull(message = "内容类型不能为空")
    private SopQuickSearchTypeEnum contentType;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public TFCustomTypeEnum getType() {
        return type;
    }

    public void setType(TFCustomTypeEnum type) {
        this.type = type;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getContent() {
        return content;
    }

    public void setContent(String content) {
        this.content = content;
    }

    public SopQuickSearchTypeEnum getContentType() {
        return contentType;
    }

    public void setContentType(SopQuickSearchTypeEnum contentType) {
        this.contentType = contentType;
    }
}
