package com.qmth.sop.business.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.IService;
import com.qmth.sop.business.bean.dto.UserArchivesAllocationDto;
import com.qmth.sop.business.bean.dto.UserArchivesDto;
import com.qmth.sop.business.bean.params.AutoAllocationParam;
import com.qmth.sop.business.bean.params.UserArchivesAllocationParam;
import com.qmth.sop.business.bean.result.CrmArchivesAllocationResult;
import com.qmth.sop.business.bean.result.CrmDetailAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationSubTotalResult;
import com.qmth.sop.business.entity.TBUserArchivesAllocation;
import com.qmth.sop.common.enums.ProductTypeEnum;
import com.qmth.sop.common.enums.SopRoleTypeCompositeEnum;
import com.qmth.sop.common.enums.SopRoleTypeEnum;

import java.util.List;

/**
 * <p>
 * 人员分配表 服务类
 * </p>
 *
 * @author wangliang
 * @since 2023-08-01
 */
public interface TBUserArchivesAllocationService extends IService<TBUserArchivesAllocation> {

    /**
     * 查询派单分配列表
     *
     * @param serviceUnitId 服务单元id
     * @param province      省份名称
     * @param city          城市名称
     * @param area          县区名称
     * @param customName    客户名称
     * @param gap           分配差额
     * @param regionId      大区id
     * @param pageNumber    分页页数
     * @param pageSize      分页容量
     * @return 分页查询结果
     */
    @Deprecated
    IPage<UserArchivesAllocationResult> findCrmAllocationPage(Long serviceUnitId, String province, String city,
            String area, String customName, Integer gap, Long regionId, Integer pageNumber, Integer pageSize);

    /**
     * 查询派单小计
     *
     * @param serviceUnitId 服务单元id
     * @param province      省份名称
     * @param city          城市名称
     * @param area          县区名称
     * @param customName    客户名称
     * @param gap           分配差额
     * @param regionId      大区id
     * @return 小计结果
     */
    @Deprecated
    UserArchivesAllocationSubTotalResult findCrmAllocationSubTotal(Long serviceUnitId, String province, String city,
            String area, String customName, Integer gap, Long regionId);

    /**
     * 人员调配
     *
     * @param userArchivesAllocationParam 调配参数
     */
    void editCrmAllocation(UserArchivesAllocationParam userArchivesAllocationParam) throws InterruptedException;

    /**
     * 根据派单号查询派单分配信息
     *
     * @param crmNo 派单号
     * @return 分配信息
     */
    List<CrmArchivesAllocationResult> findAllocationByCrmNo(String crmNo);

    /**
     * 自动批量调配派单和人员
     *
     * @param autoAllocationParam 自动调配参数
     */
    @Deprecated
    void autoEditCrmAllocationBatch(AutoAllocationParam autoAllocationParam);

    /**
     * crm派单 sop流程发布
     *
     * @param crmNo crmNo
     */
    void publishSop(String crmNo) throws Exception;

    /**
     * crm派单 sop流程撤销发布
     *
     * @param crmNo crmNo
     */
    void unPublishSop(String crmNo);

    /**
     * 查询用户在某派单中执行的工程师
     *
     * @param userId 用户id
     * @param sopNo  sopNo
     * @return 角色
     */
    List<SopRoleTypeEnum> findSopRoleTypeByUserSopNo(Long userId, String sopNo);

    /**
     * 查询派单详情中可以分配的人员
     *
     * @param productTypeEnum
     * @param crmDetailId     派单详情id
     * @param sopRoleTypeEnum 角色身份
     * @return 可选择人员集合
     */
    List<UserArchivesDto> findCanChooseArchives(ProductTypeEnum productTypeEnum, Long crmDetailId,
            SopRoleTypeEnum sopRoleTypeEnum);

    /**
     * 查询派单详情中可以分配的人员(复合角色)
     *
     * @param productTypeEnum
     * @param crmDetailId
     * @param sopRoleTypeCompositeEnum
     * @return
     */
    List<UserArchivesDto> findCanChooseArchives(ProductTypeEnum productTypeEnum, Long crmDetailId,
            SopRoleTypeCompositeEnum sopRoleTypeCompositeEnum);

    /**
     * 根据派单明细id查询派单分配信息
     *
     * @param crmDetailId 派单明细id
     * @return 分配信息
     */
    CrmDetailAllocationResult findCrmDetailAllocationInfo(Long crmDetailId);

    /**
     *  派单明细ID查找所有相关人员
     * @param crmDetailId 派单明细id
     * @return 人员信息
     */
    List<UserArchivesAllocationDto> listUserArchivesAllocation(Long crmDetailId);
}
