-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                    `name` varchar(100) NOT NULL COMMENT '文件名',
                                    `type` varchar(10) NOT NULL COMMENT '文件类型',
                                    `size` decimal(8,2) NOT NULL COMMENT '单位（KB）',
                                    `md5` varchar(32) NOT NULL COMMENT '文件md5值',
                                    `path` varchar(2000) NOT NULL COMMENT '存储路径',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
                                    `remark` varchar(500) DEFAULT NULL,
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';

-- ----------------------------
-- Records of basic_attachment
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `code` varchar(20) NOT NULL COMMENT '学校编码',
                                `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
                                `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
                                `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
                                `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
                                `remark` text COMMENT '备注',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                `logo` mediumtext COMMENT '学校logo',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校表';

-- ----------------------------
-- Records of basic_school
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for basic_sms_message
-- ----------------------------
DROP TABLE IF EXISTS `basic_sms_message`;
CREATE TABLE `basic_sms_message` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint NOT NULL COMMENT '消息接收人用户',
                                     `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                                     `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
                                     `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
                                     `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
                                     `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
                                     `variable_params` text COMMENT '变量参数内容',
                                     `template_content` mediumtext COMMENT '模板内容',
                                     `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
                                     `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
                                     `remark` text COMMENT '备注',
                                     `resend_count` int DEFAULT NULL COMMENT '重试次数',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信通知表';

-- ----------------------------
-- Records of basic_sms_message
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint NOT NULL COMMENT '用户ID',
                                     `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
                                     `expire_time` bigint NOT NULL COMMENT '到期时间',
                                     `valid_period` int NOT NULL COMMENT '单位（分）',
                                     `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Records of basic_verify_code
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_blob_triggers`;
CREATE TABLE `qrtz_blob_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `BLOB_DATA` blob,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                      KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_blob_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_calendars`;
CREATE TABLE `qrtz_calendars` (
                                  `SCHED_NAME` varchar(120) NOT NULL,
                                  `CALENDAR_NAME` varchar(200) NOT NULL,
                                  `CALENDAR` blob NOT NULL,
                                  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_calendars
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_cron_triggers`;
CREATE TABLE `qrtz_cron_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `CRON_EXPRESSION` varchar(120) NOT NULL,
                                      `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_cron_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_fired_triggers`;
CREATE TABLE `qrtz_fired_triggers` (
                                       `SCHED_NAME` varchar(120) NOT NULL,
                                       `ENTRY_ID` varchar(95) NOT NULL,
                                       `TRIGGER_NAME` varchar(200) NOT NULL,
                                       `TRIGGER_GROUP` varchar(200) NOT NULL,
                                       `INSTANCE_NAME` varchar(200) NOT NULL,
                                       `FIRED_TIME` bigint NOT NULL,
                                       `SCHED_TIME` bigint NOT NULL,
                                       `PRIORITY` int NOT NULL,
                                       `STATE` varchar(16) NOT NULL,
                                       `JOB_NAME` varchar(200) DEFAULT NULL,
                                       `JOB_GROUP` varchar(200) DEFAULT NULL,
                                       `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
                                       `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
                                       PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_fired_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_job_details`;
CREATE TABLE `qrtz_job_details` (
                                    `SCHED_NAME` varchar(120) NOT NULL,
                                    `JOB_NAME` varchar(200) NOT NULL,
                                    `JOB_GROUP` varchar(200) NOT NULL,
                                    `DESCRIPTION` varchar(250) DEFAULT NULL,
                                    `JOB_CLASS_NAME` varchar(250) NOT NULL,
                                    `IS_DURABLE` varchar(1) NOT NULL,
                                    `IS_NONCONCURRENT` varchar(1) NOT NULL,
                                    `IS_UPDATE_DATA` varchar(1) NOT NULL,
                                    `REQUESTS_RECOVERY` varchar(1) NOT NULL,
                                    `JOB_DATA` blob,
                                    PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                    KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                    KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_job_details
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_locks`;
CREATE TABLE `qrtz_locks` (
                              `SCHED_NAME` varchar(120) NOT NULL,
                              `LOCK_NAME` varchar(40) NOT NULL,
                              PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_locks
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_paused_trigger_grps`;
CREATE TABLE `qrtz_paused_trigger_grps` (
                                            `SCHED_NAME` varchar(120) NOT NULL,
                                            `TRIGGER_GROUP` varchar(200) NOT NULL,
                                            PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_paused_trigger_grps
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_scheduler_state`;
CREATE TABLE `qrtz_scheduler_state` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `INSTANCE_NAME` varchar(200) NOT NULL,
                                        `LAST_CHECKIN_TIME` bigint NOT NULL,
                                        `CHECKIN_INTERVAL` bigint NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_scheduler_state
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simple_triggers`;
CREATE TABLE `qrtz_simple_triggers` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `TRIGGER_NAME` varchar(200) NOT NULL,
                                        `TRIGGER_GROUP` varchar(200) NOT NULL,
                                        `REPEAT_COUNT` bigint NOT NULL,
                                        `REPEAT_INTERVAL` bigint NOT NULL,
                                        `TIMES_TRIGGERED` bigint NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_simple_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simprop_triggers`;
CREATE TABLE `qrtz_simprop_triggers` (
                                         `SCHED_NAME` varchar(120) NOT NULL,
                                         `TRIGGER_NAME` varchar(200) NOT NULL,
                                         `TRIGGER_GROUP` varchar(200) NOT NULL,
                                         `STR_PROP_1` varchar(512) DEFAULT NULL,
                                         `STR_PROP_2` varchar(512) DEFAULT NULL,
                                         `STR_PROP_3` varchar(512) DEFAULT NULL,
                                         `INT_PROP_1` int DEFAULT NULL,
                                         `INT_PROP_2` int DEFAULT NULL,
                                         `LONG_PROP_1` bigint DEFAULT NULL,
                                         `LONG_PROP_2` bigint DEFAULT NULL,
                                         `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
                                         `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
                                         `BOOL_PROP_1` varchar(1) DEFAULT NULL,
                                         `BOOL_PROP_2` varchar(1) DEFAULT NULL,
                                         PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_simprop_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_triggers`;
CREATE TABLE `qrtz_triggers` (
                                 `SCHED_NAME` varchar(120) NOT NULL,
                                 `TRIGGER_NAME` varchar(200) NOT NULL,
                                 `TRIGGER_GROUP` varchar(200) NOT NULL,
                                 `JOB_NAME` varchar(200) NOT NULL,
                                 `JOB_GROUP` varchar(200) NOT NULL,
                                 `DESCRIPTION` varchar(250) DEFAULT NULL,
                                 `NEXT_FIRE_TIME` bigint DEFAULT NULL,
                                 `PREV_FIRE_TIME` bigint DEFAULT NULL,
                                 `PRIORITY` int DEFAULT NULL,
                                 `TRIGGER_STATE` varchar(16) NOT NULL,
                                 `TRIGGER_TYPE` varchar(8) NOT NULL,
                                 `START_TIME` bigint NOT NULL,
                                 `END_TIME` bigint DEFAULT NULL,
                                 `CALENDAR_NAME` varchar(200) DEFAULT NULL,
                                 `MISFIRE_INSTR` smallint DEFAULT NULL,
                                 `JOB_DATA` blob,
                                 PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Records of qrtz_triggers
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sequence
-- ----------------------------
DROP TABLE IF EXISTS `sequence`;
CREATE TABLE `sequence` (
                            `name` varchar(50) NOT NULL COMMENT '名称',
                            `current_value` bigint NOT NULL DEFAULT '0' COMMENT '当前值',
                            `increment` int NOT NULL DEFAULT '1' COMMENT '增长值',
                            PRIMARY KEY (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='序列表';

-- ----------------------------
-- Records of sequence
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `org_id` bigint DEFAULT NULL COMMENT '机构id',
                              `config_key` varchar(100) NOT NULL COMMENT '参数键名',
                              `config_name` varchar(100) NOT NULL COMMENT '参数名称',
                              `config_value` varchar(100) DEFAULT NULL COMMENT '参数键值',
                              `remark` varchar(100) DEFAULT NULL COMMENT '备注',
                              `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `sort` int DEFAULT '1' COMMENT '排序',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

-- ----------------------------
-- Records of sys_config
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_custom
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom`;
CREATE TABLE `sys_custom` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `name` varchar(100) NOT NULL COMMENT '名称',
                              `code` varchar(100) DEFAULT NULL COMMENT '编码',
                              `type` varchar(20) DEFAULT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                              `province` varchar(100) NOT NULL COMMENT '省份',
                              `city` varchar(100) NOT NULL COMMENT '城市',
                              `area` varchar(100) DEFAULT NULL COMMENT '县区',
                              `address` varchar(150) DEFAULT NULL COMMENT '地址',
                              `level_id` bigint DEFAULT NULL COMMENT '档位id',
                              `peoper_day` int DEFAULT NULL COMMENT '标准人天',
                              `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户表';

-- ----------------------------
-- Records of sys_custom
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_custom_addr
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom_addr`;
CREATE TABLE `sys_custom_addr` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `custom_id` bigint NOT NULL COMMENT '客户ID',
                                   `province` varchar(100) NOT NULL COMMENT '省份',
                                   `city` varchar(100) NOT NULL COMMENT '城市',
                                   `area` varchar(100) NOT NULL COMMENT '县区',
                                   `address` varchar(150) NOT NULL COMMENT '地址',
                                   `longitude` varchar(255) NOT NULL COMMENT '经度',
                                   `latitude` varchar(255) NOT NULL COMMENT '纬度',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户地址表';

-- ----------------------------
-- Records of sys_custom_addr
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_device
-- ----------------------------
DROP TABLE IF EXISTS `sys_device`;
CREATE TABLE `sys_device` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `device_code` varchar(100) NOT NULL COMMENT '设备编号',
                              `serial_no` varchar(100) NOT NULL COMMENT '序列号',
                              `brand` varchar(100) NOT NULL COMMENT '品牌',
                              `model` varchar(255) NOT NULL COMMENT '型号',
                              `buy_time` bigint DEFAULT NULL COMMENT '购买时间',
                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                              `status` varchar(20) DEFAULT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                              `location` varchar(100) DEFAULT NULL COMMENT '当前所在地',
                              `location_arr` varchar(100) DEFAULT NULL COMMENT '当前所在地（回显）',
                              `scan_count` bigint NOT NULL DEFAULT '0' COMMENT '总扫描量',
                              `bound` varchar(20) DEFAULT NULL COMMENT '出库/入库',
                              `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `sys_device_un_serial_no` (`serial_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备配置表';

-- ----------------------------
-- Records of sys_device
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_device_brand
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_brand`;
CREATE TABLE `sys_device_brand` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `brand` varchar(100) NOT NULL COMMENT '品牌',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`),
                                    UNIQUE KEY `sys_device_brand_un_brand` (`brand`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌表';

-- ----------------------------
-- Records of sys_device_brand
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_device_model
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_model`;
CREATE TABLE `sys_device_model` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `brand_id` bigint NOT NULL COMMENT '品牌ID',
                                    `model` varchar(255) NOT NULL COMMENT '型号',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`),
                                    UNIQUE KEY `sys_device_model_un` (`brand_id`,`model`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌型号表';

-- ----------------------------
-- Records of sys_device_model
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_ding_date
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_date`;
CREATE TABLE `sys_ding_date` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `year` bigint NOT NULL COMMENT '年',
                                 `month` bigint NOT NULL COMMENT '月',
                                 `day` bigint NOT NULL COMMENT '日',
                                 `type` varchar(50) NOT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                 PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤特殊日期';

-- ----------------------------
-- Records of sys_ding_date
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_ding_group
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_group`;
CREATE TABLE `sys_ding_group` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `service_id` bigint NOT NULL COMMENT '服务单元id',
                                  `name` varchar(100) NOT NULL COMMENT '名称',
                                  `supplier_id` bigint NOT NULL COMMENT '供应商id',
                                  `sign_in_time` varchar(50) DEFAULT NULL COMMENT '签到时段',
                                  `sign_out_time` varchar(50) DEFAULT NULL COMMENT '签退时段',
                                  `face_open` tinyint NOT NULL DEFAULT '1' COMMENT '是否开启人脸，0：不开启，1：开启',
                                  `reissue_card_count` int DEFAULT NULL COMMENT '允许补卡次数',
                                  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组表';

-- ----------------------------
-- Records of sys_ding_group
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_ding_obj
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_obj`;
CREATE TABLE `sys_ding_obj` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `ding_group_id` bigint NOT NULL COMMENT '考勤组id',
                                `role_id` bigint DEFAULT NULL COMMENT '角色id',
                                `type` varchar(20) NOT NULL COMMENT '类型，DING：考勤，APPROVE：审核',
                                `sop_role_type` varchar(45) DEFAULT NULL COMMENT 'sop角色',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组对象表';

-- ----------------------------
-- Records of sys_ding_obj
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_job
-- ----------------------------
DROP TABLE IF EXISTS `sys_job`;
CREATE TABLE `sys_job` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                           `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                           `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                           `user_id` bigint NOT NULL COMMENT '责任人id',
                           `job_time` bigint NOT NULL COMMENT '任务时间',
                           `type` varchar(20) NOT NULL COMMENT '类型，BEFORE：任务前提醒，AFTER：任务后提醒',
                           `unit` varchar(20) NOT NULL COMMENT '单位，HOUR：小时，MINUTE：分钟',
                           `status` varchar(20) NOT NULL COMMENT '状态，NOT_EXEC：未执行，EXEC：已执行',
                           `exec_time` bigint NOT NULL COMMENT '执行时间',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统任务表';

-- ----------------------------
-- Records of sys_job
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_level
-- ----------------------------
DROP TABLE IF EXISTS `sys_level`;
CREATE TABLE `sys_level` (
                             `id` bigint NOT NULL COMMENT '主键',
                             `level` varchar(50) NOT NULL COMMENT '档位',
                             `devices` bigint DEFAULT NULL COMMENT '设备数量',
                             `type` varchar(20) NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                             `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                             `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='档位表';

-- ----------------------------
-- Records of sys_level
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_level_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_level_role`;
CREATE TABLE `sys_level_role` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `level_id` bigint NOT NULL COMMENT '服务档位id',
                                  `role_id` bigint NOT NULL COMMENT '角色id',
                                  `quota` bigint NOT NULL COMMENT '配额',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务档位角色配置表';

-- ----------------------------
-- Records of sys_level_role
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_log
-- ----------------------------
DROP TABLE IF EXISTS `sys_log`;
CREATE TABLE `sys_log` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `user_id` bigint NOT NULL COMMENT '用户id',
                           `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                           `type` varchar(20) NOT NULL COMMENT '类别，LOGIN：登录，LOGOUT：退出以及数据库DDL和DML操作',
                           `obj_name` varchar(1000) DEFAULT NULL COMMENT '实体名称',
                           `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                           `content` varchar(200) DEFAULT NULL COMMENT '内容',
                           `url` varchar(64) DEFAULT NULL COMMENT '请求的url',
                           `args` longtext COMMENT '请求的参数',
                           `update_before_obj` longtext COMMENT '更改前内容',
                           `update_after_obj` longtext COMMENT '更改后内容',
                           `create_time` bigint NOT NULL COMMENT '创建时间',
                           `run_time` bigint DEFAULT NULL COMMENT '请求的响应时间',
                           `run_status` varchar(64) DEFAULT NULL COMMENT '请求的结果',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统日志';

-- ----------------------------
-- Records of sys_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_message
-- ----------------------------
DROP TABLE IF EXISTS `sys_message`;
CREATE TABLE `sys_message` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                               `code` varchar(100) DEFAULT NULL COMMENT '编号',
                               `type` varchar(20) NOT NULL COMMENT '消息类型，BEFORE：提前提醒，AFTER：延期提醒，OFFICE_SOP：教务处SOP，CLOUD_MARK_SOP：研究生SOP，QUALITY：质量问题提醒，EXCEPTION_APPROVE：异常审核提醒，VIOLATION：违规提醒，SYSTEM_PLAN_CHANGE：系统计划变更提醒，SYSTEM：系统公告',
                               `obj_type` varchar(100) DEFAULT NULL COMMENT '实体业务类型',
                               `notice_id` bigint DEFAULT NULL COMMENT '公告id',
                               `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                               `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                               `form_user_id` bigint DEFAULT NULL COMMENT '发送人id',
                               `receive_user_id` bigint NOT NULL COMMENT '接收人id',
                               `content` varchar(10000) NOT NULL COMMENT '消息内容',
                               `send_time` bigint NOT NULL COMMENT '发送时间',
                               `receive_time` bigint DEFAULT NULL COMMENT '接收时间',
                               `status` tinyint NOT NULL DEFAULT '0' COMMENT '是否已阅，0：未阅，1：已阅',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统消息';

-- ----------------------------
-- Records of sys_message
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_notice
-- ----------------------------
DROP TABLE IF EXISTS `sys_notice`;
CREATE TABLE `sys_notice` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                              `title` varchar(200) NOT NULL COMMENT '标题',
                              `content` varchar(10000) NOT NULL COMMENT '内容',
                              `type` varchar(20) DEFAULT NULL COMMENT '类型，SYSTEM：公告，SUPPLIER：供应商，SERVICE：服务单元',
                              `status` varchar(20) DEFAULT NULL COMMENT '状态，PUBLISH：已发布，UN_PUBLISH：未发布',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `publish_time` bigint DEFAULT NULL COMMENT '发布日期',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建人时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='公告表';

-- ----------------------------
-- Records of sys_notice
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `type` varchar(50) DEFAULT NULL COMMENT '类型',
                           `code` varchar(20) DEFAULT NULL COMMENT '机构代码',
                           `name` varchar(100) NOT NULL COMMENT '机构名称',
                           `parent_id` bigint DEFAULT NULL COMMENT '上级机构id',
                           `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint DEFAULT NULL COMMENT '创建人',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint DEFAULT NULL COMMENT '修改人',
                           `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                           PRIMARY KEY (`id`) USING BTREE,
                           UNIQUE KEY `sys_org_UN_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='组织架构表';

-- ----------------------------
-- Records of sys_org
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `name` varchar(100) NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件',
                                 `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int NOT NULL COMMENT '序号',
                                 `property` varchar(30) DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

-- ----------------------------
-- Records of sys_privilege
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `name` varchar(50) NOT NULL COMMENT '角色名称',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `type` varchar(20) DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，PMO：总负责人，BUSSINESS：业务线负责人，REGION_MANAGER：大区经理，REGION_COORDINATOR：区域协调人，EFFECT_ENGINEER：实施工程师，ASSISTANT_ENGINEER：助理工程师，QA：QA，CUSTOM：技术客服',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                            `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
                            `interpret` varchar(1000) DEFAULT NULL COMMENT '角色说明',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_role_UN_type` (`name`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

-- ----------------------------
-- Records of sys_role
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
                                      `id` bigint NOT NULL COMMENT '主键',
                                      `role_id` bigint NOT NULL COMMENT '角色id',
                                      `privilege_id` bigint NOT NULL COMMENT '菜单id',
                                      `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Records of sys_role_privilege
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_supplier
-- ----------------------------
DROP TABLE IF EXISTS `sys_supplier`;
CREATE TABLE `sys_supplier` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `name` varchar(100) NOT NULL COMMENT '名称',
                                `code` varchar(100) NOT NULL COMMENT '编码',
                                `type` varchar(20) NOT NULL COMMENT '类型，HUMAN：人力，DEVICE：设备',
                                `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                `remark` varchar(500) DEFAULT NULL COMMENT '备注',
                                `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `sys_supplier_un_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='供应商表';

-- ----------------------------
-- Records of sys_supplier
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `login_name` varchar(100) NOT NULL COMMENT '用户名',
                            `real_name` varchar(100) NOT NULL COMMENT '姓名',
                            `code` varchar(100) DEFAULT NULL COMMENT '工号',
                            `password` varchar(50) NOT NULL COMMENT '密码',
                            `mobile_number` varchar(30) DEFAULT NULL COMMENT '手机号',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `gender` varchar(20) DEFAULT NULL COMMENT '性别，MAN：男，WOMAN：女',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
                            `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
                            `remark` varchar(200) DEFAULT NULL COMMENT '备注',
                            `source` varchar(20) DEFAULT 'SYSTEM' COMMENT '来源，SYSTEM：系统自带，ARCHIVES：档案',
                            `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                            `rtzf_sync_status` varchar(20) DEFAULT NULL COMMENT '软通智服同步状态',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_user_UN_login_name` (`login_name`),
                            UNIQUE KEY `sys_user_UN_moblie_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

-- ----------------------------
-- Records of sys_user
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `user_id` bigint NOT NULL COMMENT '用户id',
                                 `role_id` bigint NOT NULL COMMENT '角色id',
                                 `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

-- ----------------------------
-- Records of sys_user_role
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_crm
-- ----------------------------
DROP TABLE IF EXISTS `t_b_crm`;
CREATE TABLE `t_b_crm` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) NOT NULL COMMENT '项目单号',
                           `name` varchar(100) NOT NULL COMMENT '项目名称',
                           `begin_time` bigint NOT NULL COMMENT '派单时间',
                           `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                           `crm_user_id` bigint NOT NULL COMMENT '派单人id',
                           `custom_id` bigint NOT NULL COMMENT '客户id',
                           `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
                           `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
                           `product_id` bigint NOT NULL COMMENT '产品id',
                           `lead_id` bigint DEFAULT NULL COMMENT '大区经理id',
                           `lead_type` varchar(45) DEFAULT NULL COMMENT '大区经理指定类型（BY_PERSON("按人员指定"), BY_AREA("按地区指定");）',
                           `region_id` bigint DEFAULT NULL COMMENT '服务区域规划id',
                           `region_detail_id` bigint DEFAULT NULL COMMENT '服务区域规划明细id',
                           `region_coordinator_id` bigint DEFAULT NULL COMMENT '区域协调人id',
                           `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，false：否，true：是',
                           `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                           `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                           `status` varchar(20) NOT NULL COMMENT '派单状态(UN_PUBLISH("未发布"),PUBLISH("已发布"),FINISH("已完成");)',
                           `publish_user_id` bigint DEFAULT NULL COMMENT '发布人id',
                           `publish_time` bigint DEFAULT NULL COMMENT '发布时间',
                           `process_var` mediumtext COMMENT '过程变量',
                           PRIMARY KEY (`id`),
                           UNIQUE KEY `t_b_crm_un_crm_no` (`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='派单信息表';

-- ----------------------------
-- Records of t_b_crm
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_crm_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_crm_detail`;
CREATE TABLE `t_b_crm_detail` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `crm_no` varchar(100) NOT NULL COMMENT '派单号',
                                  `flow_deployment_id` varchar(50) NOT NULL COMMENT '流程部署id',
                                  `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                  `status` varchar(20) NOT NULL COMMENT '状态，UN_PUBLISH：未发布，PUBLISH：发布',
                                  `course_name` varchar(255) NOT NULL COMMENT '科目名称',
                                  `exam_student_count` int NOT NULL COMMENT '考生人数',
                                  `contacts` varchar(100) DEFAULT NULL COMMENT '联系人',
                                  `mobile_number` varchar(30) DEFAULT NULL COMMENT '电话',
                                  `service_scope` varchar(30) NOT NULL COMMENT '服务范围，SCAN：仅扫描，SCAN_MARK：扫描+阅卷',
                                  `scan_start_time` bigint NOT NULL COMMENT '扫描开始时间',
                                  `scan_end_time` bigint NOT NULL COMMENT '扫描结束时间',
                                  `mark_paper_start_time` bigint NOT NULL COMMENT '阅卷开始时间',
                                  `mark_paper_end_time` bigint NOT NULL COMMENT '阅卷结束时间',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  `publish_time` bigint DEFAULT NULL COMMENT '发布时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_crm_detail_unique` (`crm_no`,`course_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='派单明细';

-- ----------------------------
-- Records of t_b_crm_detail
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_delay_warn
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn`;
CREATE TABLE `t_b_delay_warn` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                  `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                  `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                  `message_id` bigint DEFAULT NULL COMMENT '消息id',
                                  `code` varchar(20) NOT NULL COMMENT '编号',
                                  `warn_time` bigint DEFAULT NULL COMMENT '预警时间',
                                  `user_id` bigint DEFAULT NULL COMMENT '负责人id',
                                  `type` varchar(20) NOT NULL COMMENT '类型，PLAN：关键信息及计划，TIME：处理时限',
                                  `field_obj` mediumtext COMMENT '预警字段',
                                  `status` varchar(20) NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  `close_time` bigint DEFAULT NULL COMMENT '关闭时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_delay_warn_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警跟进表';

-- ----------------------------
-- Records of t_b_delay_warn
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_delay_warn_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn_detail`;
CREATE TABLE `t_b_delay_warn_detail` (
                                         `id` bigint NOT NULL COMMENT '主键',
                                         `delay_warn_id` bigint NOT NULL COMMENT '延期预警id',
                                         `remark` varchar(500) NOT NULL COMMENT '跟进说明',
                                         `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                         `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警明细表';

-- ----------------------------
-- Records of t_b_delay_warn_detail
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_device_delivery
-- ----------------------------
DROP TABLE IF EXISTS `t_b_device_delivery`;
CREATE TABLE `t_b_device_delivery` (
                                       `id` bigint NOT NULL COMMENT '主键',
                                       `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                       `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                       `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                       `serial_no` varchar(50) NOT NULL COMMENT '设备序列号',
                                       `usage_type` varchar(45) NOT NULL COMMENT '用途类型，PROJECT：项目，OTHER：其它',
                                       `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                                       `num` int DEFAULT NULL COMMENT '数量',
                                       `mailing_address` varchar(500) DEFAULT NULL COMMENT '邮寄地址',
                                       `consignee` varchar(50) DEFAULT NULL COMMENT '收件人',
                                       `consignee_phone` varchar(30) DEFAULT NULL COMMENT '收件人联系电话',
                                       `express_no` varchar(50) DEFAULT NULL COMMENT '快递单号',
                                       `express_photo_path` varchar(500) DEFAULT NULL COMMENT '快递单拍照',
                                       `delivery_type` varchar(20) NOT NULL COMMENT '发货类型（出入库类型）',
                                       `status` varchar(30) DEFAULT NULL COMMENT '状态，UN_DELIVER：待发货，DELIVER：已发货，RECEIVE：已签收，CANCEL：作废',
                                       `deliver_time` bigint DEFAULT NULL COMMENT '发货时间',
                                       `deliver_user_id` bigint DEFAULT NULL COMMENT '发货人id',
                                       `receive_time` bigint DEFAULT NULL COMMENT '签收时间',
                                       `receive_user_id` bigint DEFAULT NULL COMMENT '签收人id',
                                       `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                       `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                       `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                       `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                       `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                       `effect` tinyint NOT NULL COMMENT '有效的可操作的发货记录（当一个设备处于一个发货出入库生命周期时为有效）, 0: 无效，1: 有效',
                                       `device_status` varchar(30) DEFAULT NULL COMMENT '设备状态，正常：NORMAL； 故障：FAULT',
                                       `source_id` bigint DEFAULT NULL COMMENT '设备来源ID，不为空，说明该设备由其他单号中转过来',
                                       `remark` varchar(200) DEFAULT NULL COMMENT '其他的运输方式说明',
                                       `mail_type` varchar(20) DEFAULT NULL COMMENT '邮寄方式：邮寄、其他',
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备发货表';

-- ----------------------------
-- Records of t_b_device_delivery
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_device_in_out
-- ----------------------------
DROP TABLE IF EXISTS `t_b_device_in_out`;
CREATE TABLE `t_b_device_in_out` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `serial_no` varchar(20) NOT NULL COMMENT '出入库流水号',
                                     `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                     `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                     `in_out_time` bigint NOT NULL COMMENT '出/入库时间',
                                     `user_id` bigint NOT NULL COMMENT '登记人id',
                                     `custom_name` varchar(100) DEFAULT NULL COMMENT '客户名称',
                                     `type` varchar(20) DEFAULT NULL COMMENT '类型，IN：入库，OUT：出库',
                                     `device_serial_no` varchar(50) DEFAULT NULL COMMENT '设备序列号',
                                     `supplier_name` varchar(100) DEFAULT NULL COMMENT '供应商名称',
                                     `location` varchar(100) DEFAULT NULL COMMENT '当前所在地',
                                     `address` varchar(100) DEFAULT NULL COMMENT '发往地',
                                     `address_arr` varchar(100) DEFAULT NULL COMMENT '发往地（回显）',
                                     `status` varchar(20) NOT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                                     `usage_type` varchar(45) DEFAULT NULL COMMENT '用途类型（项目、其他）',
                                     `base_photo_path` varchar(500) DEFAULT NULL COMMENT '快递单拍照',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建人时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_device_in_out_un_serial_no` (`serial_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备出入库登记';

-- ----------------------------
-- Records of t_b_device_in_out
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_ding
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding`;
CREATE TABLE `t_b_ding` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                            `ding_sop_no` varchar(100) NOT NULL COMMENT '实际打卡对应的sop单号',
                            `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                            `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                            `user_id` bigint NOT NULL COMMENT '用户id',
                            `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                            `role_type` varchar(20) DEFAULT NULL COMMENT '项目角色',
                            `sop_role_type` varchar(100) NOT NULL COMMENT '打卡时该用户在sop的身份集合',
                            `flow_id` bigint DEFAULT NULL COMMENT '流程id',
                            `task_id` bigint DEFAULT NULL COMMENT '流程任务id',
                            `sign_year` varchar(20) NOT NULL COMMENT '签到年份',
                            `sign_date` varchar(20) NOT NULL COMMENT '签到日期',
                            `date_type` varchar(20) DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                            `sign_in_time` bigint DEFAULT NULL COMMENT '签到时间',
                            `sign_in_address` varchar(100) DEFAULT NULL COMMENT '签到地址',
                            `axis_in_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                            `axis_in_y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                            `face_in_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `sign_out_time` bigint DEFAULT NULL COMMENT '签退时间',
                            `sign_out_address` varchar(100) DEFAULT NULL COMMENT '签退地址',
                            `axis_out_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                            `axis_out_Y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                            `face_out_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `ding_exception` tinyint NOT NULL DEFAULT '0' COMMENT '打卡异常,0:正常,1:异常',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            PRIMARY KEY (`id`),
                            UNIQUE KEY `t_b_ding_un` (`sop_no`,`user_id`,`sign_year`,`sign_date`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Records of t_b_ding
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_ding_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_apply`;
CREATE TABLE `t_b_ding_apply` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `sop_no` varchar(100) NOT NULL COMMENT 'sop编号',
                                  `crm_no` varchar(100) NOT NULL COMMENT '派单编号',
                                  `ding_id` bigint DEFAULT NULL COMMENT '考勤id',
                                  `type` varchar(20) NOT NULL COMMENT '类型，IN：签到，OUT：签退',
                                  `apply_time` bigint NOT NULL COMMENT '补卡时间',
                                  `reason` varchar(500) NOT NULL COMMENT '补卡原因',
                                  `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件id数组',
                                  `attachment_paths` varchar(800) DEFAULT NULL COMMENT '附件地址',
                                  `ding_exception_no` varchar(100) NOT NULL COMMENT '考勤异常编号',
                                  `approve` varchar(20) DEFAULT NULL COMMENT '审批操作，PASS：通过，NO_PASS：不通过',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_ding_apply_UN_ding_exception_no` (`ding_exception_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤补卡申请表';

-- ----------------------------
-- Records of t_b_ding_apply
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_ding_history
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_history`;
CREATE TABLE `t_b_ding_history` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `ding_id` bigint DEFAULT NULL COMMENT '考勤id',
                                    `sop_no` varchar(20) DEFAULT NULL COMMENT 'sop编号',
                                    `sign_date` varchar(20) NOT NULL COMMENT '签到日期',
                                    `date_type` varchar(20) DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                    `sign_time` bigint DEFAULT NULL COMMENT '签到时间',
                                    `axis_x` varchar(20) DEFAULT NULL COMMENT '坐标轴X',
                                    `axis_y` varchar(20) DEFAULT NULL COMMENT '坐标轴Y',
                                    `face_open` tinyint DEFAULT NULL COMMENT '是否开启人脸识别',
                                    `face_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                                    `face_photo_path` varchar(500) DEFAULT NULL COMMENT '抓拍照片地址',
                                    `match_score` decimal(8,2) DEFAULT NULL COMMENT '对比分数',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Records of t_b_ding_history
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_ding_statistic
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_statistic`;
CREATE TABLE `t_b_ding_statistic` (
                                      `id` bigint NOT NULL COMMENT '主键',
                                      `service_id` bigint NOT NULL COMMENT '服务单元id',
                                      `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                      `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                      `custom_name` varchar(100) DEFAULT NULL COMMENT '客户名称',
                                      `province` varchar(100) DEFAULT NULL COMMENT '省份',
                                      `city` varchar(100) DEFAULT NULL COMMENT '城市',
                                      `user_id` bigint NOT NULL COMMENT '打卡用户id',
                                      `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                                      `user_archives_name` varchar(100) DEFAULT NULL COMMENT '档案名',
                                      `user_archives_code` varchar(100) DEFAULT NULL COMMENT '档案编号',
                                      `scan_start_time` bigint DEFAULT NULL COMMENT '扫描开始时间',
                                      `scan_end_time` bigint DEFAULT NULL COMMENT '扫描结束时间',
                                      `mark_paper_start_time` bigint DEFAULT NULL COMMENT '阅卷开始时间',
                                      `mark_paper_end_time` bigint DEFAULT NULL COMMENT '阅卷开始时间',
                                      `role_name` varchar(100) NOT NULL COMMENT '打卡时该用户在sop的身份集合',
                                      `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                                      `supplier_name` varchar(100) DEFAULT NULL COMMENT '供应商名称',
                                      `actual_days` int DEFAULT NULL COMMENT '实际出勤（天）',
                                      `weekdays` int DEFAULT NULL COMMENT '工作日（天）',
                                      `weekends` int DEFAULT NULL COMMENT '周末（天）',
                                      `legal_holidays` int DEFAULT NULL COMMENT '法定节假日（天）',
                                      `work_hours` decimal(8,2) DEFAULT NULL COMMENT '累计工时（小时）',
                                      `violation_days` int DEFAULT NULL COMMENT '违规天数（天）',
                                      `ding_exception_count` bigint DEFAULT NULL COMMENT '考勤异常数',
                                      `remain_count` bigint DEFAULT NULL COMMENT '剩余补卡次数',
                                      `exception_count` bigint DEFAULT NULL COMMENT '待处理异常数',
                                      `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                      `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                      `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                      PRIMARY KEY (`id`),
                                      UNIQUE KEY `t_b_ding_statistic_un` (`sop_no`,`user_archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤提交统计表';

-- ----------------------------
-- Records of t_b_ding_statistic
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_ding_submit
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_submit`;
CREATE TABLE `t_b_ding_submit` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `service_id` bigint NOT NULL COMMENT '服务单元id',
                                   `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                   `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                   `user_id` bigint NOT NULL COMMENT '人员id',
                                   `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                                   `scan_start_time` bigint DEFAULT NULL COMMENT '扫描开始时间',
                                   `scan_end_time` bigint DEFAULT NULL COMMENT '扫描结束时间',
                                   `mark_paper_start_time` bigint DEFAULT NULL COMMENT '阅卷开始时间',
                                   `mark_paper_end_time` bigint DEFAULT NULL COMMENT '阅卷结束时间',
                                   `approach_time` bigint DEFAULT NULL COMMENT '进场时间',
                                   `departure_time` bigint DEFAULT NULL COMMENT '出场时间',
                                   `role_name` varchar(100) NOT NULL COMMENT '项目角色类型',
                                   `actual_days` int DEFAULT NULL COMMENT '实际出勤（天）',
                                   `weekdays` int DEFAULT NULL COMMENT '工作日（天）',
                                   `weekends` int DEFAULT NULL COMMENT '周末（天）',
                                   `legal_holidays` int DEFAULT NULL COMMENT '法定节假日（天）',
                                   `work_hours` decimal(8,2) DEFAULT NULL COMMENT '累计工时（小时）',
                                   `violation_days` int DEFAULT NULL COMMENT '违规天数（天）',
                                   `submit_user_id` bigint NOT NULL COMMENT '提交人id',
                                   `submit_time` bigint NOT NULL COMMENT '提交时间',
                                   `submit_status` varchar(20) NOT NULL COMMENT '提交状态',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                   `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`),
                                   UNIQUE KEY `t_b_ding_submit_un` (`sop_no`,`user_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤提交审核表';

-- ----------------------------
-- Records of t_b_ding_submit
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_product
-- ----------------------------
DROP TABLE IF EXISTS `t_b_product`;
CREATE TABLE `t_b_product` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `name` varchar(100) NOT NULL COMMENT '名称',
                               `code` varchar(100) NOT NULL COMMENT '编码，OFFICE：教务处，CLOUD_MARK：研究生',
                               `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_product_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='产品表';

-- ----------------------------
-- Records of t_b_product
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_project_exchange
-- ----------------------------
DROP TABLE IF EXISTS `t_b_project_exchange`;
CREATE TABLE `t_b_project_exchange` (
                                        `id` bigint NOT NULL COMMENT '主键',
                                        `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                        `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                        `exchange_no` varchar(100) DEFAULT NULL COMMENT '变更编号',
                                        `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                        `type` varchar(20) NOT NULL COMMENT '变更类型，PLAN：项目信息及计划，CANCEL：项目取消',
                                        `reason` varchar(500) NOT NULL COMMENT '变更原因',
                                        `content_json` mediumtext COMMENT '变更内容json',
                                        `result` varchar(20) DEFAULT NULL COMMENT '处理结果，FINISH：已完成，NOT_UPDATE：经沟通，取消变更，PARTIAL_UPDATE：经沟通，部分变更',
                                        `remark` varchar(500) DEFAULT NULL COMMENT '变更备注',
                                        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`),
                                        UNIQUE KEY `t_b_project_exchange_un_exchange_no` (`exchange_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='项目变更计划表';

-- ----------------------------
-- Records of t_b_project_exchange
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_quality_problem_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_quality_problem_apply`;
CREATE TABLE `t_b_quality_problem_apply` (
                                             `id` bigint NOT NULL COMMENT '主键',
                                             `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                             `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                             `problem_no` varchar(100) NOT NULL COMMENT '问题编号',
                                             `service_id` bigint NOT NULL COMMENT '服务单元id',
                                             `summary` varchar(1000) NOT NULL COMMENT '问题简要',
                                             `remark` varchar(500) NOT NULL COMMENT '问题情况说明',
                                             `user_ids` varchar(500) DEFAULT NULL COMMENT '责任人id数组',
                                             `type` varchar(20) DEFAULT NULL COMMENT '问题类型，UPDATE：修正类，OPTIMIZE：优化类，NO_PROBLEM：不是问题',
                                             `reason` varchar(20) DEFAULT NULL COMMENT '问题原因类型，EXEC：执行类，MANAGER：管理协调类，FLOW：流程制度类，PRODUCT_PROBLEM：产品缺陷类，PRODUCT_MANAGER：产品运维类，OTHER：其它',
                                             `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件id数组',
                                             `Influence_degree` varchar(20) DEFAULT NULL COMMENT '影响度：A，B，C，D',
                                             `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                             `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                             `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                             PRIMARY KEY (`id`),
                                             UNIQUE KEY `t_b_quality_problem_apply_UN_probem_no` (`problem_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='质量问题反馈表';

-- ----------------------------
-- Records of t_b_quality_problem_apply
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_service
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service`;
CREATE TABLE `t_b_service` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `name` varchar(100) NOT NULL COMMENT '名称',
                               `type` varchar(20) NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                               `start_time` bigint NOT NULL COMMENT '开始时间',
                               `end_time` bigint NOT NULL COMMENT '结束时间',
                               `region_people_count` int NOT NULL COMMENT '区域协调人数量',
                               `region_project_count` int NOT NULL COMMENT '区域项目数量',
                               `service_lead_id` bigint NOT NULL COMMENT '服务单元负责人id',
                               `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `status` varchar(20) NOT NULL COMMENT '状态，NEW：新建，PUBLISH：已发布，FINISH：已完结，CANCEL：已作废',
                               `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                               `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_service_un_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元表';

-- ----------------------------
-- Records of t_b_service
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_service_region
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region`;
CREATE TABLE `t_b_service_region` (
                                      `id` bigint NOT NULL COMMENT '主键',
                                      `service_id` bigint NOT NULL COMMENT '服务单元id',
                                      `region_name` varchar(100) NOT NULL COMMENT '大区名称',
                                      `lead_id` bigint NOT NULL COMMENT '大区经理id',
                                      `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                      `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                      `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                      PRIMARY KEY (`id`),
                                      UNIQUE KEY `t_b_service_region_un` (`service_id`,`region_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划表';

-- ----------------------------
-- Records of t_b_service_region
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_service_region_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region_detail`;
CREATE TABLE `t_b_service_region_detail` (
                                             `id` bigint NOT NULL COMMENT '主键',
                                             `service_region_id` bigint NOT NULL COMMENT '服务区域规划id',
                                             `country` varchar(100) NOT NULL DEFAULT '中国' COMMENT '国家',
                                             `province` varchar(100) NOT NULL COMMENT '省份',
                                             `city` varchar(100) NOT NULL COMMENT '城市',
                                             `area` varchar(100) DEFAULT NULL COMMENT '县区',
                                             `address` varchar(150) DEFAULT NULL COMMENT '地址',
                                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划明细表';

-- ----------------------------
-- Records of t_b_service_region_detail
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_service_scope
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_scope`;
CREATE TABLE `t_b_service_scope` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `service_id` bigint NOT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_service_scope_un` (`service_id`,`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元范围管理表';

-- ----------------------------
-- Records of t_b_service_scope
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
                               `id` varchar(100) NOT NULL COMMENT '主键',
                               `org_id` bigint DEFAULT NULL COMMENT '机构id',
                               `identity` varchar(100) NOT NULL COMMENT '用户标识',
                               `type` varchar(500) NOT NULL COMMENT '用户类型',
                               `source` varchar(50) NOT NULL COMMENT '访问来源',
                               `platform` varchar(50) NOT NULL COMMENT '设备分类',
                               `device_id` varchar(100) NOT NULL COMMENT '设备标识',
                               `address` varchar(100) NOT NULL COMMENT '登录IP地址',
                               `access_token` varchar(50) NOT NULL COMMENT '访问令牌',
                               `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
                               `last_access_ip` varchar(100) DEFAULT NULL COMMENT '最近访问IP地址',
                               `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
                               `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
                               `app_source` varchar(50) DEFAULT NULL COMMENT '应用来源',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Records of t_b_session
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_sop_info
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_info`;
CREATE TABLE `t_b_sop_info` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                `sop_no` varchar(100) NOT NULL COMMENT 'sop单号',
                                `service_id` bigint NOT NULL COMMENT '服务单元id',
                                `custom_id` bigint NOT NULL COMMENT '客户id',
                                `product_id` bigint NOT NULL COMMENT '产品id',
                                `type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP',
                                `status` varchar(20) NOT NULL COMMENT '状态，DRAFT：草稿，START：提交',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `t_b_sop_info_un_sop_no` (`sop_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop表';

-- ----------------------------
-- Records of t_b_sop_info
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_sop_log
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_log`;
CREATE TABLE `t_b_sop_log` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `sop_id` bigint NOT NULL COMMENT 'sop主键',
                               `type` varchar(30) DEFAULT NULL COMMENT '类型，APPROVE：sop审核，EDIT：sop编辑，CHANGE_EFFECT_ENGINEER：更改实施工程师，CHANGE_ASSISTANT_ENGINEER：更改助理工程师，ADD_EFFECT_ENGINEER：新增实施工程师，ADD_ASSISTANT_ENGINEER：新增助理工程师，REMOVE_EFFECT_ENGINEER：删除实施工程师，REMOVE_ASSISTANT_ENGINEER：删除助理工程师',
                               `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                               `setup` tinyint NOT NULL COMMENT '计划步骤',
                               `update_before_obj` mediumtext COMMENT '更改前内容',
                               `update_after_obj` mediumtext COMMENT '更改后内容',
                               `create_id` bigint NOT NULL COMMENT '创建人id',
                               `create_time` bigint NOT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop日志';

-- ----------------------------
-- Records of t_b_sop_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_sop_plan_date_log
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_plan_date_log`;
CREATE TABLE `t_b_sop_plan_date_log` (
                                         `id` bigint NOT NULL COMMENT '主键',
                                         `sop_id` bigint DEFAULT NULL COMMENT 'sopid',
                                         `flow_id` bigint NOT NULL COMMENT '流程id',
                                         `begin_time` bigint NOT NULL COMMENT '计划开始时间',
                                         `end_time` bigint NOT NULL COMMENT '计划结束时间',
                                         `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                         `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                         `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop进程计划时间日志表';

-- ----------------------------
-- Records of t_b_sop_plan_date_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_sop_quick_search
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_quick_search`;
CREATE TABLE `t_b_sop_quick_search` (
                                        `id` bigint NOT NULL COMMENT '主键',
                                        `type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                        `flow_deployment_id` varchar(50) NOT NULL COMMENT '流程部署id',
                                        `version` int NOT NULL COMMENT '版本号',
                                        `user_id` bigint NOT NULL COMMENT '用户id',
                                        `name` varchar(100) NOT NULL COMMENT '内容名称',
                                        `content` mediumtext NOT NULL COMMENT '内容',
                                        `content_type` varchar(20) NOT NULL COMMENT '内容类型，VIEW：显示列表，SEARCH：快捷搜索',
                                        PRIMARY KEY (`id`),
                                        UNIQUE KEY `t_b_sop_quick_search_unique` (`type`,`name`,`content_type`,`user_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop快捷搜索表';

-- ----------------------------
-- Records of t_b_sop_quick_search
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `type` varchar(50) NOT NULL COMMENT '任务类型，USER_ARCHIVES_IMPORT：人员档案导入，USER_ARCHIVES_EXPORT：人员档案导出，CUSTOM_IMPORT：客户导入',
                            `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
                            `status` varchar(30) NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                            `summary` text COMMENT '实时摘要信息',
                            `progress` double DEFAULT NULL COMMENT '执行进度',
                            `result` varchar(30) DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `import_file_name` varchar(100) DEFAULT NULL COMMENT '导入文件名',
                            `import_file_path` varchar(500) DEFAULT NULL COMMENT '导入文件路径',
                            `result_file_path` varchar(500) DEFAULT NULL COMMENT '导出文件路径',
                            `report_file_path` varchar(500) DEFAULT NULL COMMENT '报告路径',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `remark` mediumtext COMMENT '备注',
                            `obj_name` varchar(500) DEFAULT NULL COMMENT '实体名称',
                            `reset_count` int DEFAULT '0' COMMENT '重试次数',
                            `version` int DEFAULT '0' COMMENT '更新版本号',
                            `error_message` varchar(500) DEFAULT NULL COMMENT '人工错误原因',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Records of t_b_task
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_user_archives
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives`;
CREATE TABLE `t_b_user_archives` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `code` varchar(50) NOT NULL COMMENT '编号',
                                     `name` varchar(100) NOT NULL COMMENT '姓名',
                                     `country` varchar(100) NOT NULL DEFAULT '中国' COMMENT '国家',
                                     `province` varchar(100) NOT NULL COMMENT '省份',
                                     `city` varchar(100) NOT NULL COMMENT '城市',
                                     `area` varchar(100) DEFAULT NULL COMMENT '县区',
                                     `address` varchar(150) DEFAULT NULL COMMENT '地址',
                                     `identity` varchar(50) NOT NULL COMMENT '身份证号',
                                     `gender` varchar(20) NOT NULL COMMENT '性别，MAN：男，WOMAN：女',
                                     `education` varchar(20) DEFAULT NULL COMMENT '学历',
                                     `mobile_number` varchar(30) NOT NULL COMMENT '手机号',
                                     `email` varchar(30) NOT NULL COMMENT '邮箱',
                                     `base_photo_path` varchar(500) DEFAULT NULL COMMENT '底照保存地址',
                                     `status` varchar(20) NOT NULL COMMENT '人力资源状态（FREE("空闲"),NON_EXECUTION_SOP("未执行SOP"),EXECUTION_SOP("执行SOP");）',
                                     `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '是否启用（1启用，0作废）',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_user_archives_un_identity` (`identity`),
                                     UNIQUE KEY `t_b_user_archives_un_mobile_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案表';

-- ----------------------------
-- Records of t_b_user_archives
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_user_archives_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_allocation`;
CREATE TABLE `t_b_user_archives_allocation` (
                                                `id` bigint NOT NULL COMMENT '主键',
                                                `crm_detail_id` bigint NOT NULL COMMENT '派单详情表id',
                                                `service_id` bigint NOT NULL COMMENT '服务单元id',
                                                `crm_no` varchar(100) NOT NULL COMMENT 'crm单号',
                                                `sop_role_type` varchar(45) NOT NULL COMMENT 'sop角色类型\nREGION_COORDINATOR("区域协调人"),PROJECT_MANAGER("项目经理"),ENGINEER("工程师");',
                                                `user_id` bigint NOT NULL COMMENT '用户id',
                                                `archives_id` bigint NOT NULL COMMENT '人员档案id',
                                                `remark` varchar(45) DEFAULT NULL COMMENT '备注',
                                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                                PRIMARY KEY (`id`),
                                                UNIQUE KEY `t_b_user_archives_allocation_un` (`crm_detail_id`,`sop_role_type`,`archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员分配表';

-- ----------------------------
-- Records of t_b_user_archives_allocation
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_user_archives_allocation_log
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_allocation_log`;
CREATE TABLE `t_b_user_archives_allocation_log` (
                                                    `id` bigint NOT NULL COMMENT '主键',
                                                    `crm_detail_id` bigint NOT NULL COMMENT '派单详情表id',
                                                    `sop_role_type` varchar(45) NOT NULL COMMENT 'sop角色类型\nREGION_COORDINATOR("区域协调人"),PROJECT_MANAGER("项目经理"),ENGINEER("工程师");',
                                                    `user_id` bigint NOT NULL COMMENT '用户id',
                                                    `archives_id` bigint NOT NULL COMMENT '人员档案id',
                                                    `archives_name` varchar(100) DEFAULT NULL COMMENT '档案名称',
                                                    `supplier_name` varchar(100) DEFAULT NULL COMMENT '供应商名称',
                                                    `city` varchar(100) DEFAULT NULL COMMENT '城市',
                                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                                    `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员调配历史记录';

-- ----------------------------
-- Records of t_b_user_archives_allocation_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_user_archives_supplier
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_supplier`;
CREATE TABLE `t_b_user_archives_supplier` (
                                              `id` bigint NOT NULL COMMENT '主键',
                                              `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                                              `archives_time` bigint NOT NULL COMMENT '入档时间',
                                              `authentication_time` bigint DEFAULT NULL COMMENT '认证时间',
                                              `authentication_score` double DEFAULT NULL COMMENT '认证成绩',
                                              `authentication_valid_time` bigint DEFAULT NULL COMMENT '认证有效时间',
                                              `remark` varchar(500) DEFAULT NULL COMMENT '备注',
                                              PRIMARY KEY (`id`),
                                              UNIQUE KEY `t_b_user_archives_supplier_un` (`user_archives_id`,`supplier_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案与供应商表';

-- ----------------------------
-- Records of t_b_user_archives_supplier
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_violation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation`;
CREATE TABLE `t_b_violation` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                 `sop_no` varchar(100) DEFAULT NULL COMMENT 'sop单号',
                                 `code` varchar(200) NOT NULL COMMENT '编号',
                                 `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                 `user_id` bigint NOT NULL COMMENT '负责人id',
                                 `type` varchar(20) NOT NULL COMMENT '类型，CONTENT_ERROR：内容错误虚假，LOGIC_ERROR：逻辑不合理，OTHER：其它',
                                 `content` varchar(500) NOT NULL COMMENT '违规情况',
                                 `status` varchar(20) NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                 `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                 `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                 `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                 `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                 `close_time` bigint DEFAULT NULL COMMENT '关闭时间',
                                 PRIMARY KEY (`id`),
                                 UNIQUE KEY `t_b_violation_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop违规登记表';

-- ----------------------------
-- Records of t_b_violation
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_b_violation_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation_detail`;
CREATE TABLE `t_b_violation_detail` (
                                        `id` bigint NOT NULL COMMENT '主键',
                                        `violation_id` bigint NOT NULL COMMENT '违规id',
                                        `remark` varchar(500) NOT NULL COMMENT '跟进说明',
                                        `attachment_ids` varchar(500) DEFAULT NULL COMMENT '附件ids',
                                        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='违规明细表';

-- ----------------------------
-- Records of t_b_violation_detail
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_d_form_widget
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget`;
CREATE TABLE `t_d_form_widget` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `code` varchar(20) DEFAULT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                   `type` varchar(20) DEFAULT NULL COMMENT '类型，FORM：表单，TABLE：表格，OTHER：其它',
                                   `form_id` varchar(100) DEFAULT NULL COMMENT '表单id',
                                   `form_name` varchar(100) DEFAULT NULL COMMENT '表单名称',
                                   `title` varchar(100) DEFAULT NULL COMMENT '标题',
                                   `input_type` varchar(20) DEFAULT NULL COMMENT '输入类型，STRING：字符串，INT：整型，LONG：长整型，DOUBLE：浮点型，ENUM：枚举，DATE：日期，BOOL：布尔，BIGDECIMAL：精确浮点，FILE：文件，ARRAY：数组',
                                   `required` tinyint DEFAULT '1' COMMENT '是否必填，0：非必填，1：必填',
                                   `readable` tinyint DEFAULT '0' COMMENT '是否只读，0：非只读，1：只读',
                                   `writable` tinyint DEFAULT '1' COMMENT '是否可写，0：非可写，1：可写',
                                   `visable` tinyint DEFAULT '1' COMMENT '是否可见，0：不可见，1：可见',
                                   `scale` int DEFAULT NULL COMMENT '保留小数点几位',
                                   `length` int DEFAULT NULL COMMENT '长度',
                                   `binding` varchar(30) DEFAULT NULL COMMENT '绑定字段',
                                   `data_grid` varchar(100) DEFAULT NULL COMMENT '数据来源',
                                   `tips` varchar(100) DEFAULT NULL COMMENT '提示',
                                   `format` varchar(20) DEFAULT NULL COMMENT '格式',
                                   `span` int DEFAULT '6' COMMENT '占宽符',
                                   `sub_title` varchar(1000) DEFAULT NULL COMMENT '子标题',
                                   `options` varchar(500) DEFAULT NULL COMMENT '选项值，RADIO或CHECKBOX填此值',
                                   `flow_type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `form_setup` int NOT NULL COMMENT '表单步骤',
                                   `handle` tinyint NOT NULL DEFAULT '0' COMMENT '处理时限，0：不处理，1：可处理',
                                   `form_group` int DEFAULT NULL COMMENT '控件组',
                                   `role_type` varchar(500) DEFAULT NULL COMMENT '角色类型',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单控件表';

-- ----------------------------
-- Records of t_d_form_widget
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_d_form_widget_metadata
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget_metadata`;
CREATE TABLE `t_d_form_widget_metadata` (
                                            `id` bigint NOT NULL COMMENT '主键',
                                            `type` varchar(20) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                            `flow_deployment_id` varchar(50) NOT NULL COMMENT '流程部署id',
                                            `version` int NOT NULL DEFAULT '1' COMMENT '版本号',
                                            `code` varchar(20) NOT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                            `field_id` varchar(100) NOT NULL COMMENT '字段id',
                                            `field_name` varchar(100) NOT NULL COMMENT '字段名称',
                                            `field_title` varchar(50) DEFAULT NULL COMMENT '字段标题',
                                            `field_check` tinyint NOT NULL DEFAULT '0' COMMENT '是否勾选，0：未勾选，1：勾选',
                                            `setup` int NOT NULL COMMENT '流程步骤',
                                            `change_join` tinyint DEFAULT NULL COMMENT '是否是项目计划变更关联字段，0：不是，1：是',
                                            `create_time` bigint NOT NULL COMMENT '创建时间',
                                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单元数据表';

-- ----------------------------
-- Records of t_d_form_widget_metadata
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_d_table_prop
-- ----------------------------
DROP TABLE IF EXISTS `t_d_table_prop`;
CREATE TABLE `t_d_table_prop` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `widget_id` bigint DEFAULT NULL COMMENT '控件id',
                                  `td_index` int DEFAULT NULL COMMENT '序号',
                                  `td_id` varchar(50) DEFAULT NULL COMMENT '列id',
                                  `td_name` varchar(50) DEFAULT NULL COMMENT '列名称',
                                  `title` varchar(100) DEFAULT NULL COMMENT '标题',
                                  `width` double DEFAULT NULL COMMENT '宽度',
                                  `height` double DEFAULT NULL COMMENT '高度',
                                  `color` varchar(20) DEFAULT NULL COMMENT '颜色',
                                  `rowspan` int DEFAULT NULL COMMENT '跨行',
                                  `colspan` int DEFAULT NULL COMMENT '跨列',
                                  `sub_title` varchar(1000) DEFAULT NULL COMMENT '子标题',
                                  `tips` varchar(100) DEFAULT NULL COMMENT '提示',
                                  `td_order` tinyint DEFAULT '1' COMMENT '是否排序，0：不排序，1：排序',
                                  `td_search` tinyint DEFAULT '1' COMMENT '是否搜索，0：不搜索，1：搜索',
                                  `edit_widget_id` bigint DEFAULT NULL COMMENT '编辑控件id',
                                  `obj` mediumtext COMMENT '实体内容',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='表格属性表';

-- ----------------------------
-- Records of t_d_table_prop
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow`;
CREATE TABLE `t_f_custom_flow` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                   `name` varchar(100) NOT NULL COMMENT '流程名称',
                                   `model_type` varchar(50) NOT NULL COMMENT '流程模式类型，USER_FIXED：指定人员，APPROVE_SET：发起人指定',
                                   `type` varchar(100) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                   `publish` tinyint NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
                                   `object_data` mediumtext COMMENT '自定义流程源数据',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                   `version` int NOT NULL DEFAULT '1' COMMENT '版本号',
                                   `flow_definition_id` varchar(50) DEFAULT NULL COMMENT '流程定义id',
                                   `flow_deployment_id` varchar(50) DEFAULT NULL COMMENT '流程部署id',
                                   `flow_process_var` mediumtext COMMENT '流程过程变量数据',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程表';

-- ----------------------------
-- Records of t_f_custom_flow
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_f_custom_flow_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_allocation`;
CREATE TABLE `t_f_custom_flow_allocation` (
                                              `id` bigint NOT NULL COMMENT '主键',
                                              `type` varchar(100) NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                              `t_f_custom_flow_id` bigint DEFAULT NULL COMMENT '自定义流程id',
                                              `t_f_custom_flow_entity_id` bigint DEFAULT NULL COMMENT '流程实体id',
                                              `role_allocation` mediumtext NOT NULL COMMENT '角色分配',
                                              `version` int DEFAULT NULL COMMENT '版本号',
                                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程角色分配表';

-- ----------------------------
-- Records of t_f_custom_flow_allocation
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_entity`;
CREATE TABLE `t_f_custom_flow_entity` (
                                          `id` bigint NOT NULL COMMENT '主键',
                                          `t_f_custom_flow_id` bigint NOT NULL COMMENT '自定义流程id',
                                          `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                          `code` varchar(30) NOT NULL COMMENT '编号',
                                          `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                                          `flow_id` bigint DEFAULT NULL COMMENT '流程id',
                                          `flow_process_var` mediumtext NOT NULL COMMENT '流程过程变量',
                                          `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                          `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                          `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                          `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                          PRIMARY KEY (`id`) USING BTREE,
                                          UNIQUE KEY `t_f_custom_flow_entity_UN_code` (`code`,`flow_id`),
                                          KEY `flow_entity_flowId_idx` (`flow_id`),
                                          KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程实体数据';

-- ----------------------------
-- Records of t_f_custom_flow_entity
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_approve`;
CREATE TABLE `t_f_flow_approve` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                    `flow_id` bigint NOT NULL COMMENT '流程id',
                                    `approve_id` bigint DEFAULT NULL COMMENT '审核id',
                                    `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    `setup` tinyint NOT NULL COMMENT '计划步骤',
                                    `approve_setup` int DEFAULT NULL COMMENT '审批步骤',
                                    `approve_max_setup` int DEFAULT NULL COMMENT '最大审批步骤',
                                    `transfer_assignee` mediumtext COMMENT '流程转办',
                                    `all_assignee` mediumtext COMMENT '会签审批人',
                                    `reject_link` mediumtext COMMENT '驳回路线',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程审核表';

-- ----------------------------
-- Records of t_f_flow_approve
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_log`;
CREATE TABLE `t_f_flow_log` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                `flow_id` bigint NOT NULL COMMENT '流程id',
                                `object_table` varchar(100) DEFAULT NULL COMMENT '实体表',
                                `object_id` bigint NOT NULL COMMENT '实体id',
                                `approve_id` bigint DEFAULT NULL COMMENT '审核人id',
                                `approve_setup` tinyint DEFAULT NULL COMMENT '审核步骤',
                                `approve_remark` mediumtext COMMENT '审核意见',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                `approve_operation` varchar(50) NOT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                `pend_approve_id` mediumtext COMMENT '待审核人id',
                                `attachment_ids` mediumtext COMMENT '附件id集合',
                                `approve_multi_setup` int DEFAULT NULL COMMENT '审批步骤',
                                `approve_user_approve_type` varchar(30) DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
                                `task_id` bigint DEFAULT NULL COMMENT '流程节点id',
                                PRIMARY KEY (`id`) USING BTREE,
                                KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程流水表';

-- ----------------------------
-- Records of t_f_flow_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
                             `id` bigint NOT NULL COMMENT '主键',
                             `org_id` bigint DEFAULT NULL COMMENT '机构id',
                             `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                             `summary` mediumtext COMMENT '实时摘要信息',
                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Records of t_g_error
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `school_id` bigint DEFAULT NULL COMMENT '学校id',
                            `access_key` varchar(255) DEFAULT NULL COMMENT 'accessKey',
                            `access_secret` varchar(255) DEFAULT NULL COMMENT 'accessSecret',
                            `description` text COMMENT '离线授权证书',
                            `type` varchar(16) NOT NULL COMMENT '授权类型',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
                            `file` mediumblob COMMENT '文件数据',
                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';

-- ----------------------------
-- Records of t_s_auth
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_s_job_log
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_log`;
CREATE TABLE `t_s_job_log` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `job_remind_id` bigint NOT NULL COMMENT '任务提醒id',
                               `type` varchar(20) NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                               `code` varchar(100) DEFAULT NULL COMMENT '编码',
                               `obj_id` bigint NOT NULL COMMENT '实体id',
                               `obj_name` varchar(100) DEFAULT NULL COMMENT '实体名称',
                               `remind_type` varchar(20) NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                               `create_id` bigint NOT NULL COMMENT '创建人id',
                               `create_time` bigint NOT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务日志表';

-- ----------------------------
-- Records of t_s_job_log
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_s_job_remind
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_remind`;
CREATE TABLE `t_s_job_remind` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `type` varchar(20) NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                                  `crm_no` varchar(100) DEFAULT NULL COMMENT 'crm单号',
                                  `code` varchar(100) DEFAULT NULL COMMENT '编码',
                                  `obj_id` bigint NOT NULL COMMENT '实体id',
                                  `obj_name` varchar(100) DEFAULT NULL COMMENT '实体名称',
                                  `remind_type` varchar(20) NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                                  `exec_time` bigint DEFAULT NULL COMMENT '执行时间',
                                  `exec_field` varchar(50) DEFAULT NULL COMMENT '执行字段',
                                  `receive_user_id` bigint DEFAULT NULL COMMENT '接收人id',
                                  `approve_operation` varchar(20) DEFAULT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint NOT NULL COMMENT '创建人id',
                                  `create_time` bigint NOT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务提醒表';

-- ----------------------------
-- Records of t_s_job_remind
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Table structure for t_wxapp_info
-- ----------------------------
DROP TABLE IF EXISTS `t_wxapp_info`;
CREATE TABLE `t_wxapp_info` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `open_id` varchar(100) NOT NULL COMMENT 'openId',
                                `phone` varchar(30) NOT NULL COMMENT '手机号',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='微信信息表';

-- ----------------------------
-- Records of t_wxapp_info
-- ----------------------------
BEGIN;
COMMIT;

-- ----------------------------
-- Function structure for currval
-- ----------------------------
DROP FUNCTION IF EXISTS `currval`;
delimiter ;;
CREATE FUNCTION `currval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
  DECLARE value LONG;
  SET value = 0;
SELECT current_value INTO value
FROM sequence
WHERE name = seq_name;
RETURN value;
END;
;;
delimiter ;

-- ----------------------------
-- Function structure for nextval
-- ----------------------------
DROP FUNCTION IF EXISTS `nextval`;
delimiter ;;
CREATE FUNCTION `nextval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
UPDATE sequence
SET current_value = current_value + increment
WHERE name = seq_name;
RETURN currval(seq_name);
END;
;;
delimiter ;

INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('cloudMarkSopFlow', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('delayWarn', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('deviceInOutCode', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('dingExceptionFlow', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('officeSopFlow', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('projectExchangeFlow', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('qualityProblemFlow', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('sysCustomCode', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('userArchivesCode', 0, 1);
INSERT INTO `sequence` (`name`, `current_value`, `increment`) VALUES ('violationWarn', 0, 1);

INSERT INTO `t_f_custom_flow_allocation` (`id`, `type`, `t_f_custom_flow_id`, `t_f_custom_flow_entity_id`, `role_allocation`, `version`) VALUES (1, 'PROJECT_EXCHANGE_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);
INSERT INTO `t_f_custom_flow_allocation` (`id`, `type`, `t_f_custom_flow_id`, `t_f_custom_flow_entity_id`, `role_allocation`, `version`) VALUES (2, 'QUALITY_PROBLEM_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);

INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (1, 2023, 9, 29, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (2, 2023, 9, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (3, 2023, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (4, 2023, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (5, 2023, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (6, 2023, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (7, 2023, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (8, 2023, 10, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (9, 2023, 10, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (10, 2023, 10, 8, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (11, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (12, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (13, 2024, 1, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (14, 2024, 2, 4, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (15, 2024, 2, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (16, 2024, 2, 11, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (17, 2024, 2, 12, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (18, 2024, 2, 13, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (19, 2024, 2, 14, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (20, 2024, 2, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (21, 2024, 2, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (22, 2024, 2, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (23, 2024, 2, 18, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (24, 2024, 4, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (25, 2024, 4, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (26, 2024, 4, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (27, 2024, 4, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (28, 2024, 5, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (29, 2024, 5, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (30, 2024, 5, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (31, 2024, 5, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (32, 2024, 5, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (33, 2024, 5, 11, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (34, 2024, 6, 8, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (35, 2024, 6, 9, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (36, 2024, 6, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (37, 2024, 9, 14, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (38, 2024, 9, 29, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (39, 2024, 9, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (40, 2024, 9, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (41, 2024, 9, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (42, 2024, 10, 12, 'WEEKDAY');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (43, 2024, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (44, 2024, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (45, 2024, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (46, 2024, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (47, 2024, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (48, 2024, 10, 7, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` (`id`, `year`, `month`, `day`, `type`) VALUES (49, 2024, 4, 28, 'WEEKDAY');

INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1, '系统管理', 'systemManage', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2, '配置管理', 'configureManage', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3, '客户配置', 'custom', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4, '供应商配置', 'supplier', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (5, '设备配置', 'device', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (6, '服务档位配置', 'level', 'MENU', 2, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (7, '考勤配置', 'dingConfigure', 'MENU', 2, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (8, '系统日志', 'logManage', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (9, '通知公告管理', 'noticeManage', 'MENU', 8, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (10, '日志查询', 'log', 'MENU', 8, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (11, '用户管理', 'userManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (12, '组织架构管理', 'orgManage', 'MENU', 11, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (13, '组织架构管理', 'org', 'MENU', 12, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (14, '用户权限管理', 'userPrivilegeManage', 'MENU', 11, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (15, '用户管理', 'user', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (16, '角色管理', 'role', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (17, '初始化密码', 'InitPwd', 'LINK', 15, 3, 'AUTH', '18', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (18, '用户初始化密码', '/api/admin/user/init_password', 'URL', 15, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (19, '项目质量管理', 'projectQualityManage', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (20, '项目质量管理', 'projectChildQualityManage', 'MENU', 19, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (21, '质量问题反馈', 'qualityProblem', 'MENU', 20, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (23, '质量监控分析', 'qualityProblemControl', 'MENU', 20, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (24, '工时管理', 'hoursManage', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (25, '工时管理', 'hoursChildManage', 'MENU', 24, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (26, '考勤打卡', 'ding', 'MENU', 25, 1, NULL, '3017,2044,2045,2046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (27, '异常审核', 'exception', 'MENU', 25, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (28, '考勤提交', 'dingSubmit', 'MENU', 25, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (29, '工时统计', 'hourSubmit', 'MENU', 25, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (30, '考勤监控预警', 'dingWarn', 'MENU', 25, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (31, '资源保障', 'resourceManage', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (32, '人资保障', 'humanManage', 'MENU', 31, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (33, '人员档案管理', 'userArchives', 'MENU', 32, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (34, '人员调配', 'userAllocate', 'MENU', 32, 2, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (35, '人资监控', 'userArchivesControl', 'MENU', 32, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (36, '设备保障', 'deviceManage', 'MENU', 31, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (37, '出入库登记查询', 'deviceInOut', 'MENU', 36, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (38, '设备资源监控', 'deviceControl', 'MENU', 36, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (39, 'SOP管理', 'sopManage', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (40, 'SOP管理', 'sopChildManage', 'MENU', 39, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (41, '教务处SOP管理', 'office', 'MENU', 40, 1, NULL, '3034,3035,3091,3092,3093,2062', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (42, '研究生SOP管理', 'cloudMark', 'MENU', 40, 2, NULL, '3034,3035,3091,3092,3093,2062', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (43, '设备出入库登记', 'deviceInOutSop', 'MENU', 40, 3, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (44, '项目计划变更报备', 'projectExchange', 'MENU', 40, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (45, 'SOP监控', 'sopControlManage', 'MENU', 39, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (46, 'SOP监控台', 'sopControl', 'MENU', 45, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (47, '延期预警', 'delayWarn', 'MENU', 45, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (48, '违规登记', 'violation', 'MENU', 45, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (49, '服务单元管理', 'serviceManage', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (50, '派单管理', 'crmManage', 'MENU', 49, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (51, '派单管理', 'crm', 'MENU', 50, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (52, '统计查询', 'crmCount', 'MENU', 50, 2, 'AUTH', '1034', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (53, '服务单元管理', 'serviceChildManage', 'MENU', 49, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (54, '服务单元管理', 'service', 'MENU', 53, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (55, '服务范围管理', 'serviceScope', 'MENU', 53, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (56, '服务区域规划', 'serviceRegion', 'MENU', 53, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (57, '统计查询', 'serviceCount', 'MENU', 53, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (58, '我的工作台', 'workManage', 'MENU', NULL, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (59, '工作台', 'workChildManage', 'MENU', 58, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (60, '消息提醒', 'message', 'MENU', 59, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (61, '待办任务', 'work', 'MENU', 59, 2, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (62, '通知公告', 'notice', 'MENU', 59, 3, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (63, '公共接口', 'common', 'MENU', NULL, 9, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (64, '权限管理', 'menuManage', 'MENU', 63, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (65, '用户登录', '/api/admin/common/login', 'URL', 64, 1, 'NO_AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (66, '用户登出', '/api/admin/common/logout', 'URL', 64, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (67, '文件上传', '/api/admin/common/file/upload', 'URL', 64, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (68, '文件下载', '/api/admin/common/file/download', 'URL', 64, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (69, '文件预览', '/api/admin/common/file/preview', 'URL', 64, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (72, '任务管理查询', '/api/admin/task/query', 'URL', 240, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (73, '下载导入模板', '/api/admin/common/download_import_template', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (74, '列表', 'List', 'LIST', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (75, '查询', 'Select', 'BUTTON', 3, 3, 'AUTH', '262', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (76, '查询条件', 'Condition', 'CONDITION', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (77, '修改', 'Update', 'LINK', 3, 1, 'AUTH', '260,263', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (78, '删除', 'Delete', 'LINK', 3, 2, 'AUTH', '259,260', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (79, '新增', 'Add', 'BUTTON', 3, 1, 'AUTH', '258', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (80, '批量导入', 'BatchImport', 'BUTTON', 3, 2, 'AUTH', '261', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (81, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '264,266', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (82, '列表', 'List', 'LIST', 4, 1, 'AUTH', '268', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (83, '修改', 'Update', 'LINK', 4, 1, 'AUTH', '269,266', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (84, '启用/禁用', 'Enable', 'LINK', 4, 2, 'AUTH', '265', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (85, '列表', 'List', 'LIST', 5, 1, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (86, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (87, '新增', 'Add', 'BUTTON', 5, 1, 'AUTH', '270', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (88, '作废', 'Cancel', 'BUTTON', 5, 2, 'AUTH', '271', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (89, '列表', 'List', 'LIST', 6, 1, 'AUTH', '256', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (90, '新增', 'Add', 'BUTTON', 6, 1, 'AUTH', '254,300', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (91, '修改', 'Update', 'LINK', 6, 1, 'AUTH', '254,257', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (92, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '253', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (93, '列表', 'List', 'LIST', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (94, '新增', 'Add', 'BUTTON', 7, 1, 'AUTH', '277', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (95, '修改', 'Update', 'LINK', 7, 1, 'AUTH', '282', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (96, '删除', 'Delete', 'LINK', 7, 2, 'AUTH', '278', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (97, '查询条件', 'Condition', 'CONDITION', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (98, '列表', 'List', 'LIST', 9, 1, 'AUTH', '1004', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (99, '新增', 'Add', 'BUTTON', 9, 1, 'AUTH', '1005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (100, '撤销', 'Cancel', 'LINK', 9, 2, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (101, '回执查询', 'Select', 'LINK', 9, 2, 'AUTH', '1006,1007,1008', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (102, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (103, '修改', 'Update', 'LINK', 9, 3, 'AUTH', '1002,1005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (104, '发布', 'Publish', 'LINK', 9, 1, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (105, '查询', 'Select', 'BUTTON', 5, 3, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (106, '查询', 'Select', 'BUTTON', 7, 2, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (107, '查询', 'Select', 'BUTTON', 9, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (108, '列表', 'List', 'LIST', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (109, '查询条件', 'Condition', 'CONDITION', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (110, '查询', 'Select', 'BUTTON', 10, 2, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (111, '列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (112, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (113, '查询条件', 'Condition', 'CONDITION', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (114, '修改', 'Update', 'LINK', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (115, '启用/禁用', 'Enable', 'LINK', 13, 2, 'AUTH', '250', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (116, '查询', 'Select', 'BUTTON', 13, 2, 'AUTH', '248', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (117, '列表', 'List', 'LIST', 15, 1, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (118, '新增', 'Add', 'BUTTON', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (119, '查询条件', 'Condition', 'CONDITION', 15, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (120, '修改', 'Update', 'LINK', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (121, '启用/禁用', 'Enable', 'LINK', 15, 2, 'AUTH', '284', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (122, '查询', 'Select', 'BUTTON', 15, 2, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (123, '列表', 'List', 'LIST', 16, 1, 'AUTH', '288', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (124, '新增', 'Add', 'BUTTON', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (125, '修改', 'Update', 'LINK', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (126, '删除', 'Delete', 'LINK', 16, 2, 'AUTH', '290', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (127, '列表', 'List', 'LIST', 21, 1, 'AUTH', '1031,3044,1030', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (128, '作废', 'Cancel', 'BUTTON', 21, 1, 'AUTH', '1029,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (129, '查询条件', 'Condition', 'CONDITION', 21, 1, 'AUTH', '3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (130, '查询', 'Select', 'BUTTON', 21, 2, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (131, '待审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (132, '待审核-批量通过', 'BatchPass', 'BUTTON', 27, 1, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (133, '待审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (134, '待审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (135, '待审核-批量不通过', 'BatchNoPass', 'BUTTON', 27, 2, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (136, '待审核-通过', 'Pass', 'LINK', 27, 1, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (137, '待审核-不通过', 'NoPass', 'LINK', 27, 2, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (138, '已审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (139, '已审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (140, '已审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (141, '列表', 'List', 'LIST', 28, 1, 'AUTH', '2037,2038,1098', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (142, '批量提交', 'BatchSubmit', 'BUTTON', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (143, '查询条件', 'Condition', 'CONDITION', 28, 1, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (144, '查询', 'Select', 'BUTTON', 28, 3, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (145, '批量导出', 'BatchExport', 'BUTTON', 28, 2, 'AUTH', '2042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (146, '提交', 'Submit', 'LINK', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (147, '撤回/取消撤回', 'Reject', 'LINK', 28, 2, 'AUTH', '2040,2041', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (148, '列表', 'List', 'LIST', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (149, '导出', 'Export', 'BUTTON', 29, 1, 'AUTH', '1027', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (150, '查询条件', 'Condition', 'CONDITION', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (151, '查询', 'Select', 'BUTTON', 29, 2, 'AUTH', '1026, 1028', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (152, '撤回/取消撤回', 'Reject', 'LINK', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (153, '列表', 'List', 'LIST', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (154, '新增', 'Add', 'BUTTON', 33, 1, 'AUTH', '2018,2027', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (155, '批量导入', 'BatchImport', 'BUTTON', 33, 2, 'AUTH', '2019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (156, '查询条件', 'Condition', 'CONDITION', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (157, '查询', 'Select', 'BUTTON', 33, 5, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (158, '批量导出', 'BatchExport', 'BUTTON', 33, 3, 'AUTH', '2020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (159, '作废', 'Cancel', 'BUTTON', 33, 4, 'AUTH', '2051', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (160, '修改', 'Update', 'LINK', 33, 1, 'AUTH', '2018', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (161, '列表', 'List', 'LIST', 34, 1, 'AUTH', '2021,2022,2023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (162, '查询条件', 'Condition', 'CONDITION', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (163, '查询', 'Select', 'BUTTON', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (164, '调配', 'Allocate', 'LINK', 34, 1, 'AUTH', '2024,2030,2031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (165, '发布SOP', 'Publish', 'LINK', 34, 2, 'AUTH', '2028', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (166, '撤销发布', 'UnPublish', 'LINK', 34, 3, 'AUTH', '2029', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (167, '列表', 'List', 'LIST', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (168, '查询条件', 'Condition', 'CONDITION', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (169, '查询', 'Select', 'BUTTON', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (170, '列表', 'List', 'LIST', 38, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (171, '列表', 'List', 'LIST', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (172, '查询条件', 'Condition', 'CONDITION', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (173, '作废', 'Cancel', 'BUTTON', 41, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (174, '查询', 'Select', 'BUTTON', 41, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (175, '填报', 'Fill', 'LINK', 41, 2, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (176, '计划变更报备', 'ProjectExchange', 'LINK', 41, 3, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (178, '新增违规 ', 'AddViolation', 'LINK', 41, 4, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (179, '上报质量问题', 'ProblemSubmit', 'LINK', 41, 5, 'AUTH', '1031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (180, '列表', 'List', 'LIST', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (181, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (182, '作废', 'Cancel', 'BUTTON', 42, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (183, '查询', 'Select', 'BUTTON', 42, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (184, '填报', 'Fill', 'LINK', 42, 2, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (185, '计划变更报备', 'ProjectExchange', 'LINK', 42, 3, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (186, '新增违规 ', 'AddViolation', 'LINK', 42, 4, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (187, '上报质量问题', 'ProblemSubmit', 'LINK', 42, 5, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (188, '列表', 'List', 'LIST', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (189, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (190, '新增', 'Add', 'BUTTON', 43, 1, 'AUTH', '2033,2034,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (191, '作废', 'Cancel', 'BUTTON', 43, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (192, '查询', 'Select', 'BUTTON', 43, 3, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (193, '列表', 'List', 'LIST', 44, 1, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (194, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (196, '查询', 'Select', 'BUTTON', 44, 2, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (197, '列表', 'List', 'LIST', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (198, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (199, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (200, '跟进', 'Follow', 'LINK', 47, 1, 'AUTH', '1014,1017,1018', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (201, '关闭', 'Close', 'LINK', 47, 2, 'AUTH', '1013', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (202, '重启', 'Restart', 'LINK', 47, 3, 'AUTH', '1016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (203, '列表', 'List', 'LIST', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (204, '查询条件', 'Condition', 'CONDITION', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (205, '查询', 'Select', 'BUTTON', 48, 1, 'AUTH', '1023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (206, '跟进', 'Follow', 'LINK', 48, 1, 'AUTH', '1019,1020,1022,1096', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (207, '关闭', 'Close', 'LINK', 48, 2, 'AUTH', '1021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (208, '重启', 'Restart', 'LINK', 48, 3, 'AUTH', '1024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (209, '列表', 'List', 'LIST', 51, 1, 'AUTH', '1034,1037', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (210, '查询条件', 'Condition', 'CONDITION', 51, 1, 'AUTH', '1034,1037', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (211, '新增', 'Add', 'BUTTON', 51, 1, 'AUTH', '1036,1038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (212, '作废', 'Cancel', 'BUTTON', 51, 2, 'AUTH', '1032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (213, '批量划定', 'BatchDelimit', 'BUTTON', 51, 3, 'AUTH', '2009', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (214, '查询', 'Select', 'BUTTON', 51, 4, 'AUTH', '1034,1037', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (215, '划定', 'Delimit', 'LINK', 51, 1, 'AUTH', '2009,2060', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (216, '修改', 'Update', 'LINK', 51, 2, 'AUTH', '1036,1038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (217, '重新划定', 'ReDelimit', 'LINK', 51, 3, 'AUTH', '2009,2060', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (218, '列表', 'List', 'LIST', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (219, '查询条件', 'Condition', 'CONDITION', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (220, '新增', 'Add', 'BUTTON', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (221, '查询', 'Select', 'BUTTON', 54, 2, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (222, '修改', 'Update', 'LINK', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (223, '作废', 'Cancel', 'LINK', 54, 2, 'AUTH', '2005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (224, '设置考勤组', 'Set', 'LINK', 54, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (225, '发布', 'Publish', 'LINK', 54, 4, 'AUTH', '2002', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (226, '重启', 'Restart', 'LINK', 54, 5, 'AUTH', '2004', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (227, '关闭', 'Close', 'LINK', 54, 6, 'AUTH', '2003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (228, '列表', 'List', 'LIST', 55, 1, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (229, '查询条件', 'Condition', 'CONDITION', 55, 1, 'AUTH', '2006', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (230, '新增服务范围', 'Add', 'BUTTON', 55, 1, 'AUTH', '2008,2009', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (231, '查询', 'Select', 'BUTTON', 55, 2, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (232, '移除', 'Delete', 'LINK', 55, 1, 'AUTH', '2010', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (233, '列表', 'List', 'LIST', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (234, '查询条件', 'Condition', 'CONDITION', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (235, '新增', 'Add', 'BUTTON', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (236, '查询', 'Select', 'BUTTON', 56, 2, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (237, '修改', 'Update', 'LINK', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (238, '删除', 'Delete', 'LINK', 56, 2, 'AUTH', '2013', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (239, '任务管理', 'taskManage', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (240, '任务管理', 'task', 'MENU', 239, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (241, '列表', 'List', 'LIST', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (242, '查询条件', 'Condition', 'CONDITION', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (243, '查询', 'Select', 'BUTTON', 240, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (244, '导出日志', 'Export', 'LINK', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (245, '下载文件', 'Download', 'LINK', 240, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (246, '获取菜单权限', '/api/admin/sys/privilege/list', 'URL', 64, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (247, '获取用户权限', '/api/admin/common/get_menu', 'URL', 64, 8, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (248, '组织架构查询', '/api/admin/org/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (249, '组织架构保存/修改', '/api/admin/org/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (250, '组织架构启用/禁用', '/api/admin/org/enable', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (251, '获取机构下所有用户', '/api/admin/common/get_org_user_list', 'URL', 64, 9, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (252, '用户查询', '/api/admin/user/list', 'URL', 15, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (253, '服务档位删除', '/api/sys/level/delete', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (254, '服务档位', '/api/sys/level/get', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (255, '服务档位列表', '/api/sys/level/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (256, '服务档位列表', '/api/sys/level/query', 'URL', 6, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (257, '服务档位修改', '/api/sys/level/update', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (258, '客户新增', '/api/sys/custom/add', 'URL', 3, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (259, '客户删除', '/api/sys/custom/delete', 'URL', 3, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (260, '客户查询', '/api/sys/custom/get', 'URL', 3, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (261, '客户批量导入', '/api/sys/custom/import', 'URL', 3, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (262, '客户查询条件', '/api/sys/custom/query', 'URL', 3, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (263, '客户修改', '/api/sys/custom/update', 'URL', 3, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (264, '供应商新增', '/api/sys/supplier/add', 'URL', 4, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (265, '供应商启用/禁用', '/api/sys/supplier/enable', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (266, '供应商列表', '/api/sys/supplier/get', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (267, '供应商列表', '/api/sys/supplier/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (268, '供应商列表', '/api/sys/supplier/query', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (269, '供应商修改', '/api/sys/supplier/update', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (270, '设备新增修改', '/api/sys/device/save', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (271, '设备作废', '/api/sys/device/delete', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (272, '设备禁用', '/api/sys/device/disable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (273, '设备查询条件', '/api/sys/device/get', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (274, '设备列表', '/api/sys/device/list', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (275, '设备查询条件', '/api/sys/device/query', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (276, '设备新增', '/api/sys/device/update', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (277, '考勤新增', '/api/sys/ding/group/add', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (278, '考勤删除', '/api/sys/ding/group/delete', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (279, '考勤查询', '/api/sys/ding/group/get', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (280, '考勤查询', '/api/sys/ding/group/list', 'URL', 7, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (281, '考勤查询条件', '/api/sys/ding/group/query', 'URL', 7, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (282, '考勤修改', '/api/sys/ding/group/update', 'URL', 7, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (283, '用户新增/修改', '/api/admin/user/save', 'URL', 15, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (284, '用户启用/禁用', '/api/admin/user/enable', 'URL', 15, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (285, '获取所有角色', '/api/admin/common/get_role_list', 'URL', 64, 10, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (286, '获取所有用户', '/api/admin/common/get_user_list', 'URL', 64, 11, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (287, '获取所有机构', '/api/admin/common/get_org_list', 'URL', 64, 12, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (288, '角色查询', '/api/admin/role/list', 'URL', 16, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (289, '角色新增/修改', '/api/admin/role/save', 'URL', 16, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (290, '角色删除', '/api/admin/role/delete', 'URL', 16, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (300, '服务档位新增', '/api/sys/level/add', 'URL', 6, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1000, '通知公告', '/api/sys/notice/delete', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1001, '通知公告', '/api/sys/notice/disable', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1002, '通知公告', '/api/sys/notice/get', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1003, '通知公告发布撤销', '/api/sys/notice/publishOrUnpublish', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1004, '通知公告列表', '/api/sys/notice/query', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1005, '通知公告新增修改', '/api/sys/notice/save', 'URL', 9, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1006, '回执查询统计', '/api/sys/message/count', 'URL', 9, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1007, '回执查询到出', '/api/sys/message/export', 'URL', 9, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1008, '回执查询', '/api/sys/message/query', 'URL', 9, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1009, '按多个类型统计当前用户未读消息', '/api/sys/message/countByTypes', 'URL', 59, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1010, '查看消息详细', '/api/sys/message/get', 'URL', 59, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1011, '按多个类型查询当前用户未读消息', '/api/sys/message/pageByTypes', 'URL', 59, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1012, '标记消息已阅读', '/api/sys/message/read', 'URL', 59, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1013, '延期预警关闭', '/api/admin/tb/delay/warn/close', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1014, '延期预警跟进', '/api/admin/tb/delay/warn/get', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1015, '延期预警查询', '/api/admin/tb/delay/warn/query', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1016, '延期预警重启', '/api/admin/tb/delay/warn/restart', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1017, '延期预警跟进', '/api/admin/tb/delay/warn/detail/list', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1018, '延期预警跟进', '/api/admin/tb/delay/warn/detail/save', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1019, '违规登记跟进', '/api/admin/tb/violation/detail/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1020, '违规登记跟进', '/api/admin/tb/violation/detail/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1021, '违规登记关闭', '/api/admin/tb/violation/close', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1022, '违规登记跟进', '/api/admin/tb/violation/get', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1023, '违规登记查询', '/api/admin/tb/violation/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1024, '违规登记重启', '/api/admin/tb/violation/restart', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1025, '违规登记保存', '/api/admin/tb/violation/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1026, '工时统计', '/api/admin/tb/ding/count', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1027, '工时统计', '/api/admin/tb/ding/export', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1028, '工时统计', '/api/admin/tb/ding/query', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1029, '质量问题反馈', '/api/admin/tb/quality/problem/apply/delete', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1030, '质量问题反馈', '/api/admin/tb/quality/problem/apply/query', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1031, '质量问题反馈', '/api/admin/tb/quality/problem/apply/save', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1032, '批量禁用', '/api/admin/tb/crm/batchDisable', 'URL', 51, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1034, '统计待划定数量', '/api/admin/tb/crm/countWaitZone', 'URL', 51, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1035, '派单信息表删除', '/api/admin/tb/crm/delete', 'URL', 51, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1036, '获取单个派单', '/api/admin/tb/crm/get', 'URL', 51, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1037, '分页查询', '/api/admin/tb/crm/query', 'URL', 51, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1038, '新增修改', '/api/admin/tb/crm/save', 'URL', 51, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1039, '客户表查询', '/api/sys/custom/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1040, '获取产品信息列表接口', '/api/admin/tb/product/list', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1041, '考勤明细查询', '/api/admin/tb/ding/attendance/count', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1042, '考勤明细查询', '/api/admin/tb/ding/attendance/export', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1043, '考勤明细查询', '/api/admin/tb/ding/attendance/query', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1044, '考勤明细查询', 'dingDetail', 'MENU', 25, 1, NULL, '1041,1042,1043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1045, '设备品牌表列表接口', '/api/sys/device/brand/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1046, '设备启用', '/api/sys/device/enable', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1047, '设备导入', '/api/sys/device/import', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1048, '批量启用', 'BatchEnable', 'BUTTON', 5, 4, 'AUTH', '1045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1049, '批量禁用', 'BatchUnEnable', 'BUTTON', 5, 5, 'AUTH', '272', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1050, '批量导入', 'BatchImport', 'BUTTON', 5, 6, 'AUTH', '1047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1051, '获取SOP用户', '/api/admin/sop/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1052, '修改', 'Update', 'LINK', 5, 1, 'AUTH', '276', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1053, '禁用启用', 'Enable', 'LINK', 5, 2, 'AUTH', '272,1046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1054, '获取用户', '/api/admin/tb/violation/creator/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1055, '获取用户', '/api/admin/tb/violation/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1056, '获取用户', '/api/admin/tb/quality/problem/apply/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1057, '二期接口', '/api/sop/analyse/attendance', 'URL', 1103, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1058, '二期接口', '/api/sop/analyse/list', 'URL', 1103, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1059, '二期接口', '/api/sop/analyse/overview', 'URL', 1103, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1060, '二期接口', '/api/sop/analyse/processing', 'URL', 1103, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1061, '二期接口', '/api/sop/analyse/trend', 'URL', 1103, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1062, '二期接口', '/api/service/analyse/device/use', 'URL', 1102, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1063, '二期接口', '/api/service/analyse/list', 'URL', 1102, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1064, '二期接口', '/api/service/analyse/overview', 'URL', 1102, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1065, '二期接口', '/api/service/analyse/personnel/quota', 'URL', 1102, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1066, '二期接口', '/api/service/analyse/project/allocation', 'URL', 1102, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1067, '二期接口', '/api/service/analyse/region/personnel', 'URL', 1102, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1068, '二期接口', '/api/service/analyse/supplier', 'URL', 1102, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1069, '二期接口', '/api/service/analyse/supplier/detail', 'URL', 1102, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1070, '二期接口', '/api/service/analyse/supplier/region', 'URL', 1102, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1071, '二期接口', '/api/service/analyse/supplier/region/detail', 'URL', 1102, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1072, '二期接口', '/api/service/analyse/supplier/role', 'URL', 1102, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1073, '二期接口', '/api/admin/tb/crm/analyse/custom/type', 'URL', 1101, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1074, '二期接口', '/api/admin/tb/crm/analyse/custom/type/detail', 'URL', 1101, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1075, '二期接口', '/api/admin/tb/crm/analyse/monthly', 'URL', 1101, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1076, '二期接口', '/api/admin/tb/crm/analyse/monthly/detail', 'URL', 1101, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1077, '二期接口', '/api/admin/tb/crm/analyse/project', 'URL', 1101, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1078, '二期接口', '/api/admin/tb/crm/analyse/project/detail', 'URL', 1101, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1079, '二期接口', '/api/admin/tb/crm/analyse/region', 'URL', 1101, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1080, '二期接口', '/api/admin/tb/crm/analyse/region/detail', 'URL', 1101, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1081, '二期接口', '/api/admin/tb/crm/analyse/supplier', 'URL', 1101, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1082, '二期接口', '/api/admin/tb/crm/analyse/supplier/detail', 'URL', 1101, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1083, '二期接口', '/api/quality/analyse/influence', 'URL', 1106, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1084, '二期接口', '/api/quality/analyse/influence/detail', 'URL', 1106, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1085, '二期接口', '/api/quality/analyse/list', 'URL', 1106, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1086, '二期接口', '/api/quality/analyse/pie', 'URL', 1106, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1087, '二期接口', '/api/quality/analyse/progress', 'URL', 1106, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1088, '二期接口', '/api/quality/analyse/radar', 'URL', 1106, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1089, '二期接口', '/api/quality/analyse/supplier/detail', 'URL', 1106, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1090, '二期接口', '/api/sys/device/monitor/count', 'URL', 1105, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1091, '二期接口', '/api/sys/device/monitor/countByModel', 'URL', 1105, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1092, '二期接口', '/api/sys/device/monitor/countByServiceUnit', 'URL', 1105, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1093, '二期接口', '/api/sys/device/monitor/countByServiceUnitDetail', 'URL', 1105, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1094, '二期接口', '/api/sop/schedule/detail', 'URL', 1104, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1095, '二期接口', '/api/sop/schedule/progress', 'URL', 1104, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1096, '违规登记跟进', '/api/admin/tb/violation/getByCode', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1097, '二期接口', '/api/sop/schedule/projectProgress', 'URL', 1104, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1098, '考勤提交-明细', '/api/admin/tb/ding/detail', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1099, '二期接口', '/api/admin/tb/crm/analyse/project/count', 'URL', 1101, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1100, '数据驾驶舱', 'dataCockpit', 'MENU', NULL, 12, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1101, '派单分析', 'crmAnalyse', 'MENU', 1100, 1, NULL, '1073,1074,1075,1076,1077,1078,1079,1080,1081,1082,1099', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1102, '服务单元分析', 'serviceAnalyse', 'MENU', 1100, 2, NULL, '1062,1063,1064,1065,1066,1067,1068,1069,1070,1071,1072', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1103, 'SOP预警监控', 'sopAnalyse', 'MENU', 1100, 3, NULL, '1057,1058,1059,1060,1061', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1104, '项目进度监控', 'sopSchedule', 'MENU', 1100, 4, NULL, '1094,1095,1097', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1105, '设备保障监控', 'deviceMonitor', 'MENU', 1100, 5, NULL, '1090,1091,1092,1093', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1106, '质量分析监控', 'qualityAnalyse', 'MENU', 1100, 6, NULL, '1083,1084,1085,1086,1087,1088,1089', 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2000, '服务单元管理-编辑', '/api/service/service/unit/edit', 'URL', 54, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2001, '服务单元管理-分页查询', '/api/service/service/unit/page', 'URL', 54, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2002, '服务单元管理-发布', '/api/service/service/unit/publish', 'URL', 54, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2003, '服务单元管理-关闭', '/api/service/service/unit/cloze', 'URL', 54, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2004, '服务单元管理-重启', '/api/service/service/unit/restart', 'URL', 54, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2005, '服务单元管理-作废', '/api/service/service/unit/cancel', 'URL', 54, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2006, '服务范围管理-分页查询', '/api/service/service/scope/page', 'URL', 55, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2007, '服务范围管理-派单小计', '/api/service/service/scope/subTotal', 'URL', 55, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2008, '服务范围管理-新增服务范围-未绑定派单分页查询', '/api/service/service/scope/unbind/page', 'URL', 55, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2009, '服务范围管理-划定派单和服务单元关系', '/api/service/service/scope/bind', 'URL', 55, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2010, '服务范围管理-移出', '/api/service/service/scope/unbind', 'URL', 55, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2011, '服务区域规划-分页查询', '/api/service/service/region/page', 'URL', 56, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2012, '服务区域规划-编辑', '/api/service/service/region/edit', 'URL', 56, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2013, '服务区域规划-删除', '/api/service/service/region/remove', 'URL', 56, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2014, '日志管理-查询', '/api/sys/log/page', 'URL', 10, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2015, '公共接口-查询服务单元列表', '/api/admin/common/query_service_unit', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2016, '人员档案管理-分页查询', '/api/admin/user/archives/page', 'URL', 33, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2017, '人员档案管理-小计', '/api/admin/user/archives/sub_total', 'URL', 33, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2018, '人员档案管理-编辑', '/api/admin/user/archives/edit', 'URL', 33, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2019, '人员档案管理-导入', '/api/admin/user/archives/import', 'URL', 33, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2020, '人员档案管理-导出', '/api/admin/user/archives/export', 'URL', 33, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2021, '人员调配管理-分页查询', '/api/admin/user/archives/allocation/page', 'URL', 34, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2022, '人员调配管理-小计查询', '/api/admin/user/archives/allocation/sub_total', 'URL', 34, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2023, '人员调配管理-根据派单号查询派单分配信息', '/api/admin/user/archives/allocation/detail_by_crm_no', 'URL', 34, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2024, '人员调配管理-调配', '/api/admin/user/archives/allocation/edit', 'URL', 3079, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2025, '人员调配管理—自动调配', '/api/admin/user/archives/allocation/auto_edit', 'URL', 34, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2026, '批量调配', 'AllocateBatch', 'BUTTON', 34, 4, 'AUTH', '2025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2027, '人员档案管理-生成编号', '/api/admin/user/archives/create_code', 'URL', 33, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2028, '人员调配管理-发布', '/api/admin/user/archives/allocation/publish', 'URL', 34, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2029, '人员调配管理-撤销发布', '/api/admin/user/archives/allocation/un_publish', 'URL', 34, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2030, '人员调配管理-查询空闲工程师', '/api/admin/user/archives/allocation/free_engineer', 'URL', 34, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2031, '人员调配管理-查询空闲区域协调人', '/api/admin/user/archives/allocation/free_coordinator', 'URL', 34, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2032, 'sop管理 - 设备出入库登记查询', '/api/admin/device/in/out/sop_page', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2033, '查询可出库的设备信息', '/api/admin/device/in/out/can_out_info', 'URL', 43, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2034, '查询可入库的设备信息', '/api/admin/device/in/out/can_in_info', 'URL', 43, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2035, '设备出入库提交', '/api/admin/device/in/out/submit', 'URL', 43, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2036, '资源保障-设备保障-出入库登记查询', '/api/admin/device/in/out/source_page', 'URL', 37, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2037, '考勤提交-分页查询', '/api/admin/tb/ding/submit/page', 'URL', 28, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2038, '考勤提交-小计', '/api/admin/tb/ding/submit/sub_total', 'URL', 28, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2039, '考勤提交-批量提交', '/api/admin/tb/ding/submit/submit_batch', 'URL', 28, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2040, '考勤提交-考勤撤回', '/api/admin/tb/ding/submit/apply_withdraw', 'URL', 28, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2041, '考勤提交-取消撤回', '/api/admin/tb/ding/submit/cancel_withdraw', 'URL', 28, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2042, '考勤提交-批量导出', '/api/admin/tb/ding/submit/export', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2043, '查询项目派单信息', '/api/admin/tb/crm/crm_project_info', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2044, '考勤打卡信息', '/api/admin/tb/ding/ding_info', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2045, '考勤结果统计', '/api/admin/tb/ding/ding_statistic', 'URL', 26, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2046, '考勤打卡保存', '/api/admin/tb/ding/ding_save', 'URL', 26, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2047, '服务区域规划-查询可选大区经理', '/api/service/service/region/find_leader', 'URL', 56, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2048, '考勤打卡-查询进行中的sop', '/api/admin/tb/ding/ding_find_running_sop', 'URL', 26, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2049, '考勤打卡-查询所有sop', '/api/admin/tb/ding/ding_find_all_sop', 'URL', 26, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2051, '人员档案管理-作废', '/api/admin/user/archives/cancel', 'URL', 33, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2052, '出入库登记查询-查询登记人列表', '/api/admin/device/in/out/registrant_list', 'URL', 64, 33, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2053, '智能客服', 'aiCustomerManage', 'MENU', NULL, 11, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2054, '智能客服', 'aiCustomerChildManage', 'MENU', 2053, 1, NULL, '2055', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2055, '软通智能客服单点登录', '/api/sso/rtzf/login', 'URL', 2054, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2056, '工时统计-同意撤回', '/api/admin/tb/ding/submit/agree_withdraw', 'URL', 29, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2057, '同意撤回', 'Reject', 'LINK', 29, 5, 'AUTH', '2056', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2060, '查询同服务单元客户绑定数据', '/api/admin/tb/crm/crm_bind_info', 'URL', 51, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2061, '派单详情调配信息查询', '/api/admin/user/archives/allocation/info', 'URL', 3079, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2062, '人员调配可选人员查询', '/api/admin/user/archives/allocation/can_choose', 'URL', 3079, 10, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3000, '修改密码', '/api/admin/common/update_password', 'URL', 64, 13, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3001, '流程审批接口', '/api/admin/flow/approve', 'URL', 64, 14, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3002, '创建流程接口', '/api/admin/flow/createDeployment', 'URL', 64, 15, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3003, '结束流程接口', '/api/admin/flow/delete', 'URL', 64, 16, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3004, '获取流程部署信息接口', '/api/admin/flow/deployment/data', 'URL', 64, 17, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3005, '获取流程表单属性接口', '/api/admin/flow/form_properties/get', 'URL', 64, 18, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3006, '获取流程待办', '/api/admin/flow/task/list', 'URL', 64, 19, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3007, '流程转办接口', '/api/admin/flow/task/approver/exchange', 'URL', 64, 20, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3008, '上传流程接口', '/api/admin/flow/upload', 'URL', 64, 21, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3009, '获取流程详细信息接口', '/api/admin/flow/view', 'URL', 64, 22, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3010, '清除缓存接口', '/api/admin/common/cache_clear', 'URL', 3012, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3011, '超管菜单', 'adminManage', 'MENU', NULL, 10, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3012, '清除缓存', 'cacheManage', 'MENU', 3011, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3013, '消息提醒规则', 'messageRuleManage', 'MENU', 3011, 2, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3014, '清除缓存', 'Clear', 'BUTTON', 3012, 1, 'AUTH', '3010', 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3015, '查询枚举接口', '/api/admin/common/query_enum', 'URL', 64, 23, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3016, '角色权限查询接口', '/api/admin/role/privilege/list', 'URL', 16, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3017, '考勤异常申请接口', '/api/admin/ding/exception/apply/save', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3018, '根据角色类型获取所有用户', '/api/admin/common/get_user_list_by_role_type', 'URL', 64, 24, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3019, '考勤异常待审核接口', '/api/admin/ding/exception/apply/flow/task/un_done/list', 'URL', 27, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3020, '考勤异常已审核接口', '/api/admin/ding/exception/apply/flow/task/done/list', 'URL', 27, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3021, '考勤异常审核接口', '/api/admin/ding/exception/apply/flow/approve', 'URL', 27, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3022, '考勤异常批量审核接口', '/api/admin/ding/exception/apply/flow/batch/approve', 'URL', 27, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3023, '获取违规延期预警待办接口', '/api/admin/my/work_bench/warn/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3024, '处理申请', 'Finish', 'LINK', 44, 1, 'AUTH', '3030,3031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3025, '项目变更计划申请接口', '/api/admin/project/exchange/save', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3026, '项目变更计划列表接口', '/api/admin/project/exchange/list', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3027, '流程控件查询接口', '/api/admin/widget/flow/select', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3028, '控件查询列表接口', '/api/admin/widget/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3029, '批量结束流程接口', '/api/admin/flow/batch/delete', 'URL', 64, 28, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3030, '项目变更计划审核接口', '/api/admin/project/exchange/flow/approve', 'URL', 40, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3031, '项目变更计划详情接口', '/api/admin/project/exchange/view', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3032, '获取流程待办计数接口', '/api/admin/flow/task/list/count', 'URL', 64, 29, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3033, '获取违规/延期预警待办计数接口', '/api/admin/my/work_bench/warn/list/count', 'URL', 64, 30, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3034, 'sop元数据查询接口', '/api/admin/sop/metadata/list', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3035, 'sop元数据修改接口', '/api/admin/sop/metadata/save', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3036, '获取流程版本信息接口', '/api/admin/flow/version/list', 'URL', 64, 31, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3037, '编辑', 'Edit', 'LINK', 41, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3038, '删除', 'Cancel', 'LINK', 41, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3039, '编辑', 'Edit', 'LINK', 42, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3040, '删除', 'Cancel', 'LINK', 42, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3041, 'sop申请接口', '/api/admin/sop/apply', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3042, 'sop填报接口', '/api/admin/sop/approve', 'URL', 40, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3043, 'sop列表接口', '/api/admin/sop/list', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3044, '质量问题反馈审核', '/api/admin/tb/quality/problem/apply/approve', 'URL', 21, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3045, 'sop删除', '/api/admin/sop/cancel', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3046, 'sop批量删除', '/api/admin/sop/batch/cancel', 'URL', 40, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3047, 'sop编辑', '/api/admin/sop/edit', 'URL', 40, 11, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3048, 'sop保存 ', '/api/admin/sop/save', 'URL', 40, 12, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3049, '审核', 'Approve', 'LINK', 21, 1, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3050, '获取所有用户（动态表单用）接口', '/api/admin/common/get_user_list/form_widget', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3051, '小程序签名预览', '/api/admin/common/file/preview/app', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3052, '考勤异常详情接口', '/api/admin/ding/exception/apply/view', 'URL', 27, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3053, '设备发货管理', 'deviceDelivery', 'MENU', 36, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3054, '列表', 'List', 'LIST', 3053, 1, 'AUTH', '3067,3071', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3055, '查询条件', 'Condition', 'CONDITION', 3053, 1, 'AUTH', '3067,3071', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3056, '批量导入', 'BatchImport', 'BUTTON', 3053, 1, 'AUTH', '3069', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3057, '批量导出', 'BatchExport', 'BUTTON', 3053, 2, 'AUTH', '3070', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3058, '批量发货', 'BatchDeliver', 'BUTTON', 3053, 3, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3059, '批量签收', 'BatchReceive', 'BUTTON', 3053, 4, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3060, '批量作废', 'BatchCancel', 'BUTTON', 3053, 5, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3061, '修改', 'Update', 'LINK', 3053, 1, 'AUTH', '3065,3068', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3062, '作废', 'Cancel', 'LINK', 3053, 2, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3063, '发货', 'Deliver', 'LINK', 3053, 3, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3064, '签收', 'Receive', 'LINK', 3053, 4, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3065, '设备发货保存接口', '/api/admin/device/delivery/save', 'URL', 3053, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3066, '设备发货状态修改接口', '/api/admin/device/delivery/status/update', 'URL', 3053, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3067, '设备发货列表分页查询', '/api/admin/device/delivery/page', 'URL', 3053, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3068, '设备发货查询单个记录接口', '/api/admin/device/delivery/find/by/id', 'URL', 3053, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3069, '设备发货导入', '/api/admin/device/delivery/import', 'URL', 3053, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3070, '设备发货导出', '/api/admin/device/delivery/export', 'URL', 3053, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3071, '设备发货人列表查询', '/api/admin/device/delivery/delivery_user_list', 'URL', 3053, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3072, '绑定供应商', 'bandSupplier', 'LINK', 15, 4, 'AUTH', '3074,3075', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3073, '根据用户角色查询供应商信息接口', '/api/sys/supplier/list/by/user_role', 'URL', 4, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3074, '查询绑定供应商', '/api/admin/user/query/supplier', 'URL', 15, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3075, '绑定供应商', '/api/admin/user/band/supplier', 'URL', 15, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3076, '批量返还', 'BatchReturn', 'BUTTON', 3053, 5, 'AUTH', '3078,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3077, '返还', 'Return', 'LINK', 3053, 6, 'AUTH', '3078,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3078, '设备发货可返还设备列表查询', '/api/admin/device/delivery/can_return_info', 'URL', 3053, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3079, '创建sop', 'CreateSop', 'LINK', 51, 4, 'AUTH', '2024,2061,2062,3080,3081,3082,3083,3084,3085,3086,3087', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3080, '导入派单明细', '/api/admin/tb/crm/detail/sop/import', 'URL', 3079, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3081, '获取派单信息', '/api/admin/tb/crm/detail/sop/var/info', 'URL', 3079, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3082, '派单明细sop变量保存', '/api/admin/tb/crm/detail/sop/var/save', 'URL', 3079, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3083, '获取派单明细sop信息', '/api/admin/tb/crm/detail/sop/info', 'URL', 3079, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3084, '创建/编辑派单明细sop', '/api/admin/tb/crm/detail/sop/save', 'URL', 3079, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3085, '派单明细sop发布', '/api/admin/tb/crm/detail/sop/publish', 'URL', 3079, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3086, '派单明细sop删除', '/api/admin/tb/crm/detail/sop/delete', 'URL', 3079, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3087, '编辑派单明细sop', '/api/admin/tb/crm/detail/sop/edit', 'URL', 3079, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3088, 'sop人员配置', '/api/admin/sop/allocation', 'URL', 40, 13, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3089, '人员配置', 'Allocation', 'LINK', 41, 1, 'AUTH', '3088', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3090, '人员配置', 'Allocation', 'LINK', 42, 1, 'AUTH', '3088', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3091, 'sop快捷搜索保存', '/api/admin/sop/quick/save', 'URL', 40, 14, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3092, 'sop快捷搜索列表', '/api/admin/sop/quick/list', 'URL', 40, 15, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3093, 'sop快捷搜索删除', '/api/admin/sop/quick/delete', 'URL', 40, 16, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3094, '设备管理', 'DeviceManage', 'LINK', 41, 1, 'AUTH', '4018,4019,4020,4021,4022,4023,4024,4025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3095, '设备管理', 'DeviceManage', 'LINK', 42, 1, 'AUTH', '4018,4019,4020,4021,4022,4023,4024,4025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4000, '设备管理', 'deviceDeliveryManage', 'MENU', 36, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4010, '设备签收列表', 'List', 'LIST', 4000, 1, 'AUTH', '4018', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4011, '签收', 'Sign', 'LINK', 4000, 2, 'AUTH', '4019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4012, '未签收', 'UnSigned', 'LINK', 4000, 3, 'AUTH', '4020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4013, '编辑', 'Update', 'LINK', 4000, 4, 'AUTH', '4021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4014, '设备去处列表', 'List', 'LIST', 4000, 1, 'AUTH', '4022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4015, '入库', 'In', 'LINK', 4000, 2, 'AUTH', '4023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4016, '中转', 'Transfer', 'LINK', 4000, 3, 'AUTH', '4024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4017, '编辑', 'Update', 'LINK', 4000, 4, 'AUTH', '4025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4018, '设备签收列表', '/api/admin/device/manage/sign/list', 'URL', 4000, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4019, '设备签收', '/api/admin/device/manage/sign', 'URL', 4000, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4020, '设备未签收', '/api/admin/device/manage/unsigned', 'URL', 4000, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4021, '设备签收编辑', '/api/admin/device/manage/sign/edit', 'URL', 4000, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4022, '设备去处列表', '/api/admin/device/manage/place/list', 'URL', 4000, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4023, '设备去处入库', '/api/admin/device/manage/in', 'URL', 4000, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4024, '设备去处中转', '/api/admin/device/manage/transfer', 'URL', 4000, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4025, '设备去处编辑', '/api/admin/device/manage/place/edit', 'URL', 4000, 4, 'AUTH', NULL, 1, 1, 0);

INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (1, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (2, NULL, '总负责人', 1, 'PMO', NULL, NULL, 530746864698392576, 1717660332923, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (3, NULL, '业务线负责人', 1, 'BUSSINESS', NULL, NULL, 1, 1716950270502, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (4, NULL, '大区经理', 1, 'REGION_MANAGER', NULL, NULL, 530401632861552640, 1717398872565, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (5, NULL, '区域协调人', 1, 'REGION_COORDINATOR', NULL, NULL, 1, 1716950252514, 1, '外包角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (6, NULL, '实施工程师', 1, 'EFFECT_ENGINEER', NULL, NULL, 530401632861552640, 1717488662716, 1, '外包角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (7, NULL, '助理工程师', 1, 'ASSISTANT_ENGINEER', NULL, NULL, 1, 1717662833783, 1, '外包角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (8, NULL, 'QA', 1, 'QA', NULL, NULL, 1, 1716950233522, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (9, NULL, '技术客服', 1, 'CUSTOM', NULL, NULL, 1, 1716950249132, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (10, NULL, '客户经理', 1, 'ACCOUNT_MANAGER', NULL, NULL, 1, 1716950216374, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (11, NULL, '乙方总负责人', 1, 'THIRD_PMO', NULL, NULL, 1, 1716950241169, 1, '系统角色');
INSERT INTO `sys_role` (`id`, `org_id`, `name`, `enable`, `type`, `create_id`, `create_time`, `update_id`, `update_time`, `default_role`, `interpret`) VALUES (12, NULL, '设备发货员', 1, 'DEVICE_DELIVERY', NULL, NULL, 1, 1716950224675, 1, '系统角色');

INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796992, 10, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796993, 10, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796994, 10, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796995, 10, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796996, 10, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796997, 10, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796998, 10, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235796999, 10, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797000, 10, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797001, 10, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797002, 10, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797003, 10, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797004, 10, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797005, 10, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797006, 10, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797007, 10, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797008, 10, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797009, 10, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797010, 10, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797011, 10, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797012, 10, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797013, 10, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797014, 10, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797015, 10, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797016, 10, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797017, 10, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797018, 10, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797019, 10, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797020, 10, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521799235797021, 10, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074624, 12, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074625, 12, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074626, 12, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074627, 12, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074628, 12, 3056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074629, 12, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074630, 12, 3058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074631, 12, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074632, 12, 3060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074633, 12, 3061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074634, 12, 3062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074635, 12, 3063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074636, 12, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074637, 12, 3065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074638, 12, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074639, 12, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074640, 12, 3068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074641, 12, 3069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074642, 12, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074643, 12, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521834082074644, 12, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470720, 8, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470721, 8, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470722, 8, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470723, 8, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470724, 8, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470725, 8, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470726, 8, 2051, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470727, 8, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470728, 8, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470729, 8, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470730, 8, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470731, 8, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470732, 8, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470733, 8, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470734, 8, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470735, 8, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470736, 8, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470737, 8, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470738, 8, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470739, 8, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470740, 8, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470741, 8, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470742, 8, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470743, 8, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470744, 8, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470745, 8, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470746, 8, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871197470747, 8, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665024, 8, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665025, 8, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665026, 8, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665027, 8, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665028, 8, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665029, 8, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665030, 8, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665031, 8, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665032, 8, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665033, 8, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665034, 8, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665035, 8, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665036, 8, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665037, 8, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665038, 8, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665039, 8, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665040, 8, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665041, 8, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665042, 8, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665043, 8, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665044, 8, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665045, 8, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665046, 8, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665047, 8, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665048, 8, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665049, 8, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665050, 8, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665051, 8, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665052, 8, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665053, 8, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665054, 8, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665055, 8, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665056, 8, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665057, 8, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665058, 8, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665059, 8, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665060, 8, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665061, 8, 1073, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665062, 8, 1074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665063, 8, 1075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665064, 8, 1076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665065, 8, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665066, 8, 1077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665067, 8, 1078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665068, 8, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665069, 8, 1079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665070, 8, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665071, 8, 1080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665072, 8, 1081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665073, 8, 1082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665074, 8, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665075, 8, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665076, 8, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665077, 8, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665078, 8, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665079, 8, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665080, 8, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665081, 8, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665082, 8, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665083, 8, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665084, 8, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665085, 8, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665086, 8, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665087, 8, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665088, 8, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665089, 8, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665090, 8, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665091, 8, 1099, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665092, 8, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665093, 8, 1101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665094, 8, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665095, 8, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665096, 8, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665097, 8, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665098, 8, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665099, 8, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665100, 8, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665101, 8, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665102, 8, 99, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665103, 8, 100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665104, 8, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665105, 8, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665106, 8, 103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665107, 8, 104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665108, 8, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665109, 8, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665110, 8, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665111, 8, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665112, 8, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665113, 8, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665114, 8, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665115, 8, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665116, 8, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665117, 8, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665118, 8, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665119, 8, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665120, 8, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665121, 8, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665122, 8, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665123, 8, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665124, 8, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665125, 8, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665126, 8, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665127, 8, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665128, 8, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665129, 8, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665130, 8, 154, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665131, 8, 155, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665132, 8, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665133, 8, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665134, 8, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665135, 8, 159, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665136, 8, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665137, 8, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665138, 8, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665139, 8, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665140, 8, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665141, 8, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665142, 8, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665143, 8, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665144, 8, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665145, 8, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665146, 8, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665147, 8, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665148, 8, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665149, 8, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665150, 8, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665151, 8, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665152, 8, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665153, 8, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665154, 8, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665155, 8, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665156, 8, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665157, 8, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665158, 8, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665159, 8, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665160, 8, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665161, 8, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665162, 8, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665163, 8, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665164, 8, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665165, 8, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665166, 8, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665167, 8, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665168, 8, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665169, 8, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665170, 8, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665171, 8, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665172, 8, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665173, 8, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665174, 8, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665175, 8, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665176, 8, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665177, 8, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665178, 8, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665179, 8, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665180, 8, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665181, 8, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665182, 8, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665183, 8, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665184, 8, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665185, 8, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665186, 8, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665187, 8, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665188, 8, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665189, 8, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665190, 8, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665191, 8, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665192, 8, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665193, 8, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665194, 8, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665195, 8, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665196, 8, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665197, 8, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665198, 8, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665199, 8, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665200, 8, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665201, 8, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665202, 8, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665203, 8, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665204, 8, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665205, 8, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665206, 8, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665207, 8, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665208, 8, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665209, 8, 2019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665210, 8, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665211, 8, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665212, 8, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665213, 8, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665214, 8, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665215, 8, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665216, 8, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665217, 8, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665218, 8, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665219, 8, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665220, 8, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665221, 8, 2025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665222, 8, 2026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665223, 8, 1002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665224, 8, 2027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665225, 8, 1003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665226, 8, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665227, 8, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665228, 8, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665229, 8, 1005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665230, 8, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665231, 8, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665232, 8, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665233, 8, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665234, 8, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665235, 8, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665236, 8, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665237, 8, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665238, 8, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665239, 8, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665240, 8, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665241, 8, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665242, 8, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665243, 8, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665244, 8, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665245, 8, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665246, 8, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665247, 8, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665248, 8, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665249, 8, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665250, 8, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521871201665251, 8, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341888, 11, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341889, 11, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341890, 11, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341891, 11, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341892, 11, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341893, 11, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341894, 11, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341895, 11, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341896, 11, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521903250341897, 11, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613120, 9, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613121, 9, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613122, 9, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613123, 9, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613124, 9, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613125, 9, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613126, 9, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613127, 9, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521936628613128, 9, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303680, 5, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303681, 5, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303682, 5, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303683, 5, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303684, 5, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303685, 5, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303686, 5, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303687, 5, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303688, 5, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303689, 5, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303690, 5, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303691, 5, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303692, 5, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303693, 5, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303694, 5, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303695, 5, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303696, 5, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303697, 5, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303698, 5, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303699, 5, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303700, 5, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303701, 5, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303702, 5, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303703, 5, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303704, 5, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303705, 5, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303706, 5, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303707, 5, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303708, 5, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303709, 5, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303710, 5, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303711, 5, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303712, 5, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303713, 5, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303714, 5, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303715, 5, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303716, 5, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303717, 5, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303718, 5, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303719, 5, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303720, 5, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303721, 5, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303722, 5, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303723, 5, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303724, 5, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303725, 5, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303726, 5, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303727, 5, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303728, 5, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303729, 5, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303730, 5, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303731, 5, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303732, 5, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303733, 5, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303734, 5, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303735, 5, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303736, 5, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303737, 5, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303738, 5, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303739, 5, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303740, 5, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303741, 5, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303742, 5, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303743, 5, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303744, 5, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303745, 5, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303746, 5, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303747, 5, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303748, 5, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303749, 5, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303750, 5, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303751, 5, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303752, 5, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303753, 5, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303754, 5, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303755, 5, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303756, 5, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303757, 5, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303758, 5, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303759, 5, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303760, 5, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303761, 5, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303762, 5, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303763, 5, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303764, 5, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303765, 5, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303766, 5, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303767, 5, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303768, 5, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303769, 5, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303770, 5, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303771, 5, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303772, 5, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303773, 5, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303774, 5, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303775, 5, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303776, 5, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303777, 5, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303778, 5, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303779, 5, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303780, 5, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303781, 5, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303782, 5, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303783, 5, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303784, 5, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303785, 5, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303786, 5, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303787, 5, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303788, 5, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303789, 5, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303790, 5, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303791, 5, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528521950847303792, 5, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804160, 3, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804161, 3, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804162, 3, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804163, 3, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804164, 3, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804165, 3, 3, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804166, 3, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804167, 3, 3076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804168, 3, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804169, 3, 3077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804170, 3, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804171, 3, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804172, 3, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804173, 3, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804174, 3, 3078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804175, 3, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804176, 3, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804177, 3, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804178, 3, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804179, 3, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804180, 3, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804181, 3, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804182, 3, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804183, 3, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804184, 3, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804185, 3, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804186, 3, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804187, 3, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804188, 3, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804189, 3, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804190, 3, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804191, 3, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804192, 3, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804193, 3, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804194, 3, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804195, 3, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804196, 3, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804197, 3, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804198, 3, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804199, 3, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804200, 3, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804201, 3, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804202, 3, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804203, 3, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804204, 3, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804205, 3, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804206, 3, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804207, 3, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804208, 3, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804209, 3, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804210, 3, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804211, 3, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804212, 3, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804213, 3, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804214, 3, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804215, 3, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804216, 3, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804217, 3, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804218, 3, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804219, 3, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804220, 3, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804221, 3, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804222, 3, 43, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804223, 3, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804224, 3, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804225, 3, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804226, 3, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804227, 3, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804228, 3, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804229, 3, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804230, 3, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804231, 3, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804232, 3, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804233, 3, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804234, 3, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804235, 3, 1073, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804236, 3, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804237, 3, 1074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804238, 3, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804239, 3, 1075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804240, 3, 52, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804241, 3, 1076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804242, 3, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804243, 3, 1077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804244, 3, 54, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804245, 3, 1078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804246, 3, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804247, 3, 1079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804248, 3, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804249, 3, 1080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804250, 3, 57, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804251, 3, 1081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804252, 3, 1082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804253, 3, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804254, 3, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804255, 3, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804256, 3, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804257, 3, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804258, 3, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804259, 3, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804260, 3, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804261, 3, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804262, 3, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804263, 3, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804264, 3, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804265, 3, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804266, 3, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804267, 3, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804268, 3, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804269, 3, 74, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804270, 3, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804271, 3, 75, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804272, 3, 1099, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804273, 3, 76, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804274, 3, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804275, 3, 1101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804276, 3, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804277, 3, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804278, 3, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804279, 3, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804280, 3, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804281, 3, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804282, 3, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804283, 3, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804284, 3, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804285, 3, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804286, 3, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804287, 3, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804288, 3, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804289, 3, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804290, 3, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804291, 3, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804292, 3, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804293, 3, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804294, 3, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804295, 3, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804296, 3, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804297, 3, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804298, 3, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804299, 3, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804300, 3, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804301, 3, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804302, 3, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804303, 3, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804304, 3, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804305, 3, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804306, 3, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804307, 3, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804308, 3, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804309, 3, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804310, 3, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804311, 3, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804312, 3, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804313, 3, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804314, 3, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804315, 3, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804316, 3, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804317, 3, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804318, 3, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804319, 3, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804320, 3, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804321, 3, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804322, 3, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804323, 3, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804324, 3, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804325, 3, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804326, 3, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804327, 3, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804328, 3, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804329, 3, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804330, 3, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804331, 3, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804332, 3, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804333, 3, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804334, 3, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804335, 3, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804336, 3, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804337, 3, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804338, 3, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804339, 3, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804340, 3, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804341, 3, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804342, 3, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804343, 3, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804344, 3, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804345, 3, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804346, 3, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804347, 3, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804348, 3, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804349, 3, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804350, 3, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804351, 3, 188, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804352, 3, 189, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804353, 3, 192, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804354, 3, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804355, 3, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804356, 3, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804357, 3, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804358, 3, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804359, 3, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804360, 3, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804361, 3, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804362, 3, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804363, 3, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804364, 3, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804365, 3, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804366, 3, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804367, 3, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804368, 3, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804369, 3, 209, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804370, 3, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804371, 3, 214, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804372, 3, 218, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804373, 3, 219, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804374, 3, 221, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804375, 3, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804376, 3, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804377, 3, 230, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804378, 3, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804379, 3, 232, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804380, 3, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804381, 3, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804382, 3, 235, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804383, 3, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804384, 3, 237, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804385, 3, 238, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804386, 3, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804387, 3, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804388, 3, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804389, 3, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804390, 3, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804391, 3, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804392, 3, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804393, 3, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804394, 3, 262, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804395, 3, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804396, 3, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804397, 3, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804398, 3, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804399, 3, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804400, 3, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804401, 3, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804402, 3, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804403, 3, 2001, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804404, 3, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804405, 3, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804406, 3, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804407, 3, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804408, 3, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804409, 3, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804410, 3, 2008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804411, 3, 2009, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804412, 3, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804413, 3, 2010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804414, 3, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804415, 3, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804416, 3, 2012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804417, 3, 2013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804418, 3, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804419, 3, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804420, 3, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804421, 3, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804422, 3, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804423, 3, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804424, 3, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804425, 3, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804426, 3, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804427, 3, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804428, 3, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804429, 3, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804430, 3, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804431, 3, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804432, 3, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804433, 3, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804434, 3, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804435, 3, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804436, 3, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804437, 3, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804438, 3, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804439, 3, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804440, 3, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804441, 3, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804442, 3, 2032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804443, 3, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804444, 3, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804445, 3, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804446, 3, 2035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804447, 3, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804448, 3, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804449, 3, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804450, 3, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804451, 3, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804452, 3, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804453, 3, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804454, 3, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804455, 3, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804456, 3, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804457, 3, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804458, 3, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804459, 3, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804460, 3, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804461, 3, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804462, 3, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804463, 3, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804464, 3, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804465, 3, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804466, 3, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804467, 3, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804468, 3, 2047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804469, 3, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (528522026323804470, 3, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555840, 4, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555841, 4, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555842, 4, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555843, 4, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555844, 4, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555845, 4, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555846, 4, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555847, 4, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555848, 4, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555849, 4, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555850, 4, 3079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555851, 4, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555852, 4, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555853, 4, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555854, 4, 3080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555855, 4, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555856, 4, 3081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555857, 4, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555858, 4, 3082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555859, 4, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555860, 4, 3083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555861, 4, 3084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555862, 4, 2061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555863, 4, 3085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555864, 4, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555865, 4, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555866, 4, 3086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555867, 4, 3087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555868, 4, 3088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555869, 4, 3089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555870, 4, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555871, 4, 3090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555872, 4, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555873, 4, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555874, 4, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555875, 4, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555876, 4, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555877, 4, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555878, 4, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555879, 4, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555880, 4, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555881, 4, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555882, 4, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555883, 4, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555884, 4, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555885, 4, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555886, 4, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555887, 4, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555888, 4, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555889, 4, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555890, 4, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555891, 4, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555892, 4, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555893, 4, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555894, 4, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555895, 4, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555896, 4, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555897, 4, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555898, 4, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555899, 4, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555900, 4, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555901, 4, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555902, 4, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555903, 4, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555904, 4, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555905, 4, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555906, 4, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555907, 4, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555908, 4, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555909, 4, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555910, 4, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555911, 4, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555912, 4, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555913, 4, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555914, 4, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555915, 4, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555916, 4, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555917, 4, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555918, 4, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555919, 4, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555920, 4, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555921, 4, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555922, 4, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555923, 4, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555924, 4, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555925, 4, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555926, 4, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555927, 4, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555928, 4, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555929, 4, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555930, 4, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555931, 4, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555932, 4, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555933, 4, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555934, 4, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555935, 4, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555936, 4, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555937, 4, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555938, 4, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555939, 4, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555940, 4, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555941, 4, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555942, 4, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555943, 4, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555944, 4, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555945, 4, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555946, 4, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555947, 4, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555948, 4, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555949, 4, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555950, 4, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555951, 4, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555952, 4, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555953, 4, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555954, 4, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555955, 4, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555956, 4, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555957, 4, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555958, 4, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555959, 4, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555960, 4, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555961, 4, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555962, 4, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555963, 4, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555964, 4, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555965, 4, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555966, 4, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555967, 4, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555968, 4, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555969, 4, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555970, 4, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555971, 4, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555972, 4, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555973, 4, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555974, 4, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555975, 4, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555976, 4, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555977, 4, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555978, 4, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555979, 4, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555980, 4, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555981, 4, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555982, 4, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555983, 4, 209, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555984, 4, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555985, 4, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555986, 4, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555987, 4, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555988, 4, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555989, 4, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555990, 4, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555991, 4, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555992, 4, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555993, 4, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555994, 4, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555995, 4, 4000, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555996, 4, 4010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555997, 4, 4011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555998, 4, 4012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675555999, 4, 4013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556000, 4, 4014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556001, 4, 4015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556002, 4, 4016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556003, 4, 4017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556004, 4, 4018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556005, 4, 4019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556006, 4, 4020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556007, 4, 4021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556008, 4, 4022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556009, 4, 4023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556010, 4, 4024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556011, 4, 4025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556012, 4, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556013, 4, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556014, 4, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556015, 4, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556016, 4, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556017, 4, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556018, 4, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556019, 4, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556020, 4, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556021, 4, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556022, 4, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556023, 4, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556024, 4, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556025, 4, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556026, 4, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556027, 4, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556028, 4, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556029, 4, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556030, 4, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556031, 4, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556032, 4, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556033, 4, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556034, 4, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556035, 4, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556036, 4, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556037, 4, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556038, 4, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556039, 4, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556040, 4, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556041, 4, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556042, 4, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556043, 4, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556044, 4, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556045, 4, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556046, 4, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556047, 4, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556048, 4, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556049, 4, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556050, 4, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556051, 4, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556052, 4, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556053, 4, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556054, 4, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556055, 4, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556056, 4, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556057, 4, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556058, 4, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556059, 4, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556060, 4, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556061, 4, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556062, 4, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556063, 4, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556064, 4, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556065, 4, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556066, 4, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530403599675556067, 4, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889920, 6, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889921, 6, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889922, 6, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889923, 6, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889924, 6, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889925, 6, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889926, 6, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889927, 6, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889928, 6, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889929, 6, 3094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889930, 6, 3095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889931, 6, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889932, 6, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889933, 6, 26, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889934, 6, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889935, 6, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889936, 6, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889937, 6, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889938, 6, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889939, 6, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889940, 6, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889941, 6, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889942, 6, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889943, 6, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889944, 6, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206839889945, 6, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084224, 6, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084225, 6, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084226, 6, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084227, 6, 4018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084228, 6, 4019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084229, 6, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084230, 6, 4020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084231, 6, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084232, 6, 4021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084233, 6, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084234, 6, 4022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084235, 6, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084236, 6, 4023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084237, 6, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084238, 6, 4024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084239, 6, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084240, 6, 4025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084241, 6, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084242, 6, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084243, 6, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084244, 6, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084245, 6, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084246, 6, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084247, 6, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084248, 6, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084249, 6, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084250, 6, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084251, 6, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084252, 6, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084253, 6, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084254, 6, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084255, 6, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084256, 6, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084257, 6, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084258, 6, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084259, 6, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084260, 6, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084261, 6, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084262, 6, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084263, 6, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084264, 6, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084265, 6, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084266, 6, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084267, 6, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084268, 6, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084269, 6, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084270, 6, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084271, 6, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084272, 6, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084273, 6, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084274, 6, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084275, 6, 2044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084276, 6, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084277, 6, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084278, 6, 2045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084279, 6, 2046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084280, 6, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (530780206844084281, 6, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814272, 2, 3072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814273, 2, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814274, 2, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814275, 2, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814276, 2, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814277, 2, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814278, 2, 3074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814279, 2, 3, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814280, 2, 2051, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814281, 2, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814282, 2, 3075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814283, 2, 3076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814284, 2, 4, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814285, 2, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814286, 2, 3077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814287, 2, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814288, 2, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814289, 2, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814290, 2, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814291, 2, 3078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814292, 2, 3079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814293, 2, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814294, 2, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814295, 2, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814296, 2, 3080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814297, 2, 2056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814298, 2, 1032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814299, 2, 2057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814300, 2, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814301, 2, 3081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814302, 2, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814303, 2, 3082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814304, 2, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814305, 2, 11, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814306, 2, 3083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814307, 2, 12, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814308, 2, 3084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814309, 2, 2060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814310, 2, 1036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814311, 2, 13, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814312, 2, 2061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814313, 2, 3085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814314, 2, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814315, 2, 14, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814316, 2, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814317, 2, 3086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814318, 2, 1038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814319, 2, 15, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814320, 2, 3087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814321, 2, 16, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814322, 2, 3088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814323, 2, 3089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814324, 2, 17, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814325, 2, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814326, 2, 3090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814327, 2, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814328, 2, 18, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814329, 2, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814330, 2, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814331, 2, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814332, 2, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814333, 2, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814334, 2, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814335, 2, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814336, 2, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814337, 2, 1045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814338, 2, 3094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814339, 2, 1046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814340, 2, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814341, 2, 1047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814342, 2, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814343, 2, 1048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814344, 2, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814345, 2, 1049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814346, 2, 1050, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814347, 2, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814348, 2, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814349, 2, 1052, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814350, 2, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814351, 2, 1053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814352, 2, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814353, 2, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814354, 2, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814355, 2, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814356, 2, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814357, 2, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814358, 2, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814359, 2, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814360, 2, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814361, 2, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814362, 2, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814363, 2, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814364, 2, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814365, 2, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814366, 2, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814367, 2, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814368, 2, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814369, 2, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814370, 2, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814371, 2, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814372, 2, 52, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814373, 2, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814374, 2, 54, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814375, 2, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814376, 2, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814377, 2, 57, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814378, 2, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814379, 2, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814380, 2, 74, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814381, 2, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814382, 2, 75, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814383, 2, 76, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814384, 2, 77, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814385, 2, 78, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814386, 2, 79, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814387, 2, 80, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814388, 2, 81, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814389, 2, 82, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814390, 2, 83, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814391, 2, 84, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814392, 2, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814393, 2, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814394, 2, 87, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814395, 2, 88, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814396, 2, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814397, 2, 90, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814398, 2, 91, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814399, 2, 92, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814400, 2, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814401, 2, 94, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814402, 2, 95, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814403, 2, 96, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814404, 2, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814405, 2, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814406, 2, 99, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814407, 2, 100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814408, 2, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814409, 2, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814410, 2, 103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814411, 2, 104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814412, 2, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814413, 2, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814414, 2, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814415, 2, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814416, 2, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814417, 2, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814418, 2, 111, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814419, 2, 112, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814420, 2, 113, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814421, 2, 114, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814422, 2, 115, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814423, 2, 116, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814424, 2, 117, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814425, 2, 118, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814426, 2, 119, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814427, 2, 120, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814428, 2, 121, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814429, 2, 122, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814430, 2, 123, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814431, 2, 124, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814432, 2, 125, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814433, 2, 126, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814434, 2, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814435, 2, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814436, 2, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814437, 2, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814438, 2, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814439, 2, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814440, 2, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814441, 2, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814442, 2, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814443, 2, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814444, 2, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814445, 2, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814446, 2, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814447, 2, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814448, 2, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814449, 2, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814450, 2, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814451, 2, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814452, 2, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814453, 2, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814454, 2, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814455, 2, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814456, 2, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814457, 2, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814458, 2, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814459, 2, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814460, 2, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814461, 2, 154, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814462, 2, 155, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814463, 2, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814464, 2, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814465, 2, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814466, 2, 159, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814467, 2, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814468, 2, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814469, 2, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814470, 2, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814471, 2, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814472, 2, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814473, 2, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814474, 2, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814475, 2, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814476, 2, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814477, 2, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814478, 2, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814479, 2, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814480, 2, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814481, 2, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814482, 2, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814483, 2, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814484, 2, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814485, 2, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814486, 2, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814487, 2, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814488, 2, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814489, 2, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814490, 2, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814491, 2, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814492, 2, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814493, 2, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814494, 2, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814495, 2, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814496, 2, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814497, 2, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814498, 2, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814499, 2, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814500, 2, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814501, 2, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814502, 2, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814503, 2, 209, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814504, 2, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814505, 2, 211, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814506, 2, 212, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814507, 2, 213, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814508, 2, 214, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814509, 2, 215, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814510, 2, 216, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814511, 2, 217, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814512, 2, 218, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814513, 2, 219, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814514, 2, 220, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244005814515, 2, 221, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008576, 2, 222, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008577, 2, 223, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008578, 2, 224, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008579, 2, 225, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008580, 2, 226, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008581, 2, 227, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008582, 2, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008583, 2, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008584, 2, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008585, 2, 232, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008586, 2, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008587, 2, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008588, 2, 235, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008589, 2, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008590, 2, 237, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008591, 2, 238, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008592, 2, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008593, 2, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008594, 2, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008595, 2, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008596, 2, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008597, 2, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008598, 2, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008599, 2, 248, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008600, 2, 249, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008601, 2, 250, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008602, 2, 252, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008603, 2, 253, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008604, 2, 254, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008605, 2, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008606, 2, 257, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008607, 2, 258, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008608, 2, 259, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008609, 2, 260, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008610, 2, 261, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008611, 2, 262, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008612, 2, 263, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008613, 2, 264, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008614, 2, 265, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008615, 2, 266, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008616, 2, 268, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008617, 2, 269, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008618, 2, 270, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008619, 2, 271, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008620, 2, 272, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008621, 2, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008622, 2, 276, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008623, 2, 277, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008624, 2, 278, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008625, 2, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008626, 2, 282, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008627, 2, 283, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008628, 2, 284, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008629, 2, 288, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008630, 2, 289, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008631, 2, 290, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008632, 2, 300, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008633, 2, 4000, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008634, 2, 4010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008635, 2, 4011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008636, 2, 4012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008637, 2, 4013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008638, 2, 4014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008639, 2, 4015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008640, 2, 4016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008641, 2, 4017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008642, 2, 4018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008643, 2, 4019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008644, 2, 4020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008645, 2, 4021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008646, 2, 4022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008647, 2, 4023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008648, 2, 4024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008649, 2, 4025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008650, 2, 3016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008651, 2, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008652, 2, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008653, 2, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008654, 2, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008655, 2, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008656, 2, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008657, 2, 2000, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008658, 2, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008659, 2, 2001, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008660, 2, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008661, 2, 2002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008662, 2, 2003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008663, 2, 2004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008664, 2, 2005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008665, 2, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008666, 2, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008667, 2, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008668, 2, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008669, 2, 2009, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008670, 2, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008671, 2, 2010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008672, 2, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008673, 2, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008674, 2, 2012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008675, 2, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008676, 2, 2013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008677, 2, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008678, 2, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008679, 2, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008680, 2, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008681, 2, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008682, 2, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008683, 2, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008684, 2, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008685, 2, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008686, 2, 2019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008687, 2, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008688, 2, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008689, 2, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008690, 2, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008691, 2, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008692, 2, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008693, 2, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008694, 2, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008695, 2, 1002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008696, 2, 1003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008697, 2, 2027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008698, 2, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008699, 2, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008700, 2, 1005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008701, 2, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008702, 2, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008703, 2, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008704, 2, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008705, 2, 3056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008706, 2, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008707, 2, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008708, 2, 3058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008709, 2, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008710, 2, 2035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008711, 2, 3060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008712, 2, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008713, 2, 3061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008714, 2, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008715, 2, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008716, 2, 3062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008717, 2, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008718, 2, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008719, 2, 3063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008720, 2, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008721, 2, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008722, 2, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008723, 2, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008724, 2, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008725, 2, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008726, 2, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008727, 2, 3065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008728, 2, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008729, 2, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008730, 2, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008731, 2, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008732, 2, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008733, 2, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008734, 2, 3068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008735, 2, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008736, 2, 3069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008737, 2, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008738, 2, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008739, 2, 2047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008740, 2, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531500244010008741, 2, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749888, 7, 2062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749889, 7, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749890, 7, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749891, 7, 3091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749892, 7, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749893, 7, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749894, 7, 3092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749895, 7, 3093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749896, 7, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749897, 7, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749898, 7, 26, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749899, 7, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749900, 7, 4000, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749901, 7, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749902, 7, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749903, 7, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749904, 7, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749905, 7, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749906, 7, 4010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749907, 7, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749908, 7, 4011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749909, 7, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749910, 7, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749911, 7, 4012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749912, 7, 4013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749913, 7, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749914, 7, 4014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749915, 7, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749916, 7, 4015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749917, 7, 4016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749918, 7, 4017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749919, 7, 4018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749920, 7, 4019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749921, 7, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749922, 7, 4020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749923, 7, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749924, 7, 4021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749925, 7, 4022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749926, 7, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749927, 7, 4023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749928, 7, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749929, 7, 4024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749930, 7, 4025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749931, 7, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749932, 7, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749933, 7, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749934, 7, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749935, 7, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749936, 7, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749937, 7, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749938, 7, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749939, 7, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749940, 7, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749941, 7, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749942, 7, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749943, 7, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749944, 7, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749945, 7, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749946, 7, 2044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749947, 7, 2045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (531510733347749948, 7, 2046, 1);

INSERT INTO `sys_user` (`id`, `login_name`, `real_name`, `code`, `password`, `mobile_number`, `org_id`, `gender`, `enable`, `pwd_count`, `pwd_update_time`, `remark`, `source`, `supplier_id`, `rtzf_sync_status`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, 'sysadmin', 'sysadmin', NULL, 'cW10aDIwMjQpKCo=', '0123456789*', NULL, NULL, 1, 1, 1629353956868, NULL, 'SYSTEM', NULL, NULL, NULL, NULL, 1, 1699606957631);

INSERT INTO `sys_user_role` (`id`, `user_id`, `role_id`, `privilege_id`, `enable`) VALUES (1, 1, 1, NULL, 1);

INSERT INTO `t_b_product` (`id`, `name`, `code`, `enable`, `create_id`, `create_time`) VALUES (2, '云阅卷', 'CLOUD_MARK', 1, 1, NULL);

INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (1, 'TEXT', 'FORM', 'problem_point', 'problem_point', '问题简要', 'STRING', 1, 0, 1, 1, NULL, 30, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (2, 'TEXTAREA', 'FORM', 'problem_remark', 'problem_remark', '问题情况说明', 'STRING', 1, 0, 1, 1, NULL, 300, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (3, 'FILE', 'FORM', 'attachments_info', 'attachments_info', '附件说明', 'FILE', 0, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (4, 'TEXT', 'FORM', 'verify_point', 'verify_point', '核实情况简要', 'STRING', 1, 0, 1, 1, NULL, 50, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (5, 'TEXTAREA', 'FORM', 'verify_remark', 'verify_remark', '核实情况备注', 'STRING', 1, 0, 1, 1, NULL, 500, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (6, 'RADIO', 'FORM', 'problem_type', 'problem_type', '质量问题类型', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"UPDATE\",\"label\":\"修正类\"},{\"value\":\"OPTIMIZE\",\"label\":\"优化类\"},{\"value\":\"NO_PROBLEM\",\"label\":\"不是问题\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (7, 'RADIO', 'FORM', 'problem_reason_radio', 'problem_reason_radio', '问题归因', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"EXEC\",\"label\":\"执行类\"},{\"value\":\"MANAGER\",\"label\":\"管理协调类\"},{\"value\":\"PRODUCT_PROBLEM\",\"label\":\"产品缺陷类\"},{\"value\":\"PRODUCT_MANAGER\",\"label\":\"产品运维类\"},{\"value\":\"FLOW\",\"label\":\"流程制度类\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (8, 'RADIO', 'FORM', 'problem_degree_radio', 'problem_degree_radio', '影响度', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"A\",\"label\":\"A级\"},{\"value\":\"B\",\"label\":\"B级\"},{\"value\":\"C\",\"label\":\"C级\"},{\"value\":\"D\",\"label\":\"D级\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (9, 'MULTIPLE_SELECT', 'FORM', 'problem_users', 'problem_users', '责任人', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=ALL', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (10, 'MULTIPLE_SELECT', 'FORM', 'approve_check_users', 'approve_check_users', '甲方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (11, 'MULTIPLE_SELECT', 'FORM', 'approve_check_third_users', 'approve_check_third_users', '乙方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (12, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (13, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (14, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (15, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (16, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (17, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (18, 'RADIO', 'FORM', 'service_scope_radio', 'service_scope_radio', '服务范围', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"SCAN\",\"label\":\"仅扫描\"},{\"value\":\"SCAN_MARK\",\"label\":\"扫描+阅卷\"}]', 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (19, 'RADIO', 'FORM', 'deploy_way_radio', 'deploy_way_radio', '部署方式', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"ONLINE\",\"label\":\"线上部署\"},{\"value\":\"LOCAL\",\"label\":\"本地部署\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (20, 'RADIO', 'FORM', 'mark_way_radio', 'mark_way_radio', '阅卷方式', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"TOGETHER\",\"label\":\"集中阅卷\"},{\"value\":\"ANYTIME\",\"label\":\"分散阅卷\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (21, 'RADIO', 'FORM', 'scan_net_radio', 'scan_net_radio', '扫描场地网络情况', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"INTERNET\",\"label\":\"提供外网\"},{\"value\":\"LOCALHOST\",\"label\":\"不提供外网\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (22, 'TEXT', 'FORM', 'contacts', 'contacts', '联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (23, 'TEXT', 'FORM', 'mobile_number', 'mobile_number', '电话', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (24, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (25, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (26, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '阅卷参数检查时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (27, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '阅卷收尾检查时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (28, 'MULTIPLE_SELECT', 'FORM', 'engineer_users_id', 'engineer_users_id', '工程师', 'ARRAY', 1, 1, 0, 1, NULL, NULL, NULL, '/api/admin/user/archives/allocation/can_choose?crmDetailId=#{id}&sopRoleType=ENGINEER', NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 11);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (29, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (30, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (31, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (32, 'NUMBER', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (33, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (34, 'FILE', 'FORM', 'scan_set', 'scan_set', '扫描仪设置', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (35, 'FORM_GROUP_TITLE', 'FORM', 'scan_test_title', 'scan_test_title', '试扫检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (36, 'FILE', 'FORM', 'scan_test', 'scan_test', '试扫检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (37, 'FORM_GROUP_TITLE', 'FORM', 'scan_end_title', 'scan_end_title', '扫描收尾检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (38, 'FILE', 'FORM', 'scan_end', 'scan_end', '上传检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (39, 'FORM_GROUP_TITLE', 'FORM', 'not_upload_confirm_title', 'not_upload_confirm_title', '未上传确认', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (40, 'FILE', 'FORM', 'not_upload_confirm', 'not_upload_confirm', '未上传确认', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (41, 'FORM_GROUP_TITLE', 'FORM', 'identify_title', 'identify_title', '识别对照', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (42, 'FILE', 'FORM', 'identify', 'identify', '识别对照', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (43, 'FORM_GROUP_TITLE', 'FORM', 'absent_title', 'absent_title', '缺考识别检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (44, 'FILE', 'FORM', 'absent', 'absent', '缺考识别检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (45, 'FORM_GROUP_TITLE', 'FORM', 'manual_confirm_title', 'manual_confirm_title', '人工确认检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (46, 'FILE', 'FORM', 'manual_confirm', 'manual_confirm', '人工确认检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (47, 'FORM_GROUP_TITLE', 'FORM', 'mark_paper_title', 'mark_paper_title', '阅卷参数检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (48, 'FILE', 'FORM', 'mark_paper', 'mark_paper', '阅卷参数检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (49, 'FORM_GROUP_TITLE', 'FORM', 'mark_paper_progress_title', 'mark_paper_progress_title', '阅卷进度检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (50, 'FILE', 'FORM', 'mark_paper_progress', 'mark_paper_progress', '阅卷进度检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (51, 'FORM_GROUP_TITLE', 'FORM', 'objective_check_title', 'objective_check_title', '实考客观题0分检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (52, 'FILE', 'FORM', 'objective_check', 'objective_check', '实考客观题0分检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (53, 'FORM_GROUP_TITLE', 'FORM', 'subjective_check_title', 'subjective_check_title', '实考主观题0分，客观题有分', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (54, 'FILE', 'FORM', 'subjective_check', 'subjective_check', '实考主观题0分，客观题有分', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (92, 'FORM_GROUP_TITLE', 'FORM', 'project_basic_info_title', 'project_basic_info_title', '项目基本信息', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (93, 'NUMBER', 'FORM', 'course_count', 'course_count', '科目数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (94, 'NUMBER', 'FORM', 'exam_student_course_count', 'exam_student_course_count', '考生科次', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (95, 'RADIO', 'FORM', 'bar_code_paper_count', 'bar_code_paper_count', '条码标准张数', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"3\",\"label\":\"3张\"},{\"value\":\"4\",\"label\":\"4张\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (96, 'RADIO_WITH_INPUT', 'FORM', 'bar_code_paste_way', 'bar_code_paste_way', '条码粘贴方式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"EXAM_STUDENT\",\"label\":\"考生自贴/数码印刷\"},{\"value\":\"PAPER\",\"label\":\"回卷后粘贴\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (97, 'RADIO', 'FORM', 'has_objective_card', 'has_objective_card', '有无客观题卡', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"HAVA\",\"label\":\"有\"},{\"value\":\"NOT_HAVA\",\"label\":\"无\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (98, 'NUMBER', 'FORM', 'scan_count', 'scan_count', '扫描仪数量(台)', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (99, 'DATE', 'FORM', 'exam_student_Internal_date', 'exam_student_Internal_date', '考生数据内部交接时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (100, 'NUMBER', 'FORM', 'week_people_day', 'week_people_day', '现场标准服务周期（人天）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (101, 'DATE', 'FORM', 'project_key_info_date', 'project_key_info_date', '项目关键信息提交截止时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (102, 'DATE', 'FORM', 'service_finish_plan_date', 'service_finish_plan_date', '现场服务完成撤场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (103, 'FORM_GROUP_TITLE', 'FORM', 'project_people_plan_title', 'project_people_plan_title', '项目人员安排及风险预估', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 13);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (104, 'SINGLE_SELECT', 'FORM', 'region_user_id', 'region_user_id', '区域协调人', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 14);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (105, 'SINGLE_SELECT', 'FORM', 'engineer_user_id', 'engineer_user_id', '实施工程师', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 15);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (106, 'MULTIPLE_SELECT', 'FORM', 'assistant_engineer_user_id', 'assistant_engineer_user_id', '助理实施工程师', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 16);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (107, 'ONLY_TITLE', 'FORM', 'project_rish_title', 'project_rish_title', '项目风险预估（仅供参考）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 17);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (108, 'RADIO', 'FORM', 'delay_rish_cb', 'delay_rish_cb', '延期风险', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 18);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (109, 'RADIO', 'FORM', 'engineer_rish_cb', 'engineer_rish_cb', '实施难度', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 19);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (110, 'TEXTAREA', 'FORM', 'other_remark', 'other_remark', '其它备注（建议关注的其它方面）', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 20);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (111, 'FORM_GROUP_TITLE', 'FORM', 'project_contacts_title', 'project_contacts_title', '项目联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 21);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (112, 'TABLE', 'FORM', 'project_contacts_table', 'project_contacts_table', '项目联系人表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 22);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (113, 'TEXT', 'TABLE', 'school_college', 'school_college', '项目联系人表格_机构输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 23);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (114, 'TEXT', 'TABLE', 'school_name', 'school_name', '项目联系人表格_姓名输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 24);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (115, 'TEXT', 'TABLE', 'school_job', 'school_job', '项目联系人表格_职务输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 25);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (116, 'TEXT', 'TABLE', 'school_tel', 'school_tel', '项目联系人表格_电话输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 26);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (117, 'FORM_GROUP_TITLE', 'FORM', 'cloud_mark_title', 'cloud_mark_title', '云阅卷（填写前请先认真查阅移交的项目初审及项目基本信息）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (118, 'RADIO', 'FORM', 'run_env_accord', 'run_env_accord', '运行环境调研是否符合要求', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"ACCORD\",\"label\":\"符合\"},{\"value\":\"UN_ACCORD\",\"label\":\"不符合\"}]', 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (119, 'DATE', 'FORM', 'env_deployment_time', 'env_deployment_time', '环境部署时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (120, 'LABEL', 'FORM', 'env_list_label', 'env_list_label', '[{\"label\":\"参照《软硬件准备检查清单》对学校进行调研\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (121, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (122, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (123, 'LABEL', 'FORM', 'unite_label', 'unite_label', '[{\"label\":\"统一元旦后安排进场\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (124, 'DATE', 'FORM', 'mark_room_check_time', 'mark_room_check_time', '评卷机房检查时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (125, 'TEXT', 'FORM', 'mark_room_count', 'mark_room_count', '评卷机房数量（个）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (126, 'TEXT', 'FORM', 'max_mark_teacher_count', 'max_mark_teacher_count', '最大评卷教师数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (127, 'DATE', 'FORM', 'mark_param_time', 'mark_param_time', '评卷参数提供时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (128, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '评卷开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (129, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '评卷结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (130, 'TEXT', 'FORM', 'choose_course_count', 'choose_course_count', '有选做题的科目数量（门）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (131, 'TEXTAREA', 'FORM', 'ramark', 'ramark', '其它特殊要求', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (133, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_region', 'inside_title_region', '大区经理项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (134, 'RADIO', 'FORM', 'approve_radio_region', 'approve_radio_region', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (135, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (136, 'TEXTAREA', 'FORM', 'approve_ramark_region', 'approve_ramark_region', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (137, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_engineer', 'inside_title_engineer', '工程师项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (138, 'RADIO', 'FORM', 'approve_radio_engineer', 'approve_radio_engineer', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (139, 'SIGN', 'FORM', 'sign_engineer', 'sign_engineer', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (140, 'TEXTAREA', 'FORM', 'approve_ramark_engineer', 'approve_ramark_engineer', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (141, 'FORM_GROUP_TITLE', 'FORM', 'upload_list_title', 'upload_list_title', '上传清单和报告', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (144, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备出库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (145, 'DATE', 'FORM', 'device_out_time', 'device_out_time', '设备出库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (146, 'DEVICE_OUT_TABLE', 'FORM', 'device_out_table', 'device_out_table', '设备出库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (147, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (148, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (149, 'NUMBER', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (150, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (151, 'CHECKBOX', 'FORM', 'scan_set_cb', 'scan_set_cb', '扫描仪设置', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、扫描仪分辨率设置是150\"},{\"value\":\"2\",\"label\":\"2、是否勾选“自动剪切”，“纠正歪斜”\"},{\"value\":\"3\",\"label\":\"3、是否勾选“边界去除”\"},{\"value\":\"4\",\"label\":\"4、勾选“双张进纸检测”-“有效”-动作“停止”\"},{\"value\":\"5\",\"label\":\"5、爱普生型号，选择“扫描模式”-“设置”-“装订边缘”-“上”\"},{\"value\":\"6\",\"label\":\"6、华高扫描仪型号，检查背面图像；如背面图像倒转，则勾选“背面旋转180°”\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (152, 'FORM_GROUP_TITLE', 'FORM', 'card_format_title', 'card_format_title', '卡格式制作', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (153, 'CHECKBOX', 'FORM', 'card_format_cb', 'card_format_cb', '卡格式制作', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、定位线：上方紧挨装钉“线”字，但是不超过“线”字；考生信息遮挡区需按要求框选\"},{\"value\":\"2\",\"label\":\"2、扫描原图正反面检查是否正确\"},{\"value\":\"3\",\"label\":\"3、扫描裁切图屏蔽区检查是否正确\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (154, 'FORM_GROUP_TITLE', 'FORM', 'paper_list_title', 'paper_list_title', '纸质检查表上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (155, 'FILE', 'FORM', 'paper_check_photos', 'paper_check_photos', '上传关键节点检查(纸质)表单拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (156, 'FORM_GROUP_TITLE', 'FORM', 'manual_bar_code_title', 'manual_bar_code_title', '手工输入条码确认', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (157, 'CHECKBOX', 'FORM', 'manual_bar_code_cb', 'manual_bar_code_cb', '手工输入条码确认', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"1\",\"label\":\"“数据检查”-“人工确认”，”确认已完成”\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (158, 'NUMBER', 'FORM', 'complete_count', 'complete_count', '完成数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (159, 'FORM_GROUP_TITLE', 'FORM', 'bar_code_full_title', 'bar_code_full_title', '条码数量完整性检查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (160, 'CHECKBOX', 'FORM', 'bar_code_full_cb', 'bar_code_full_cb', '条码数量完整性检查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"“缺页异常”查询需无数据“\"},{\"value\":\"2\",\"label\":\"“无条码考生”查询需无数据“\"},{\"value\":\"3\",\"label\":\"“追加题卡”需和学校记录一致“\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (161, 'FORM_GROUP_TITLE', 'FORM', 'missing_list_title', 'missing_list_title', '缺考名单比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (162, 'CHECKBOX', 'FORM', 'missing_list_cb', 'missing_list_cb', '缺考名单比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"核对“无答题卡考生”的考生是否与学校提供的缺考名单吻合，人工指定为缺考（缺考一致性>导入缺考>导入缺考缺页）；需核对平台“考生管理”中状态为“未上传”的考生是否全部为缺考、是否全部变为“人工指定”。\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (163, 'NUMBER', 'FORM', 'missing_course_count', 'missing_course_count', '缺考科次数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (164, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (165, 'RADIO', 'FORM', 'school_sign_radio', 'school_sign_radio', '学校是否确认签字', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"是\"},{\"value\":\"NO\",\"label\":\"否\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (166, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (167, 'FORM_GROUP_TITLE', 'FORM', 'mark_param_check_title', 'mark_param_check_title', '评卷参数核对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (168, 'CHECKBOX', 'FORM', 'mark_param_check_cb', 'mark_param_check_cb', '评卷参数核对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核查每个科目的满分是否正确(100分、150分、300分)\"},{\"value\":\"2\",\"label\":\"2、检查所有科目结构和分组是否全部导入。（评卷管理>评卷进度查看分组状态）\"}]', 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (169, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (170, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (171, 'FORM_GROUP_TITLE', 'FORM', 'score_check_title', 'score_check_title', '分数复核', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (172, 'CHECKBOX', 'FORM', 'score_check_cb', 'score_check_cb', '分数复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校复核小题0分，大题0分，全卷0分的情况\"}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (173, 'RADIO', 'FORM', 'school_score_check_radio', 'school_score_check_radio', '学校是否选择复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"复核(推荐)\"},{\"value\":\"NO\",\"label\":\"不复核\",\"remark\":[{\"content\":\"若选择该项，则需要学校签字确认，签字随关键节点检查表上传\",\"color\":\"red\"}]}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (175, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (176, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (177, 'FORM_GROUP_TITLE', 'FORM', 'mark_progress_title', 'mark_progress_title', '评卷进度核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (178, 'CHECKBOX', 'FORM', 'mark_progress_cb', 'mark_progress_cb', '评卷进度核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对阅卷总进度是否为100%\"},{\"value\":\"2\",\"label\":\"2、核对科目阅卷进度是否为100%\"},{\"value\":\"3\",\"label\":\"3、核对问题卷是否处理完成\"},{\"value\":\"4\",\"label\":\"4、核对所有科目状态点击“结束”\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (179, 'FORM_GROUP_TITLE', 'FORM', 'cross_check_title', 'cross_check_title', '交叉比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (180, 'CHECKBOX', 'FORM', 'cross_check_cb', 'cross_check_cb', '交叉比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校进行同一考生多科目分数交叉比对（成绩复核-成绩校验），需所有科目全部改完后管理员账号才能查看。\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (181, 'FORM_GROUP_TITLE', 'FORM', 'mark_submit_title', 'mark_submit_title', '成绩提交核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (182, 'CHECKBOX', 'FORM', 'mark_submit_cb', 'mark_submit_cb', '成绩提交核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对导出成绩表数据条数是否与考生表数量一致，不能带有“未导出全量数据”字样\"},{\"value\":\"2\",\"label\":\"2、导出验收报告（考试管理-详情-验收报告），和学校签字确认盖章，上报验收报告至质控平台\"},{\"value\":\"3\",\"label\":\"3、核对导出图片工具是否正常运行\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (183, 'NUMBER', 'FORM', 'export_score_count', 'export_score_count', '导出成绩数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (184, 'FILE', 'FORM', 'upload_check_report_photos', 'upload_check_report_photos', '上传验收报告(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (185, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备入库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (186, 'DATE', 'FORM', 'device_in_time', 'device_in_time', '设备入库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (187, 'DEVICE_IN_TABLE', 'FORM', 'device_in_table', 'device_in_table', '设备入库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (188, 'FILE', 'FORM', 'upload_device_version_photos', 'upload_device_version_photos', '上传扫描仪固件版本号拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (190, 'DATE', 'FORM', 'service_finish_plan_begin_date', 'service_finish_plan_begin_date', '现场服务完成进场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 11);

INSERT INTO `t_d_table_prop` (`id`, `widget_id`, `td_index`, `td_id`, `td_name`, `title`, `width`, `height`, `color`, `rowspan`, `colspan`, `sub_title`, `tips`, `td_order`, `td_search`, `edit_widget_id`, `obj`, `create_id`, `create_time`) VALUES (5, 112, 1, 'college', 'college', '学院/分（子）机构', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 113, NULL, 1, 1);
INSERT INTO `t_d_table_prop` (`id`, `widget_id`, `td_index`, `td_id`, `td_name`, `title`, `width`, `height`, `color`, `rowspan`, `colspan`, `sub_title`, `tips`, `td_order`, `td_search`, `edit_widget_id`, `obj`, `create_id`, `create_time`) VALUES (6, 112, 2, 'name', 'name', '姓名', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 114, NULL, 1, 1);
INSERT INTO `t_d_table_prop` (`id`, `widget_id`, `td_index`, `td_id`, `td_name`, `title`, `width`, `height`, `color`, `rowspan`, `colspan`, `sub_title`, `tips`, `td_order`, `td_search`, `edit_widget_id`, `obj`, `create_id`, `create_time`) VALUES (7, 112, 3, 'job', 'job', '职务', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 115, NULL, 1, 1);
INSERT INTO `t_d_table_prop` (`id`, `widget_id`, `td_index`, `td_id`, `td_name`, `title`, `width`, `height`, `color`, `rowspan`, `colspan`, `sub_title`, `tips`, `td_order`, `td_search`, `edit_widget_id`, `obj`, `create_id`, `create_time`) VALUES (8, 112, 4, 'mobileNumber', 'mobileNumber', '电话', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 116, NULL, 1, 1);

INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (6, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (7, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (8, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (9, NULL, 'thread.pool.core.size', '自定义线程池大小', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (10, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (11, NULL, 'session.active', 'session会话时长', '6h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (12, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (13, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (14, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (15, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, 1700209205546);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (16, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (17, NULL, 'sms.sign.name', '短信签名', '启明泰和', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (18, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (19, NULL, 'flow.code.length', '流程编码长度', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (20, NULL, 'flow.job.before.time', '流程提前提醒任务时间（单位：分）', '720', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (21, NULL, 'flow.job.after.time', '流程超时提醒任务时间（单位：分）', '1440', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (22, NULL, 'notice.job.after.time', '公告超时提醒任务时间（单位：分）', '30', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (23, NULL, 'remind.job.poll.time', '任务提醒轮询时间', '0 0/5 * * * ?', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (24, NULL, 'remind.job.db.limit', '任务提醒查询条数', 'limit 0,100', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (25, NULL, 'sms.notice.code', '系统通知公告', 'SMS_462685325', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (26, NULL, 'sms.sop.before.remind.code', '教务处/研究生SOP提前提醒通知', 'SMS_462805277', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (27, NULL, 'sms.sop.done.code', '教务处/研究生SOP节点负责人通知', 'SMS_462730333', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (28, NULL, 'sms.flow.project.exchange.finish.code', '系统计划变更审核通知', 'SMS_462680680', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (29, NULL, 'sms.flow.ding.exception.finish.code', '异常审核通知', 'SMS_462700329', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (30, NULL, 'sms.sop.after.remind.code', '延期提醒通知', 'SMS_462700328', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (31, NULL, 'sms.violation.remind.code', '违规提醒通知', 'SMS_462805303', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (32, NULL, 'baidu.face.compare.url', '百度人脸识别人脸比对接口', '/face-api/v3/face/match', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (33, NULL, 'baidu.face.compare.score', '百度人脸识别人脸比对阈值', '70', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (34, NULL, 'fxxk.app.auth.url', '纷享销客应用级授权接口', 'https://open.fxiaoke.com/cgi/corpAccessToken/get/V2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (35, NULL, 'fxxk.user.by.mobile.url', '纷享销客根据手机号查询员工接口', 'https://open.fxiaoke.com/cgi/user/getByMobile', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (36, NULL, 'fxxk.custom.query.url', '纷享销客查询自定义对象接口', 'https://open.fxiaoke.com/cgi/crm/custom/v2/data/query', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (37, NULL, 'fxxk.user.query.url', '纷享销客查询人员对象的列表', 'https://open.fxiaoke.com/cgi/crm/v2/data/query', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (38, NULL, 'rtzf.oss.login.url', '软通智服单点登录接口', 'https://qmth.echo-isoftstone.com/login', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (39, NULL, 'sms.custom.info.missing.remind.code', '客户信息缺失通知', 'SMS_467015006', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` (`id`, `org_id`, `config_key`, `config_name`, `config_value`, `remark`, `enable`, `sort`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (40, NULL, 'sop.quick.count', 'sop快捷条数', '5', NULL, 1, 1, 1, NULL, NULL, NULL);