USE sop_db;

-- ----------------------------
-- Table structure for basic_attachment
-- ----------------------------
DROP TABLE IF EXISTS `basic_attachment`;
CREATE TABLE `basic_attachment` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                    `name` varchar(100) NOT NULL COMMENT '文件名',
                                    `type` varchar(10) NOT NULL COMMENT '文件类型',
                                    `size` decimal(8,2) NOT NULL COMMENT '单位（KB）',
                                    `md5` varchar(32) NOT NULL COMMENT '文件md5值',
                                    `path` varchar(2000) NOT NULL COMMENT '存储路径',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    `obj_id` bigint DEFAULT NULL COMMENT '关联业务id',
                                    `remark` varchar(500) DEFAULT NULL,
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='所有附件记录表';

-- ----------------------------
-- Table structure for basic_school
-- ----------------------------
DROP TABLE IF EXISTS `basic_school`;
CREATE TABLE `basic_school` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `code` varchar(20) NOT NULL COMMENT '学校编码',
                                `domain_name` varchar(20) DEFAULT NULL COMMENT '域名',
                                `name` varchar(100) DEFAULT NULL COMMENT '学校名称',
                                `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                                `access_key` varchar(50) DEFAULT NULL COMMENT '访问key',
                                `access_secret` varchar(50) DEFAULT NULL COMMENT '访问秘钥',
                                `remark` text COMMENT '备注',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                `logo` mediumtext COMMENT '学校logo',
                                PRIMARY KEY (`id`) USING BTREE,
                                UNIQUE KEY `code` (`code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='学校表';

-- ----------------------------
-- Table structure for basic_sms_message
-- ----------------------------
DROP TABLE IF EXISTS `basic_sms_message`;
CREATE TABLE `basic_sms_message` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint NOT NULL COMMENT '消息接收人用户',
                                     `user_name` varchar(100) DEFAULT NULL COMMENT '用户名称',
                                     `mobile_number` varchar(11) DEFAULT NULL COMMENT '电话号码',
                                     `message_type` varchar(45) DEFAULT NULL COMMENT '消息类型',
                                     `business_operate` varchar(45) NOT NULL COMMENT '业务操作',
                                     `template_code` varchar(45) NOT NULL COMMENT '消息模板代码',
                                     `variable_params` text COMMENT '变量参数内容',
                                     `template_content` mediumtext COMMENT '模板内容',
                                     `send_status` varchar(45) DEFAULT NULL COMMENT '消息发送状态',
                                     `send_result` varchar(200) DEFAULT NULL COMMENT '消息发送结果',
                                     `remark` text COMMENT '备注',
                                     `resend_count` int DEFAULT NULL COMMENT '重试次数',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信通知表';

-- ----------------------------
-- Table structure for basic_verify_code
-- ----------------------------
DROP TABLE IF EXISTS `basic_verify_code`;
CREATE TABLE `basic_verify_code` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                     `user_id` bigint NOT NULL COMMENT '用户ID',
                                     `mobile_number` varchar(11) NOT NULL COMMENT '手机号',
                                     `expire_time` bigint NOT NULL COMMENT '到期时间',
                                     `valid_period` int NOT NULL COMMENT '单位（分）',
                                     `verify_code` varchar(4) NOT NULL COMMENT '4位数字',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='短信验证码记录表';

-- ----------------------------
-- Table structure for qrtz_blob_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_blob_triggers`;
CREATE TABLE `qrtz_blob_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `BLOB_DATA` blob,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                      KEY `SCHED_NAME` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_calendars
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_calendars`;
CREATE TABLE `qrtz_calendars` (
                                  `SCHED_NAME` varchar(120) NOT NULL,
                                  `CALENDAR_NAME` varchar(200) NOT NULL,
                                  `CALENDAR` blob NOT NULL,
                                  PRIMARY KEY (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_cron_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_cron_triggers`;
CREATE TABLE `qrtz_cron_triggers` (
                                      `SCHED_NAME` varchar(120) NOT NULL,
                                      `TRIGGER_NAME` varchar(200) NOT NULL,
                                      `TRIGGER_GROUP` varchar(200) NOT NULL,
                                      `CRON_EXPRESSION` varchar(120) NOT NULL,
                                      `TIME_ZONE_ID` varchar(80) DEFAULT NULL,
                                      PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_fired_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_fired_triggers`;
CREATE TABLE `qrtz_fired_triggers` (
                                       `SCHED_NAME` varchar(120) NOT NULL,
                                       `ENTRY_ID` varchar(95) NOT NULL,
                                       `TRIGGER_NAME` varchar(200) NOT NULL,
                                       `TRIGGER_GROUP` varchar(200) NOT NULL,
                                       `INSTANCE_NAME` varchar(200) NOT NULL,
                                       `FIRED_TIME` bigint NOT NULL,
                                       `SCHED_TIME` bigint NOT NULL,
                                       `PRIORITY` int NOT NULL,
                                       `STATE` varchar(16) NOT NULL,
                                       `JOB_NAME` varchar(200) DEFAULT NULL,
                                       `JOB_GROUP` varchar(200) DEFAULT NULL,
                                       `IS_NONCONCURRENT` varchar(1) DEFAULT NULL,
                                       `REQUESTS_RECOVERY` varchar(1) DEFAULT NULL,
                                       PRIMARY KEY (`SCHED_NAME`,`ENTRY_ID`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TRIG_INST_NAME` (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_INST_JOB_REQ_RCVRY` (`SCHED_NAME`,`INSTANCE_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_J_G` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_T_G` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                       KEY `IDX_QRTZ_FT_TG` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_job_details
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_job_details`;
CREATE TABLE `qrtz_job_details` (
                                    `SCHED_NAME` varchar(120) NOT NULL,
                                    `JOB_NAME` varchar(200) NOT NULL,
                                    `JOB_GROUP` varchar(200) NOT NULL,
                                    `DESCRIPTION` varchar(250) DEFAULT NULL,
                                    `JOB_CLASS_NAME` varchar(250) NOT NULL,
                                    `IS_DURABLE` varchar(1) NOT NULL,
                                    `IS_NONCONCURRENT` varchar(1) NOT NULL,
                                    `IS_UPDATE_DATA` varchar(1) NOT NULL,
                                    `REQUESTS_RECOVERY` varchar(1) NOT NULL,
                                    `JOB_DATA` blob,
                                    PRIMARY KEY (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                    KEY `IDX_QRTZ_J_REQ_RECOVERY` (`SCHED_NAME`,`REQUESTS_RECOVERY`) USING BTREE,
                                    KEY `IDX_QRTZ_J_GRP` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_locks
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_locks`;
CREATE TABLE `qrtz_locks` (
                              `SCHED_NAME` varchar(120) NOT NULL,
                              `LOCK_NAME` varchar(40) NOT NULL,
                              PRIMARY KEY (`SCHED_NAME`,`LOCK_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_paused_trigger_grps
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_paused_trigger_grps`;
CREATE TABLE `qrtz_paused_trigger_grps` (
                                            `SCHED_NAME` varchar(120) NOT NULL,
                                            `TRIGGER_GROUP` varchar(200) NOT NULL,
                                            PRIMARY KEY (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_scheduler_state
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_scheduler_state`;
CREATE TABLE `qrtz_scheduler_state` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `INSTANCE_NAME` varchar(200) NOT NULL,
                                        `LAST_CHECKIN_TIME` bigint NOT NULL,
                                        `CHECKIN_INTERVAL` bigint NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`INSTANCE_NAME`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simple_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simple_triggers`;
CREATE TABLE `qrtz_simple_triggers` (
                                        `SCHED_NAME` varchar(120) NOT NULL,
                                        `TRIGGER_NAME` varchar(200) NOT NULL,
                                        `TRIGGER_GROUP` varchar(200) NOT NULL,
                                        `REPEAT_COUNT` bigint NOT NULL,
                                        `REPEAT_INTERVAL` bigint NOT NULL,
                                        `TIMES_TRIGGERED` bigint NOT NULL,
                                        PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_simprop_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_simprop_triggers`;
CREATE TABLE `qrtz_simprop_triggers` (
                                         `SCHED_NAME` varchar(120) NOT NULL,
                                         `TRIGGER_NAME` varchar(200) NOT NULL,
                                         `TRIGGER_GROUP` varchar(200) NOT NULL,
                                         `STR_PROP_1` varchar(512) DEFAULT NULL,
                                         `STR_PROP_2` varchar(512) DEFAULT NULL,
                                         `STR_PROP_3` varchar(512) DEFAULT NULL,
                                         `INT_PROP_1` int DEFAULT NULL,
                                         `INT_PROP_2` int DEFAULT NULL,
                                         `LONG_PROP_1` bigint DEFAULT NULL,
                                         `LONG_PROP_2` bigint DEFAULT NULL,
                                         `DEC_PROP_1` decimal(13,4) DEFAULT NULL,
                                         `DEC_PROP_2` decimal(13,4) DEFAULT NULL,
                                         `BOOL_PROP_1` varchar(1) DEFAULT NULL,
                                         `BOOL_PROP_2` varchar(1) DEFAULT NULL,
                                         PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for qrtz_triggers
-- ----------------------------
DROP TABLE IF EXISTS `qrtz_triggers`;
CREATE TABLE `qrtz_triggers` (
                                 `SCHED_NAME` varchar(120) NOT NULL,
                                 `TRIGGER_NAME` varchar(200) NOT NULL,
                                 `TRIGGER_GROUP` varchar(200) NOT NULL,
                                 `JOB_NAME` varchar(200) NOT NULL,
                                 `JOB_GROUP` varchar(200) NOT NULL,
                                 `DESCRIPTION` varchar(250) DEFAULT NULL,
                                 `NEXT_FIRE_TIME` bigint DEFAULT NULL,
                                 `PREV_FIRE_TIME` bigint DEFAULT NULL,
                                 `PRIORITY` int DEFAULT NULL,
                                 `TRIGGER_STATE` varchar(16) NOT NULL,
                                 `TRIGGER_TYPE` varchar(8) NOT NULL,
                                 `START_TIME` bigint NOT NULL,
                                 `END_TIME` bigint DEFAULT NULL,
                                 `CALENDAR_NAME` varchar(200) DEFAULT NULL,
                                 `MISFIRE_INSTR` smallint DEFAULT NULL,
                                 `JOB_DATA` blob,
                                 PRIMARY KEY (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_J` (`SCHED_NAME`,`JOB_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_JG` (`SCHED_NAME`,`JOB_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_C` (`SCHED_NAME`,`CALENDAR_NAME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_G` (`SCHED_NAME`,`TRIGGER_GROUP`) USING BTREE,
                                 KEY `IDX_QRTZ_T_STATE` (`SCHED_NAME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_STATE` (`SCHED_NAME`,`TRIGGER_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_N_G_STATE` (`SCHED_NAME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NEXT_FIRE_TIME` (`SCHED_NAME`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST` (`SCHED_NAME`,`TRIGGER_STATE`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_STATE`) USING BTREE,
                                 KEY `IDX_QRTZ_T_NFT_ST_MISFIRE_GRP` (`SCHED_NAME`,`MISFIRE_INSTR`,`NEXT_FIRE_TIME`,`TRIGGER_GROUP`,`TRIGGER_STATE`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4;

-- ----------------------------
-- Table structure for sequence
-- ----------------------------
DROP TABLE IF EXISTS `sequence`;
CREATE TABLE `sequence` (
                            `name` varchar(50) NOT NULL COMMENT '名称',
                            `current_value` bigint NOT NULL DEFAULT '0' COMMENT '当前值',
                            `increment` int NOT NULL DEFAULT '1' COMMENT '增长值',
                            PRIMARY KEY (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='序列表';

-- ----------------------------
-- Table structure for sys_config
-- ----------------------------
DROP TABLE IF EXISTS `sys_config`;
CREATE TABLE `sys_config` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `org_id` bigint DEFAULT NULL COMMENT '机构id',
                              `config_key` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '参数键名',
                              `config_name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '参数名称',
                              `config_value` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '参数键值',
                              `remark` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '备注',
                              `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `sort` int DEFAULT '1' COMMENT '排序',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='参数表';

-- ----------------------------
-- Table structure for sys_custom
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom`;
CREATE TABLE `sys_custom` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '名称',
                              `code` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '编码',
                              `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                              `province` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '省份',
                              `city` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '城市',
                              `area` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '县区',
                              `address` varchar(150) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '地址',
                              `level_id` bigint DEFAULT NULL COMMENT '档位id',
                              `peoper_day` int DEFAULT NULL COMMENT '标准人天',
                              `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户表';

-- ----------------------------
-- Table structure for sys_custom_addr
-- ----------------------------
DROP TABLE IF EXISTS `sys_custom_addr`;
CREATE TABLE `sys_custom_addr` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `custom_id` bigint NOT NULL COMMENT '客户ID',
                                   `province` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '省份',
                                   `city` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '城市',
                                   `area` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '县区',
                                   `address` varchar(150) CHARACTER SET utf8mb4 NOT NULL COMMENT '地址',
                                   `longitude` varchar(255) CHARACTER SET utf8mb4 NOT NULL COMMENT '经度',
                                   `latitude` varchar(255) CHARACTER SET utf8mb4 NOT NULL COMMENT '纬度',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='客户地址表';

-- ----------------------------
-- Table structure for sys_device
-- ----------------------------
DROP TABLE IF EXISTS `sys_device`;
CREATE TABLE `sys_device` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `device_code` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '设备编号',
                              `serial_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '序列号',
                              `brand` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '品牌',
                              `model` varchar(255) CHARACTER SET utf8mb4 NOT NULL COMMENT '型号',
                              `buy_time` bigint DEFAULT NULL COMMENT '购买时间',
                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                              `status` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                              `location` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '当前所在地',
                              `location_arr` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '当前所在地（回显）',
                              `scan_count` bigint NOT NULL DEFAULT '0' COMMENT '总扫描量',
                              `bound` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '出库/入库',
                              `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`),
                              UNIQUE KEY `sys_device_un_serial_no` (`serial_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备配置表';

-- ----------------------------
-- Table structure for sys_device_brand
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_brand`;
CREATE TABLE `sys_device_brand` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `brand` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '品牌',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`),
                                    UNIQUE KEY `sys_device_brand_un_brand` (`brand`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌表';

-- ----------------------------
-- Table structure for sys_device_model
-- ----------------------------
DROP TABLE IF EXISTS `sys_device_model`;
CREATE TABLE `sys_device_model` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `brand_id` bigint NOT NULL COMMENT '品牌ID',
                                    `model` varchar(255) CHARACTER SET utf8mb4 NOT NULL COMMENT '型号',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    PRIMARY KEY (`id`),
                                    UNIQUE KEY `sys_device_model_un` (`brand_id`,`model`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备品牌型号表';

-- ----------------------------
-- Table structure for sys_ding_date
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_date`;
CREATE TABLE `sys_ding_date` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `year` bigint NOT NULL COMMENT '年',
                                 `month` bigint NOT NULL COMMENT '月',
                                 `day` bigint NOT NULL COMMENT '日',
                                 `type` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                 PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤特殊日期';

-- ----------------------------
-- Table structure for sys_ding_group
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_group`;
CREATE TABLE `sys_ding_group` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `service_id` bigint NOT NULL COMMENT '服务单元id',
                                  `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '名称',
                                  `supplier_id` bigint NOT NULL COMMENT '供应商id',
                                  `sign_in_time` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '签到时段',
                                  `sign_out_time` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '签退时段',
                                  `face_open` tinyint NOT NULL DEFAULT '1' COMMENT '是否开启人脸，0：不开启，1：开启',
                                  `reissue_card_count` int DEFAULT NULL COMMENT '允许补卡次数',
                                  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组表';

-- ----------------------------
-- Table structure for sys_ding_obj
-- ----------------------------
DROP TABLE IF EXISTS `sys_ding_obj`;
CREATE TABLE `sys_ding_obj` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `ding_group_id` bigint NOT NULL COMMENT '考勤组id',
                                `role_id` bigint NOT NULL COMMENT '角色id',
                                `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，DING：考勤，APPROVE：审核',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤组对象表';

-- ----------------------------
-- Table structure for sys_job
-- ----------------------------
DROP TABLE IF EXISTS `sys_job`;
CREATE TABLE `sys_job` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                           `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                           `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                           `user_id` bigint NOT NULL COMMENT '责任人id',
                           `job_time` bigint NOT NULL COMMENT '任务时间',
                           `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，BEFORE：任务前提醒，AFTER：任务后提醒',
                           `unit` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '单位，HOUR：小时，MINUTE：分钟',
                           `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，NOT_EXEC：未执行，EXEC：已执行',
                           `exec_time` bigint NOT NULL COMMENT '执行时间',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统任务表';

-- ----------------------------
-- Table structure for sys_level
-- ----------------------------
DROP TABLE IF EXISTS `sys_level`;
CREATE TABLE `sys_level` (
                             `id` bigint NOT NULL COMMENT '主键',
                             `level` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '档位',
                             `devices` bigint DEFAULT NULL COMMENT '设备数量',
                             `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                             `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                             `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='档位表';

-- ----------------------------
-- Table structure for sys_level_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_level_role`;
CREATE TABLE `sys_level_role` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `level_id` bigint NOT NULL COMMENT '服务档位id',
                                  `role_id` bigint NOT NULL COMMENT '角色id',
                                  `quota` bigint NOT NULL COMMENT '配额',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务档位角色配置表';

-- ----------------------------
-- Table structure for sys_log
-- ----------------------------
DROP TABLE IF EXISTS `sys_log`;
CREATE TABLE `sys_log` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `user_id` bigint NOT NULL COMMENT '用户id',
                           `user_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '用户名称',
                           `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类别，LOGIN：登录，LOGOUT：退出以及数据库DDL和DML操作',
                           `obj_name` varchar(1000) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体名称',
                           `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                           `content` varchar(200) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '内容',
                           `url` varchar(64) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '请求的url',
                           `args` longtext CHARACTER SET utf8mb4 COMMENT '请求的参数',
                           `update_before_obj` longtext CHARACTER SET utf8mb4 COMMENT '更改前内容',
                           `update_after_obj` longtext CHARACTER SET utf8mb4 COMMENT '更改后内容',
                           `create_time` bigint NOT NULL COMMENT '创建时间',
                           `run_time` bigint DEFAULT NULL COMMENT '请求的响应时间',
                           `run_status` varchar(64) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '请求的结果',
                           PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统日志';

-- ----------------------------
-- Table structure for sys_message
-- ----------------------------
DROP TABLE IF EXISTS `sys_message`;
CREATE TABLE `sys_message` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                               `code` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '编号',
                               `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '消息类型，BEFORE：提前提醒，AFTER：延期提醒，OFFICE_SOP：教务处SOP，CLOUD_MARK_SOP：研究生SOP，QUALITY：质量问题提醒，EXCEPTION_APPROVE：异常审核提醒，VIOLATION：违规提醒，SYSTEM_PLAN_CHANGE：系统计划变更提醒，SYSTEM：系统公告',
                               `obj_type` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体业务类型',
                               `notice_id` bigint DEFAULT NULL COMMENT '公告id',
                               `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                               `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                               `form_user_id` bigint DEFAULT NULL COMMENT '发送人id',
                               `receive_user_id` bigint NOT NULL COMMENT '接收人id',
                               `content` varchar(10000) CHARACTER SET utf8mb4 NOT NULL COMMENT '消息内容',
                               `send_time` bigint NOT NULL COMMENT '发送时间',
                               `receive_time` bigint DEFAULT NULL COMMENT '接收时间',
                               `status` tinyint NOT NULL DEFAULT '0' COMMENT '是否已阅，0：未阅，1：已阅',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='系统消息';

-- ----------------------------
-- Table structure for sys_notice
-- ----------------------------
DROP TABLE IF EXISTS `sys_notice`;
CREATE TABLE `sys_notice` (
                              `id` bigint NOT NULL COMMENT '主键',
                              `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                              `title` varchar(200) CHARACTER SET utf8mb4 NOT NULL COMMENT '标题',
                              `content` varchar(10000) CHARACTER SET utf8mb4 NOT NULL COMMENT '内容',
                              `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '类型，SYSTEM：公告，SUPPLIER：供应商，SERVICE：服务单元',
                              `status` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '状态，PUBLISH：已发布，UN_PUBLISH：未发布',
                              `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                              `publish_time` bigint DEFAULT NULL COMMENT '发布日期',
                              `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                              `create_time` bigint DEFAULT NULL COMMENT '创建人时间',
                              `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                              `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='公告表';

-- ----------------------------
-- Table structure for sys_org
-- ----------------------------
DROP TABLE IF EXISTS `sys_org`;
CREATE TABLE `sys_org` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `type` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '类型',
                           `code` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '机构代码',
                           `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '机构名称',
                           `parent_id` bigint DEFAULT NULL COMMENT '上级机构id',
                           `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint DEFAULT NULL COMMENT '创建人',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint DEFAULT NULL COMMENT '修改人',
                           `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                           PRIMARY KEY (`id`) USING BTREE,
                           UNIQUE KEY `sys_org_UN_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='组织架构表';

-- ----------------------------
-- Table structure for sys_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_privilege`;
CREATE TABLE `sys_privilege` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '菜单名称',
                                 `url` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'URL地址',
                                 `type` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，MENU：菜单，BUTTON：按钮，LINK：链接，URL：接口地址，LIST：列表，CONDITION：查询条件',
                                 `parent_id` bigint DEFAULT NULL COMMENT '上级菜单',
                                 `sequence` int NOT NULL COMMENT '序号',
                                 `property` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '属性，NO_AUTH：无需鉴权，AUTH：鉴权，SYS：系统公用',
                                 `related` text CHARACTER SET utf8mb4 COMMENT '关联属性，type为URL且property为AUTH才有',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 `default_auth` tinyint DEFAULT '1' COMMENT '默认权限，0：否，1：是',
                                 `front_display` tinyint(1) DEFAULT '1' COMMENT '是否前端展示',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='菜单权限表';

-- ----------------------------
-- Table structure for sys_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_role`;
CREATE TABLE `sys_role` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `name` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '角色名称',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '角色类别，ADMIN：超级管理员，PMO：总负责人，BUSSINESS：业务线负责人，REGION_MANAGER：大区经理，REGION_COORDINATOR：区域协调人，EFFECT_ENGINEER：实施工程师，ASSISTANT_ENGINEER：助理工程师，QA：QA，CUSTOM：技术客服',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                            `default_role` tinyint NOT NULL DEFAULT '0' COMMENT '是否系统内置角色，1：是，0：不是',
                            `interpret` varchar(1000) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '角色说明',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_role_UN_type` (`name`,`type`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色表';

-- ----------------------------
-- Table structure for sys_role_privilege
-- ----------------------------
DROP TABLE IF EXISTS `sys_role_privilege`;
CREATE TABLE `sys_role_privilege` (
                                      `id` bigint NOT NULL COMMENT '主键',
                                      `role_id` bigint NOT NULL COMMENT '角色id',
                                      `privilege_id` bigint NOT NULL COMMENT '菜单id',
                                      `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='角色和菜单关联表';

-- ----------------------------
-- Table structure for sys_supplier
-- ----------------------------
DROP TABLE IF EXISTS `sys_supplier`;
CREATE TABLE `sys_supplier` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '名称',
                                `code` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '编码',
                                `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，HUMAN：人力，DEVICE：设备',
                                `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                `remark` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '备注',
                                `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，0：否，1：是',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `sys_supplier_un_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='供应商表';

-- ----------------------------
-- Table structure for sys_user
-- ----------------------------
DROP TABLE IF EXISTS `sys_user`;
CREATE TABLE `sys_user` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `login_name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '用户名',
                            `real_name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '姓名',
                            `code` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '工号',
                            `password` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '密码',
                            `mobile_number` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '手机号',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `gender` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '性别，MAN：男，WOMAN：女',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                            `pwd_count` tinyint DEFAULT '0' COMMENT '密码修改次数，默认为0',
                            `pwd_update_time` bigint DEFAULT NULL COMMENT '密码修改时间',
                            `remark` varchar(200) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '备注',
                            `source` varchar(20) CHARACTER SET utf8mb4 DEFAULT 'SYSTEM' COMMENT '来源，SYSTEM：系统自带，ARCHIVES：档案',
                            `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                            `rtzf_sync_status` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '软通智服同步状态',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '修改人',
                            `update_time` bigint DEFAULT NULL COMMENT '修改时间',
                            PRIMARY KEY (`id`) USING BTREE,
                            UNIQUE KEY `sys_user_UN_login_name` (`login_name`),
                            UNIQUE KEY `sys_user_UN_moblie_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户表';

-- ----------------------------
-- Table structure for sys_user_role
-- ----------------------------
DROP TABLE IF EXISTS `sys_user_role`;
CREATE TABLE `sys_user_role` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `user_id` bigint NOT NULL COMMENT '用户id',
                                 `role_id` bigint NOT NULL COMMENT '角色id',
                                 `privilege_id` bigint DEFAULT NULL COMMENT '权限id',
                                 `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                 PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='用户和角色关联表';

-- ----------------------------
-- Table structure for t_b_crm
-- ----------------------------
DROP TABLE IF EXISTS `t_b_crm`;
CREATE TABLE `t_b_crm` (
                           `id` bigint NOT NULL COMMENT '主键',
                           `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '项目单号',
                           `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '项目名称',
                           `begin_time` bigint NOT NULL COMMENT '派单时间',
                           `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                           `crm_user_id` bigint NOT NULL COMMENT '派单人id',
                           `custom_id` bigint NOT NULL COMMENT '客户id',
                           `exam_start_time` bigint DEFAULT NULL COMMENT '考试开始时间',
                           `exam_end_time` bigint DEFAULT NULL COMMENT '考试结束时间',
                           `product_id` bigint NOT NULL COMMENT '产品id',
                           `lead_id` bigint DEFAULT NULL COMMENT '大区经理id',
                           `region_id` bigint DEFAULT NULL COMMENT '服务区域规划id',
                           `region_detail_id` bigint DEFAULT NULL COMMENT '服务区域规划明细id',
                           `region_coordinator_id` bigint DEFAULT NULL COMMENT '区域协调人id',
                           `sync` tinyint NOT NULL DEFAULT '0' COMMENT '是否crm同步，false：否，true：是',
                           `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                           `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                           `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                           `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                           `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                           `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '派单状态(UN_PUBLISH("未发布"),PUBLISH("已发布"),FINISH("已完成");)',
                           `publish_user_id` bigint DEFAULT NULL COMMENT '发布人id',
                           `publish_time` bigint DEFAULT NULL COMMENT '发布时间',
                           PRIMARY KEY (`id`),
                           UNIQUE KEY `t_b_crm_un_crm_no` (`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='派单信息表';

-- ----------------------------
-- Table structure for t_b_delay_warn
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn`;
CREATE TABLE `t_b_delay_warn` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                  `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                  `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                  `message_id` bigint DEFAULT NULL COMMENT '消息id',
                                  `code` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '编号',
                                  `warn_time` bigint DEFAULT NULL COMMENT '预警时间',
                                  `user_id` bigint DEFAULT NULL COMMENT '负责人id',
                                  `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，PLAN：关键信息及计划，TIME：处理时限',
                                  `field_obj` mediumtext CHARACTER SET utf8mb4 COMMENT '预警字段',
                                  `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                  `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                  `close_time` bigint DEFAULT NULL COMMENT '关闭时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_delay_warn_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警跟进表';

-- ----------------------------
-- Table structure for t_b_delay_warn_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_delay_warn_detail`;
CREATE TABLE `t_b_delay_warn_detail` (
                                         `id` bigint NOT NULL COMMENT '主键',
                                         `delay_warn_id` bigint NOT NULL COMMENT '延期预警id',
                                         `remark` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '跟进说明',
                                         `attachment_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件ids',
                                         `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='延期预警明细表';

-- ----------------------------
-- Table structure for t_b_device_delivery
-- ----------------------------
DROP TABLE IF EXISTS `t_b_device_delivery`;
CREATE TABLE `t_b_device_delivery` (
                                       `id` bigint NOT NULL COMMENT '主键',
                                       `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                       `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                       `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                       `serial_no` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '设备序列号',
                                       `usage_type` varchar(45) CHARACTER SET utf8mb4 NOT NULL COMMENT '用途类型，PROJECT：项目，OTHER：其它',
                                       `supplier_id` bigint DEFAULT NULL COMMENT '供应商id',
                                       `num` int DEFAULT NULL COMMENT '数量',
                                       `mailing_address` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '邮寄地址',
                                       `consignee` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '收件人',
                                       `consignee_phone` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '收件人联系电话',
                                       `express_no` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '快递单号',
                                       `express_photo_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '快递单拍照',
                                       `delivery_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '发货类型（出入库类型）',
                                       `status` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '状态，UN_DELIVER：待发货，DELIVER：已发货，RECEIVE：已签收，CANCEL：作废',
                                       `deliver_time` bigint DEFAULT NULL COMMENT '发货时间',
                                       `deliver_user_id` bigint DEFAULT NULL COMMENT '发货人id',
                                       `receive_time` bigint DEFAULT NULL COMMENT '签收时间',
                                       `receive_user_id` bigint DEFAULT NULL COMMENT '签收人id',
                                       `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                       `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                       `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                       `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                       `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                       `effect` tinyint NOT NULL COMMENT '有效的可操作的发货记录（当一个设备处于一个发货出入库生命周期时为有效）, 0: 无效，1: 有效',
                                       PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备发货表';

-- ----------------------------
-- Table structure for t_b_device_in_out
-- ----------------------------
DROP TABLE IF EXISTS `t_b_device_in_out`;
CREATE TABLE `t_b_device_in_out` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `serial_no` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '出入库流水号',
                                     `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                     `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                     `in_out_time` bigint NOT NULL COMMENT '出/入库时间',
                                     `user_id` bigint NOT NULL COMMENT '登记人id',
                                     `custom_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '客户名称',
                                     `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '类型，IN：入库，OUT：出库',
                                     `device_serial_no` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '设备序列号',
                                     `supplier_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '供应商名称',
                                     `location` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '当前所在地',
                                     `address` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '发往地',
                                     `address_arr` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '发往地（回显）',
                                     `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，NORMAL：正常，BREAK_DOWN：故障',
                                     `usage_type` varchar(45) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '用途类型（项目、其他）',
                                     `base_photo_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '快递单拍照',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建人时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_device_in_out_un_serial_no` (`serial_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='设备出入库登记';

-- ----------------------------
-- Table structure for t_b_ding
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding`;
CREATE TABLE `t_b_ding` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'sop单号',
                            `ding_sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '实际打卡对应的sop单号',
                            `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                            `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                            `user_id` bigint NOT NULL COMMENT '用户id',
                            `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                            `role_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '项目角色',
                            `flow_id` bigint DEFAULT NULL COMMENT '流程id',
                            `task_id` bigint DEFAULT NULL COMMENT '流程任务id',
                            `sign_year` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '签到年份',
                            `sign_date` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '签到日期',
                            `date_type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                            `sign_in_time` bigint DEFAULT NULL COMMENT '签到时间',
                            `sign_in_address` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '签到地址',
                            `axis_in_x` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴X',
                            `axis_in_y` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴Y',
                            `face_in_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `sign_out_time` bigint DEFAULT NULL COMMENT '签退时间',
                            `sign_out_address` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '签退地址',
                            `axis_out_x` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴X',
                            `axis_out_Y` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴Y',
                            `face_out_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            PRIMARY KEY (`id`),
                            UNIQUE KEY `t_b_ding_un` (`sop_no`,`user_id`,`sign_year`,`sign_date`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Table structure for t_b_ding_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_apply`;
CREATE TABLE `t_b_ding_apply` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'sop编号',
                                  `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '派单编号',
                                  `ding_id` bigint DEFAULT NULL COMMENT '考勤id',
                                  `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，IN：签到，OUT：签退',
                                  `apply_time` bigint NOT NULL COMMENT '补卡时间',
                                  `reason` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '补卡原因',
                                  `attachment_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件id数组',
                                  `attachment_paths` varchar(800) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件地址',
                                  `ding_exception_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '考勤异常编号',
                                  `approve` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '审批操作，PASS：通过，NO_PASS：不通过',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`),
                                  UNIQUE KEY `t_b_ding_apply_UN_ding_exception_no` (`ding_exception_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤补卡申请表';

-- ----------------------------
-- Table structure for t_b_ding_history
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_history`;
CREATE TABLE `t_b_ding_history` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `ding_id` bigint DEFAULT NULL COMMENT '考勤id',
                                    `sop_no` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop编号',
                                    `sign_date` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '签到日期',
                                    `date_type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '日期类型：WEEKDAY("工作日"),WEEKEND("周末"),LEGAL_HOLIDAYS("法定节假日")',
                                    `sign_time` bigint DEFAULT NULL COMMENT '签到时间',
                                    `axis_x` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴X',
                                    `axis_y` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '坐标轴Y',
                                    `face_open` tinyint DEFAULT NULL COMMENT '是否开启人脸识别',
                                    `face_pass` tinyint DEFAULT NULL COMMENT '人脸识别是否通过，0：未通过，1：通过',
                                    `face_photo_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '抓拍照片地址',
                                    `match_score` decimal(8,2) DEFAULT NULL COMMENT '对比分数',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤打卡表';

-- ----------------------------
-- Table structure for t_b_ding_submit
-- ----------------------------
DROP TABLE IF EXISTS `t_b_ding_submit`;
CREATE TABLE `t_b_ding_submit` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `service_id` bigint NOT NULL COMMENT '服务单元id',
                                   `sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'sop单号',
                                   `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'crm单号',
                                   `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                                   `approach_time` bigint DEFAULT NULL COMMENT '进场时间',
                                   `departure_time` bigint DEFAULT NULL COMMENT '出场时间',
                                   `role_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '项目角色类型',
                                   `actual_days` int DEFAULT NULL COMMENT '实际出勤（天）',
                                   `weekdays` int DEFAULT NULL COMMENT '工作日（天）',
                                   `weekends` int DEFAULT NULL COMMENT '周末（天）',
                                   `legal_holidays` int DEFAULT NULL COMMENT '法定节假日（天）',
                                   `work_hours` decimal(8,2) DEFAULT NULL COMMENT '累计工时（小时）',
                                   `violation_days` decimal(8,2) DEFAULT NULL COMMENT '违规工时',
                                   `submit_user_id` bigint NOT NULL COMMENT '提交人id',
                                   `submit_time` bigint NOT NULL COMMENT '提交时间',
                                   `submit_status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '提交状态',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                   `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                   PRIMARY KEY (`id`),
                                   UNIQUE KEY `t_b_ding_submit_un` (`sop_no`,`user_archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='考勤提交审核表';

-- ----------------------------
-- Table structure for t_b_product
-- ----------------------------
DROP TABLE IF EXISTS `t_b_product`;
CREATE TABLE `t_b_product` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '名称',
                               `code` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '编码，OFFICE：教务处，CLOUD_MARK：研究生',
                               `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_product_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='产品表';

-- ----------------------------
-- Table structure for t_b_project_exchange
-- ----------------------------
DROP TABLE IF EXISTS `t_b_project_exchange`;
CREATE TABLE `t_b_project_exchange` (
                                        `id` bigint NOT NULL COMMENT '主键',
                                        `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                        `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                        `exchange_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '变更编号',
                                        `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                        `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '变更类型，PLAN：项目信息及计划，CANCEL：项目取消',
                                        `reason` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '变更原因',
                                        `content_json` mediumtext CHARACTER SET utf8mb4 COMMENT '变更内容json',
                                        `result` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '处理结果，FINISH：已完成，NOT_UPDATE：经沟通，取消变更，PARTIAL_UPDATE：经沟通，部分变更',
                                        `remark` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '变更备注',
                                        `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`),
                                        UNIQUE KEY `t_b_project_exchange_un_exchange_no` (`exchange_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='项目变更计划表';

-- ----------------------------
-- Table structure for t_b_quality_problem_apply
-- ----------------------------
DROP TABLE IF EXISTS `t_b_quality_problem_apply`;
CREATE TABLE `t_b_quality_problem_apply` (
                                             `id` bigint NOT NULL COMMENT '主键',
                                             `sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'sop单号',
                                             `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'crm单号',
                                             `problem_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '问题编号',
                                             `service_id` bigint NOT NULL COMMENT '服务单元id',
                                             `summary` varchar(1000) CHARACTER SET utf8mb4 NOT NULL COMMENT '问题简要',
                                             `remark` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '问题情况说明',
                                             `user_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '责任人id数组',
                                             `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '问题类型，UPDATE：修正类，OPTIMIZE：优化类，NO_PROBLEM：不是问题',
                                             `reason` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '问题原因类型，EXEC：执行类，MANAGER：管理协调类，FLOW：流程制度类，PRODUCT_PROBLEM：产品缺陷类，PRODUCT_MANAGER：产品运维类，OTHER：其它',
                                             `attachment_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件id数组',
                                             `Influence_degree` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '影响度：A，B，C，D',
                                             `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                             `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                             `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                             PRIMARY KEY (`id`),
                                             UNIQUE KEY `t_b_quality_problem_apply_UN_probem_no` (`problem_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='质量问题反馈表';

-- ----------------------------
-- Table structure for t_b_service
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service`;
CREATE TABLE `t_b_service` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '名称',
                               `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，OFFICE：教务处，CLOUD_MARK：研究生',
                               `start_time` bigint NOT NULL COMMENT '开始时间',
                               `end_time` bigint NOT NULL COMMENT '结束时间',
                               `region_people_count` int NOT NULL COMMENT '区域协调人数量',
                               `region_project_count` int NOT NULL COMMENT '区域项目数量',
                               `service_lead_id` bigint NOT NULL COMMENT '服务单元负责人id',
                               `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                               `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，NEW：新建，PUBLISH：已发布，FINISH：已完结，CANCEL：已作废',
                               `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                               `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                               `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                               `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                               PRIMARY KEY (`id`),
                               UNIQUE KEY `t_b_service_un_name` (`name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元表';

-- ----------------------------
-- Table structure for t_b_service_region
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region`;
CREATE TABLE `t_b_service_region` (
                                      `id` bigint NOT NULL COMMENT '主键',
                                      `service_id` bigint NOT NULL COMMENT '服务单元id',
                                      `region_name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '大区名称',
                                      `lead_id` bigint NOT NULL COMMENT '大区经理id',
                                      `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                      `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                      `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                      `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                      `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                      PRIMARY KEY (`id`),
                                      UNIQUE KEY `t_b_service_region_un` (`service_id`,`region_name`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划表';

-- ----------------------------
-- Table structure for t_b_service_region_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_region_detail`;
CREATE TABLE `t_b_service_region_detail` (
                                             `id` bigint NOT NULL COMMENT '主键',
                                             `service_region_id` bigint NOT NULL COMMENT '服务区域规划id',
                                             `country` varchar(100) CHARACTER SET utf8mb4 NOT NULL DEFAULT '中国' COMMENT '国家',
                                             `province` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '省份',
                                             `city` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '城市',
                                             `area` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '县区',
                                             `address` varchar(150) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '地址',
                                             PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务区域规划明细表';

-- ----------------------------
-- Table structure for t_b_service_scope
-- ----------------------------
DROP TABLE IF EXISTS `t_b_service_scope`;
CREATE TABLE `t_b_service_scope` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `service_id` bigint NOT NULL COMMENT '服务单元id',
                                     `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'crm单号',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_service_scope_un` (`service_id`,`crm_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='服务单元范围管理表';

-- ----------------------------
-- Table structure for t_b_session
-- ----------------------------
DROP TABLE IF EXISTS `t_b_session`;
CREATE TABLE `t_b_session` (
                               `id` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '主键',
                               `org_id` bigint DEFAULT NULL COMMENT '机构id',
                               `identity` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '用户标识',
                               `type` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '用户类型',
                               `source` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '访问来源',
                               `platform` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '设备分类',
                               `device_id` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '设备标识',
                               `address` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '登录IP地址',
                               `access_token` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '访问令牌',
                               `last_access_time` bigint DEFAULT NULL COMMENT '最近访问时间',
                               `last_access_ip` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '最近访问IP地址',
                               `update_time` bigint DEFAULT NULL COMMENT '令牌更新时间',
                               `expire_time` bigint DEFAULT NULL COMMENT '令牌强制失效时间',
                               `app_source` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '应用来源',
                               PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='会话表';

-- ----------------------------
-- Table structure for t_b_sop_info
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_info`;
CREATE TABLE `t_b_sop_info` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'crm单号',
                                `sop_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'sop单号',
                                `service_id` bigint NOT NULL COMMENT '服务单元id',
                                `custom_id` bigint NOT NULL COMMENT '客户id',
                                `product_id` bigint NOT NULL COMMENT '产品id',
                                `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP',
                                `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，DRAFT：草稿，START：提交',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                PRIMARY KEY (`id`),
                                UNIQUE KEY `t_b_sop_info_un_sop_no` (`sop_no`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop表';

-- ----------------------------
-- Table structure for t_b_sop_log
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_log`;
CREATE TABLE `t_b_sop_log` (
                               `id` bigint(20) NOT NULL COMMENT '主键',
                               `sop_id` bigint(20) NOT NULL COMMENT 'sop主键',
                               `type` varchar(30) DEFAULT NULL COMMENT '类型，APPROVE：sop审核，EDIT：sop编辑，CHANGE_EFFECT_ENGINEER：更改实施工程师，CHANGE_ASSISTANT_ENGINEER：更改助理工程师，ADD_EFFECT_ENGINEER：新增实施工程师，ADD_ASSISTANT_ENGINEER：新增助理工程师，REMOVE_EFFECT_ENGINEER：删除实施工程师，REMOVE_ASSISTANT_ENGINEER：删除助理工程师',
                               `status` varchar(50) NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                               `setup` tinyint(4) NOT NULL COMMENT '计划步骤',
                               `update_before_obj` mediumtext COMMENT '更改前内容',
                               `update_after_obj` mediumtext COMMENT '更改后内容',
                               `create_id` bigint(20) NOT NULL COMMENT '创建人id',
                               `create_time` bigint(20) NOT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop日志';

-- ----------------------------
-- Table structure for t_b_sop_plan_date_log
-- ----------------------------
DROP TABLE IF EXISTS `t_b_sop_plan_date_log`;
CREATE TABLE `t_b_sop_plan_date_log` (
                                         `id` bigint NOT NULL COMMENT '主键',
                                         `sop_id` bigint DEFAULT NULL COMMENT 'sopId',
                                         `flow_id` bigint NOT NULL COMMENT '流程id',
                                         `begin_time` bigint NOT NULL COMMENT '计划开始时间',
                                         `end_time` bigint NOT NULL COMMENT '计划结束时间',
                                         `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                         `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                         `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                         `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                         PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop进程计划时间日志表';

-- ----------------------------
-- Table structure for t_b_task
-- ----------------------------
DROP TABLE IF EXISTS `t_b_task`;
CREATE TABLE `t_b_task` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `type` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '任务类型，USER_ARCHIVES_IMPORT：人员档案导入，USER_ARCHIVES_EXPORT：人员档案导出，CUSTOM_IMPORT：客户导入',
                            `entity_id` bigint DEFAULT NULL COMMENT '关联业务对象id',
                            `status` varchar(30) CHARACTER SET utf8mb4 NOT NULL COMMENT '任务状态，INIT：未开始，RUNNING：进行中，FINISH：已完成',
                            `summary` text CHARACTER SET utf8mb4 COMMENT '实时摘要信息',
                            `progress` double DEFAULT NULL COMMENT '执行进度',
                            `result` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '数据结果，SUCCESS：成功，ERROR：失败',
                            `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，false：停用，true：启用',
                            `import_file_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '导入文件名',
                            `import_file_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '导入文件路径',
                            `result_file_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '导出文件路径',
                            `report_file_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '报告路径',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `org_id` bigint DEFAULT NULL COMMENT '机构id',
                            `remark` mediumtext CHARACTER SET utf8mb4 COMMENT '备注',
                            `obj_name` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体名称',
                            `reset_count` int DEFAULT '0' COMMENT '重试次数',
                            `version` int DEFAULT '0' COMMENT '更新版本号',
                            `error_message` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '人工错误原因',
                            PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='导入导出任务表';

-- ----------------------------
-- Table structure for t_b_user_archives
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives`;
CREATE TABLE `t_b_user_archives` (
                                     `id` bigint NOT NULL COMMENT '主键',
                                     `code` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '编号',
                                     `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '姓名',
                                     `country` varchar(100) CHARACTER SET utf8mb4 NOT NULL DEFAULT '中国' COMMENT '国家',
                                     `province` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '省份',
                                     `city` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '城市',
                                     `area` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '县区',
                                     `address` varchar(150) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '地址',
                                     `identity` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '身份证号',
                                     `gender` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '性别，MAN：男，WOMAN：女',
                                     `education` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '学历',
                                     `mobile_number` varchar(30) CHARACTER SET utf8mb4 NOT NULL COMMENT '手机号',
                                     `email` varchar(30) CHARACTER SET utf8mb4 NOT NULL COMMENT '邮箱',
                                     `base_photo_path` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '底照保存地址',
                                     `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '人力资源状态（FREE("空闲"),NON_EXECUTION_SOP("未执行SOP"),EXECUTION_SOP("执行SOP");）',
                                     `enable` tinyint(1) NOT NULL DEFAULT '1' COMMENT '是否启用（1启用，0作废）',
                                     `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                     `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                     `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                     `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                     PRIMARY KEY (`id`),
                                     UNIQUE KEY `t_b_user_archives_un_identity` (`identity`),
                                     UNIQUE KEY `t_b_user_archives_un_mobile_number` (`mobile_number`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案表';

-- ----------------------------
-- Table structure for t_b_user_archives_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_allocation`;
CREATE TABLE `t_b_user_archives_allocation` (
                                                `id` bigint NOT NULL COMMENT '主键',
                                                `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                                `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                                `crm_no` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'crm单号',
                                                `region_user_id` bigint DEFAULT NULL COMMENT '区域协调人id',
                                                `user_id` bigint NOT NULL COMMENT '工程师id',
                                                `role_id` bigint NOT NULL COMMENT '角色id',
                                                `archives_id` bigint NOT NULL COMMENT '人员档案id',
                                                PRIMARY KEY (`id`),
                                                UNIQUE KEY `t_b_user_archives_allocation_un` (`crm_no`,`archives_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员分配表';

-- ----------------------------
-- Table structure for t_b_user_archives_supplier
-- ----------------------------
DROP TABLE IF EXISTS `t_b_user_archives_supplier`;
CREATE TABLE `t_b_user_archives_supplier` (
                                              `id` bigint NOT NULL COMMENT '主键',
                                              `user_archives_id` bigint NOT NULL COMMENT '人员档案id',
                                              `supplier_id` bigint DEFAULT NULL COMMENT '供应商ID',
                                              `archives_time` bigint NOT NULL COMMENT '入档时间',
                                              `authentication_time` bigint DEFAULT NULL COMMENT '认证时间',
                                              `authentication_score` double DEFAULT NULL COMMENT '认证成绩',
                                              `authentication_valid_time` bigint DEFAULT NULL COMMENT '认证有效时间',
                                              `remark` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '备注',
                                              PRIMARY KEY (`id`),
                                              UNIQUE KEY `t_b_user_archives_supplier_un` (`user_archives_id`,`supplier_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='人员档案与供应商表';

-- ----------------------------
-- Table structure for t_b_violation
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation`;
CREATE TABLE `t_b_violation` (
                                 `id` bigint NOT NULL COMMENT '主键',
                                 `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                 `sop_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'sop单号',
                                 `code` varchar(200) CHARACTER SET utf8mb4 NOT NULL COMMENT '编号',
                                 `service_id` bigint DEFAULT NULL COMMENT '服务单元id',
                                 `user_id` bigint NOT NULL COMMENT '负责人id',
                                 `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，CONTENT_ERROR：内容错误虚假，LOGIC_ERROR：逻辑不合理，OTHER：其它',
                                 `content` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '违规情况',
                                 `status` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，NOT_START：未跟进，FOLLOW：跟进，CLOSE：关闭，RESTART：重启',
                                 `attachment_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件ids',
                                 `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                 `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                 `update_id` bigint DEFAULT NULL COMMENT '更新人id',
                                 `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                 `close_time` bigint DEFAULT NULL COMMENT '关闭时间',
                                 PRIMARY KEY (`id`),
                                 UNIQUE KEY `t_b_violation_un_code` (`code`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='sop违规登记表';

-- ----------------------------
-- Table structure for t_b_violation_detail
-- ----------------------------
DROP TABLE IF EXISTS `t_b_violation_detail`;
CREATE TABLE `t_b_violation_detail` (
                                        `id` bigint NOT NULL COMMENT '主键',
                                        `violation_id` bigint NOT NULL COMMENT '违规id',
                                        `remark` varchar(500) CHARACTER SET utf8mb4 NOT NULL COMMENT '跟进说明',
                                        `attachment_ids` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '附件ids',
                                        `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                        `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                        PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='违规明细表';

-- ----------------------------
-- Table structure for t_d_form_widget
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget`;
CREATE TABLE `t_d_form_widget` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `code` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                   `type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '类型，FORM：表单，TABLE：表格，OTHER：其它',
                                   `form_id` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '表单id',
                                   `form_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '表单名称',
                                   `title` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '标题',
                                   `input_type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '输入类型，STRING：字符串，INT：整型，LONG：长整型，DOUBLE：浮点型，ENUM：枚举，DATE：日期，BOOL：布尔，BIGDECIMAL：精确浮点，FILE：文件，ARRAY：数组',
                                   `required` tinyint DEFAULT '1' COMMENT '是否必填，0：非必填，1：必填',
                                   `readable` tinyint DEFAULT '0' COMMENT '是否只读，0：非只读，1：只读',
                                   `writable` tinyint DEFAULT '1' COMMENT '是否可写，0：非可写，1：可写',
                                   `visable` tinyint DEFAULT '1' COMMENT '是否可见，0：不可见，1：可见',
                                   `scale` int DEFAULT NULL COMMENT '保留小数点几位',
                                   `length` int DEFAULT NULL COMMENT '长度',
                                   `binding` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '绑定字段',
                                   `data_grid` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '数据来源',
                                   `tips` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '提示',
                                   `format` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '格式',
                                   `span` int DEFAULT '6' COMMENT '占宽符',
                                   `sub_title` varchar(1000) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '子标题',
                                   `options` varchar(500) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '选项值，RADIO或CHECKBOX填此值',
                                   `flow_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `form_setup` int NOT NULL COMMENT '表单步骤',
                                   `handle` tinyint NOT NULL DEFAULT '0' COMMENT '处理时限，0：不处理，1：可处理',
                                   `form_group` int DEFAULT NULL COMMENT '控件组',
                                   `role_type` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '角色类型',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单控件表';

-- ----------------------------
-- Table structure for t_d_form_widget_metadata
-- ----------------------------
DROP TABLE IF EXISTS `t_d_form_widget_metadata`;
CREATE TABLE `t_d_form_widget_metadata` (
                                            `id` bigint NOT NULL COMMENT '主键',
                                            `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                            `flow_deployment_id` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程部署id',
                                            `version` int NOT NULL DEFAULT '1' COMMENT '版本号',
                                            `code` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '编码，TEXT：文本，NUMBER：数字文本，TEXTAREA：文本域，SINGLE_SELECT：单选下拉框，MULTIPLE_SELECT：多选下拉框，RADIO：单选框，CHECKBOX：复选框，DATE：日期，BUTTON：按钮，IMG：图片，VIDEO：视频，AUDIO：音频，FILE：文件，LABEL：标签，TABLE：表格，SIGN：签名，CASCADE_ADDRESS：城市选择控件，OTHER：其它，FORM_GROUP_TITLE：带虚线分割线的标题，ONLE_TITLE：普通标题，独占一行，RADIO_WITH_INPUT：带1个输入框的单选框，DEVICE_IN_TABLE：设备入库表格，DEVICE_OUT_TABLE：设备出库表格',
                                            `field_id` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '字段id',
                                            `field_name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '字段名称',
                                            `field_title` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '字段标题',
                                            `field_check` tinyint NOT NULL DEFAULT '0' COMMENT '是否勾选，0：未勾选，1：勾选',
                                            `setup` int NOT NULL COMMENT '流程步骤',
                                            `change_join` tinyint DEFAULT NULL COMMENT '是否是项目计划变更关联字段，0：不是，1：是',
                                            `create_time` bigint NOT NULL COMMENT '创建时间',
                                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='动态表单元数据表';

-- ----------------------------
-- Table structure for t_d_table_prop
-- ----------------------------
DROP TABLE IF EXISTS `t_d_table_prop`;
CREATE TABLE `t_d_table_prop` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `widget_id` bigint DEFAULT NULL COMMENT '控件id',
                                  `td_index` int DEFAULT NULL COMMENT '序号',
                                  `td_id` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '列id',
                                  `td_name` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '列名称',
                                  `title` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '标题',
                                  `width` double DEFAULT NULL COMMENT '宽度',
                                  `height` double DEFAULT NULL COMMENT '高度',
                                  `color` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '颜色',
                                  `rowspan` int DEFAULT NULL COMMENT '跨行',
                                  `colspan` int DEFAULT NULL COMMENT '跨列',
                                  `sub_title` varchar(1000) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '子标题',
                                  `tips` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '提示',
                                  `td_order` tinyint DEFAULT '1' COMMENT '是否排序，0：不排序，1：排序',
                                  `td_search` tinyint DEFAULT '1' COMMENT '是否搜索，0：不搜索，1：搜索',
                                  `edit_widget_id` bigint DEFAULT NULL COMMENT '编辑控件id',
                                  `obj` mediumtext CHARACTER SET utf8mb4 COMMENT '实体内容',
                                  `create_id` bigint DEFAULT NULL COMMENT '创建人id',
                                  `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='表格属性表';

-- ----------------------------
-- Table structure for t_f_custom_flow
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow`;
CREATE TABLE `t_f_custom_flow` (
                                   `id` bigint NOT NULL COMMENT '主键',
                                   `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                   `name` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程名称',
                                   `model_type` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程模式类型，USER_FIXED：指定人员，APPROVE_SET：发起人指定',
                                   `type` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                   `enable` tinyint DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                   `publish` tinyint NOT NULL DEFAULT '1' COMMENT '是否发布，0：否，1：是',
                                   `object_data` mediumtext CHARACTER SET utf8mb4 COMMENT '自定义流程源数据',
                                   `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                   `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                   `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                   `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                   `version` int NOT NULL DEFAULT '1' COMMENT '版本号',
                                   `flow_definition_id` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '流程定义id',
                                   `flow_deployment_id` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '流程部署id',
                                   `flow_process_var` mediumtext CHARACTER SET utf8mb4 COMMENT '流程过程变量数据',
                                   PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程表';

-- ----------------------------
-- Table structure for t_f_custom_flow_allocation
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_allocation`;
CREATE TABLE `t_f_custom_flow_allocation` (
                                              `id` bigint NOT NULL COMMENT '主键',
                                              `type` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT '流程类型，OFFICE_SOP_FLOW：教务处SOP，CLOUD_MARK_SOP_FLOW：研究生SOP，DING_EXCEPTION_FLOW：考勤异常审核流程，PROJECT_EXCHANGE_FLOW：项目计划变更流程，QUALITY_PROBLEM_FLOW：质量问题反馈流程',
                                              `t_f_custom_flow_id` bigint DEFAULT NULL COMMENT '自定义流程id',
                                              `t_f_custom_flow_entity_id` bigint DEFAULT NULL COMMENT '流程实体id',
                                              `role_allocation` mediumtext CHARACTER SET utf8mb4 NOT NULL COMMENT '角色分配',
                                              `version` int DEFAULT NULL COMMENT '版本号',
                                              PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程角色分配表';

-- ----------------------------
-- Table structure for t_f_custom_flow_entity
-- ----------------------------
DROP TABLE IF EXISTS `t_f_custom_flow_entity`;
CREATE TABLE `t_f_custom_flow_entity` (
                                          `id` bigint NOT NULL COMMENT '主键',
                                          `t_f_custom_flow_id` bigint NOT NULL COMMENT '自定义流程id',
                                          `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                          `code` varchar(30) CHARACTER SET utf8mb4 NOT NULL COMMENT '编号',
                                          `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                                          `flow_id` bigint DEFAULT NULL COMMENT '流程id',
                                          `flow_process_var` mediumtext CHARACTER SET utf8mb4 NOT NULL COMMENT '流程过程变量',
                                          `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                          `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                          `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                          `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                          PRIMARY KEY (`id`) USING BTREE,
                                          UNIQUE KEY `t_f_custom_flow_entity_UN_code` (`code`,`flow_id`),
                                          KEY `flow_entity_flowId_idx` (`flow_id`),
                                          KEY `flow_entity_custom_flowId_idx` (`t_f_custom_flow_id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='自定义流程实体数据';

-- ----------------------------
-- Table structure for t_f_flow_approve
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_approve`;
CREATE TABLE `t_f_flow_approve` (
                                    `id` bigint NOT NULL COMMENT '主键',
                                    `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                    `flow_id` bigint NOT NULL COMMENT '流程id',
                                    `approve_id` bigint DEFAULT NULL COMMENT '审核id',
                                    `status` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '状态，START：已开始，AUDITING：审核中，REJECT：已驳回，END：已终止，FINISH：已结束',
                                    `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                    `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                    `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                    `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                    `setup` tinyint NOT NULL COMMENT '计划步骤',
                                    `approve_setup` int DEFAULT NULL COMMENT '审批步骤',
                                    `approve_max_setup` int DEFAULT NULL COMMENT '最大审批步骤',
                                    `transfer_assignee` mediumtext CHARACTER SET utf8mb4 COMMENT '流程转办',
                                    `all_assignee` mediumtext CHARACTER SET utf8mb4 COMMENT '会签审批人',
                                    `reject_link` mediumtext CHARACTER SET utf8mb4 COMMENT '驳回路线',
                                    PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程审核表';

-- ----------------------------
-- Table structure for t_f_flow_log
-- ----------------------------
DROP TABLE IF EXISTS `t_f_flow_log`;
CREATE TABLE `t_f_flow_log` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `org_id` bigint DEFAULT NULL COMMENT '机构id',
                                `flow_id` bigint NOT NULL COMMENT '流程id',
                                `object_table` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体表',
                                `object_id` bigint NOT NULL COMMENT '实体id',
                                `approve_id` bigint DEFAULT NULL COMMENT '审核人id',
                                `approve_setup` tinyint DEFAULT NULL COMMENT '审核步骤',
                                `approve_remark` mediumtext CHARACTER SET utf8mb4 COMMENT '审核意见',
                                `create_id` bigint DEFAULT NULL COMMENT '创建人',
                                `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                                `update_id` bigint DEFAULT NULL COMMENT '更新人',
                                `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                                `approve_operation` varchar(50) CHARACTER SET utf8mb4 NOT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                `pend_approve_id` mediumtext CHARACTER SET utf8mb4 COMMENT '待审核人id',
                                `attachment_ids` mediumtext CHARACTER SET utf8mb4 COMMENT '附件id集合',
                                `approve_multi_setup` int DEFAULT NULL COMMENT '审批步骤',
                                `approve_user_approve_type` varchar(30) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '审批用户类别，ORDER：依次审批，ALL：会签（所有人必须审批），SOME：或签（一名审批人同意或拒绝即可）',
                                `task_id` bigint DEFAULT NULL COMMENT '流程节点id',
                                PRIMARY KEY (`id`) USING BTREE,
                                KEY `flow_log_objectId_idx` (`object_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='流程流水表';

-- ----------------------------
-- Table structure for t_g_error
-- ----------------------------
DROP TABLE IF EXISTS `t_g_error`;
CREATE TABLE `t_g_error` (
                             `id` bigint NOT NULL COMMENT '主键',
                             `org_id` bigint DEFAULT NULL COMMENT '机构id',
                             `obj_id` bigint DEFAULT NULL COMMENT '实体id',
                             `summary` mediumtext CHARACTER SET utf8mb4 COMMENT '实时摘要信息',
                             `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                             PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='全局异常错误信息表';

-- ----------------------------
-- Table structure for t_s_auth
-- ----------------------------
DROP TABLE IF EXISTS `t_s_auth`;
CREATE TABLE `t_s_auth` (
                            `id` bigint NOT NULL COMMENT '主键',
                            `school_id` bigint DEFAULT NULL COMMENT '学校id',
                            `access_key` varchar(255) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'accessKey',
                            `access_secret` varchar(255) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'accessSecret',
                            `description` text CHARACTER SET utf8mb4 COMMENT '离线授权证书',
                            `type` varchar(16) CHARACTER SET utf8mb4 NOT NULL COMMENT '授权类型',
                            `create_id` bigint DEFAULT NULL COMMENT '创建人',
                            `create_time` bigint DEFAULT NULL COMMENT '创建时间',
                            `update_id` bigint DEFAULT NULL COMMENT '更新人',
                            `update_time` bigint DEFAULT NULL COMMENT '更新时间',
                            `expire_time` bigint DEFAULT NULL COMMENT '过期时间',
                            `file` mediumblob COMMENT '文件数据',
                            PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='激活授权配置表';

-- ----------------------------
-- Table structure for t_s_job_log
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_log`;
CREATE TABLE `t_s_job_log` (
                               `id` bigint NOT NULL COMMENT '主键',
                               `job_remind_id` bigint NOT NULL COMMENT '任务提醒id',
                               `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                               `code` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '编码',
                               `obj_id` bigint NOT NULL COMMENT '实体id',
                               `obj_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体名称',
                               `remind_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                               `create_id` bigint NOT NULL COMMENT '创建人id',
                               `create_time` bigint NOT NULL COMMENT '创建时间',
                               PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务日志表';

-- ----------------------------
-- Table structure for t_s_job_remind
-- ----------------------------
DROP TABLE IF EXISTS `t_s_job_remind`;
CREATE TABLE `t_s_job_remind` (
                                  `id` bigint NOT NULL COMMENT '主键',
                                  `type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '类型，FLOW：流程，NOTICE：公告',
                                  `crm_no` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT 'crm单号',
                                  `code` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '编码',
                                  `obj_id` bigint NOT NULL COMMENT '实体id',
                                  `obj_name` varchar(100) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '实体名称',
                                  `remind_type` varchar(20) CHARACTER SET utf8mb4 NOT NULL COMMENT '提醒类别，BEFORE：提前，AFTER：超时',
                                  `exec_time` bigint DEFAULT NULL COMMENT '执行时间',
                                  `exec_field` varchar(50) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '执行字段',
                                  `receive_user_id` bigint DEFAULT NULL COMMENT '接收人id',
                                  `approve_operation` varchar(20) CHARACTER SET utf8mb4 DEFAULT NULL COMMENT '审批操作，START：提交，DRAFT：草稿，APPROVE：审批，REJECT：驳回，END：终止，CANCEL：撤销，EXCHANGE：转他人审批，FINISH：结束',
                                  `enable` tinyint NOT NULL DEFAULT '1' COMMENT '是否启用，0：停用，1：启用',
                                  `create_id` bigint NOT NULL COMMENT '创建人id',
                                  `create_time` bigint NOT NULL COMMENT '创建时间',
                                  PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='任务提醒表';

-- ----------------------------
-- Table structure for t_wxapp_info
-- ----------------------------
DROP TABLE IF EXISTS `t_wxapp_info`;
CREATE TABLE `t_wxapp_info` (
                                `id` bigint NOT NULL COMMENT '主键',
                                `open_id` varchar(100) CHARACTER SET utf8mb4 NOT NULL COMMENT 'openId',
                                `phone` varchar(30) CHARACTER SET utf8mb4 NOT NULL COMMENT '手机号',
                                PRIMARY KEY (`id`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COMMENT='微信信息表';

-- ----------------------------
-- Function structure for currval
-- ----------------------------
DROP FUNCTION IF EXISTS `currval`;
delimiter ;;
CREATE FUNCTION `currval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
	DECLARE value LONG;
	SET value = 0;
SELECT current_value INTO value
FROM sequence
WHERE name = seq_name;
RETURN value;
END;
;;
delimiter ;

-- ----------------------------
-- Function structure for nextval
-- ----------------------------
DROP FUNCTION IF EXISTS `nextval`;
delimiter ;;
CREATE FUNCTION `nextval`(seq_name VARCHAR(50)) RETURNS int
    DETERMINISTIC
BEGIN
UPDATE sequence
SET current_value = current_value + increment
WHERE name = seq_name;
RETURN currval(seq_name);
END;
;;
delimiter ;

INSERT INTO `t_b_product` VALUES (2, '云阅卷', 'CLOUD_MARK', 1, 1, NULL);

INSERT INTO `sequence` VALUES ('cloudMarkSopFlow', 0, 1);
INSERT INTO `sequence` VALUES ('delayWarn', 0, 1);
INSERT INTO `sequence` VALUES ('deviceInOutCode', 0, 1);
INSERT INTO `sequence` VALUES ('dingExceptionFlow', 0, 1);
INSERT INTO `sequence` VALUES ('officeSopFlow', 0, 1);
INSERT INTO `sequence` VALUES ('projectExchangeFlow', 0, 1);
INSERT INTO `sequence` VALUES ('qualityProblemFlow', 0, 1);
INSERT INTO `sequence` VALUES ('userArchivesCode', 0, 1);
INSERT INTO `sequence` VALUES ('violationWarn', 0, 1);

INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1, '系统管理', 'systemManage', 'MENU', NULL, 8, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2, '配置管理', 'configureManage', 'MENU', 1, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3, '客户配置', 'custom', 'MENU', 2, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (4, '供应商配置', 'supplier', 'MENU', 2, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (5, '设备配置', 'device', 'MENU', 2, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (6, '服务档位配置', 'level', 'MENU', 2, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (7, '考勤配置', 'dingConfigure', 'MENU', 2, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (8, '系统日志', 'logManage', 'MENU', 1, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (9, '通知公告管理', 'noticeManage', 'MENU', 8, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (10, '日志查询', 'log', 'MENU', 8, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (11, '用户管理', 'userManage', 'MENU', NULL, 7, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (12, '组织架构管理', 'orgManage', 'MENU', 11, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (13, '组织架构管理', 'org', 'MENU', 12, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (14, '用户权限管理', 'userPrivilegeManage', 'MENU', 11, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (15, '用户管理', 'user', 'MENU', 14, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (16, '角色管理', 'role', 'MENU', 14, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (17, '初始化密码', 'InitPwd', 'LINK', 15, 3, 'AUTH', '18', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (18, '用户初始化密码', '/api/admin/user/init_password', 'URL', 15, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (19, '项目质量管理', 'projectQualityManage', 'MENU', NULL, 6, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (20, '项目质量管理', 'projectChildQualityManage', 'MENU', 19, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (21, '质量问题反馈', 'qualityProblem', 'MENU', 20, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (23, '质量监控分析', 'qualityProblemControl', 'MENU', 20, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (24, '工时管理', 'hoursManage', 'MENU', NULL, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (25, '工时管理', 'hoursChildManage', 'MENU', 24, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (26, '考勤打卡', 'ding', 'MENU', 25, 1, NULL, '3017,2044,2045,2046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (27, '异常审核', 'exception', 'MENU', 25, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (28, '考勤提交', 'dingSubmit', 'MENU', 25, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (29, '工时统计', 'hourSubmit', 'MENU', 25, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (30, '考勤监控预警', 'dingWarn', 'MENU', 25, 5, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (31, '资源保障', 'resourceManage', 'MENU', NULL, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (32, '人资保障', 'humanManage', 'MENU', 31, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (33, '人员档案管理', 'userArchives', 'MENU', 32, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (34, '人员调配', 'userAllocate', 'MENU', 32, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (35, '人资监控', 'userArchivesControl', 'MENU', 32, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (36, '设备保障', 'deviceManage', 'MENU', 31, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (37, '出入库登记查询', 'deviceInOut', 'MENU', 36, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (38, '设备资源监控', 'deviceControl', 'MENU', 36, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (39, 'SOP管理', 'sopManage', 'MENU', NULL, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (40, 'SOP管理', 'sopChildManage', 'MENU', 39, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (41, '教务处SOP管理', 'office', 'MENU', 40, 1, NULL, '3034,3035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (42, '研究生SOP管理', 'cloudMark', 'MENU', 40, 2, NULL, '3034,3035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (43, '设备出入库登记', 'deviceInOutSop', 'MENU', 40, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (44, '项目计划变更报备', 'projectExchange', 'MENU', 40, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (45, 'SOP监控', 'sopControlManage', 'MENU', 39, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (46, 'SOP监控台', 'sopControl', 'MENU', 45, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (47, '延期预警', 'delayWarn', 'MENU', 45, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (48, '违规登记', 'violation', 'MENU', 45, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (49, '服务单元管理', 'serviceManage', 'MENU', NULL, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (50, '派单管理', 'crmManage', 'MENU', 49, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (51, '派单管理', 'crm', 'MENU', 50, 1, 'AUTH', '1032,1035,1036,1037,1038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (52, '统计查询', 'crmCount', 'MENU', 50, 2, 'AUTH', '1034', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (53, '服务单元管理', 'serviceChildManage', 'MENU', 49, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (54, '服务单元管理', 'service', 'MENU', 53, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (55, '服务范围管理', 'serviceScope', 'MENU', 53, 2, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (56, '服务区域规划', 'serviceRegion', 'MENU', 53, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (57, '统计查询', 'serviceCount', 'MENU', 53, 4, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (58, '我的工作台', 'workManage', 'MENU', NULL, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (59, '工作台', 'workChildManage', 'MENU', 58, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (60, '消息提醒', 'message', 'MENU', 59, 1, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (61, '待办任务', 'work', 'MENU', 59, 2, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (62, '通知公告', 'notice', 'MENU', 59, 3, NULL, NULL, 1, 0, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (63, '公共接口', 'common', 'MENU', NULL, 9, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (64, '权限管理', 'menuManage', 'MENU', 63, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (65, '用户登录', '/api/admin/common/login', 'URL', 64, 1, 'NO_AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (66, '用户登出', '/api/admin/common/logout', 'URL', 64, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (67, '文件上传', '/api/admin/common/file/upload', 'URL', 64, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (68, '文件下载', '/api/admin/common/file/download', 'URL', 64, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (69, '文件预览', '/api/admin/common/file/preview', 'URL', 64, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (72, '任务管理查询', '/api/admin/task/query', 'URL', 240, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (73, '下载导入模板', '/api/admin/common/download_import_template', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (74, '列表', 'List', 'LIST', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (75, '查询', 'Select', 'BUTTON', 3, 3, 'AUTH', '262', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (76, '查询条件', 'Condition', 'CONDITION', 3, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (77, '修改', 'Update', 'LINK', 3, 1, 'AUTH', '260,263', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (78, '删除', 'Delete', 'LINK', 3, 2, 'AUTH', '259,260', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (79, '新增', 'Add', 'BUTTON', 3, 1, 'AUTH', '258', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (80, '批量导入', 'BatchImport', 'BUTTON', 3, 2, 'AUTH', '261', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (81, '新增', 'Add', 'BUTTON', 4, 1, 'AUTH', '264,266', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (82, '列表', 'List', 'LIST', 4, 1, 'AUTH', '268', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (83, '修改', 'Update', 'LINK', 4, 1, 'AUTH', '269,266', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (84, '启用/禁用', 'Enable', 'LINK', 4, 2, 'AUTH', '265', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (85, '列表', 'List', 'LIST', 5, 1, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (86, '查询条件', 'Condition', 'CONDITION', 5, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (87, '新增', 'Add', 'BUTTON', 5, 1, 'AUTH', '270', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (88, '作废', 'Cancel', 'BUTTON', 5, 2, 'AUTH', '271', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (89, '列表', 'List', 'LIST', 6, 1, 'AUTH', '256', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (90, '新增', 'Add', 'BUTTON', 6, 1, 'AUTH', '254,300', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (91, '修改', 'Update', 'LINK', 6, 1, 'AUTH', '254,257', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (92, '删除', 'Delete', 'LINK', 6, 2, 'AUTH', '253', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (93, '列表', 'List', 'LIST', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (94, '新增', 'Add', 'BUTTON', 7, 1, 'AUTH', '277', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (95, '修改', 'Update', 'LINK', 7, 1, 'AUTH', '282', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (96, '删除', 'Delete', 'LINK', 7, 2, 'AUTH', '278', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (97, '查询条件', 'Condition', 'CONDITION', 7, 1, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (98, '列表', 'List', 'LIST', 9, 1, 'AUTH', '1004', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (99, '新增', 'Add', 'BUTTON', 9, 1, 'AUTH', '1005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (100, '撤销', 'Cancel', 'LINK', 9, 2, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (101, '回执查询', 'Select', 'LINK', 9, 2, 'AUTH', '1006,1007,1008', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (102, '查询条件', 'Condition', 'CONDITION', 9, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (103, '修改', 'Update', 'LINK', 9, 3, 'AUTH', '1002,1005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (104, '发布', 'Publish', 'LINK', 9, 1, 'AUTH', '1003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (105, '查询', 'Select', 'BUTTON', 5, 3, 'AUTH', '275', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (106, '查询', 'Select', 'BUTTON', 7, 2, 'AUTH', '281', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (107, '查询', 'Select', 'BUTTON', 9, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (108, '列表', 'List', 'LIST', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (109, '查询条件', 'Condition', 'CONDITION', 10, 1, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (110, '查询', 'Select', 'BUTTON', 10, 2, 'AUTH', '2014', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (111, '列表', 'List', 'LIST', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (112, '新增', 'Add', 'BUTTON', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (113, '查询条件', 'Condition', 'CONDITION', 13, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (114, '修改', 'Update', 'LINK', 13, 1, 'AUTH', '249', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (115, '启用/禁用', 'Enable', 'LINK', 13, 2, 'AUTH', '250', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (116, '查询', 'Select', 'BUTTON', 13, 2, 'AUTH', '248', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (117, '列表', 'List', 'LIST', 15, 1, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (118, '新增', 'Add', 'BUTTON', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (119, '查询条件', 'Condition', 'CONDITION', 15, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (120, '修改', 'Update', 'LINK', 15, 1, 'AUTH', '283', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (121, '启用/禁用', 'Enable', 'LINK', 15, 2, 'AUTH', '284', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (122, '查询', 'Select', 'BUTTON', 15, 2, 'AUTH', '252', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (123, '列表', 'List', 'LIST', 16, 1, 'AUTH', '288', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (124, '新增', 'Add', 'BUTTON', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (125, '修改', 'Update', 'LINK', 16, 1, 'AUTH', '289,3016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (126, '删除', 'Delete', 'LINK', 16, 2, 'AUTH', '290', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (127, '列表', 'List', 'LIST', 21, 1, 'AUTH', '1031,3044,1030', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (128, '作废', 'Cancel', 'BUTTON', 21, 1, 'AUTH', '1029,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (129, '查询条件', 'Condition', 'CONDITION', 21, 1, 'AUTH', '3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (130, '查询', 'Select', 'BUTTON', 21, 2, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (131, '待审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (132, '待审核-批量通过', 'BatchPass', 'BUTTON', 27, 1, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (133, '待审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (134, '待审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (135, '待审核-批量不通过', 'BatchNoPass', 'BUTTON', 27, 2, 'AUTH', '3017,3022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (136, '待审核-通过', 'Pass', 'LINK', 27, 1, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (137, '待审核-不通过', 'NoPass', 'LINK', 27, 2, 'AUTH', '3017,3021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (138, '已审核-列表', 'List', 'LIST', 27, 1, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (139, '已审核-查询条件', 'Condition', 'CONDITION', 27, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (140, '已审核-查询', 'Select', 'BUTTON', 27, 3, 'AUTH', '3017,3020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (141, '列表', 'List', 'LIST', 28, 1, 'AUTH', '2037,2038,1098', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (142, '批量提交', 'BatchSubmit', 'BUTTON', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (143, '查询条件', 'Condition', 'CONDITION', 28, 1, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (144, '查询', 'Select', 'BUTTON', 28, 3, 'AUTH', '2037,2038', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (145, '批量导出', 'BatchExport', 'BUTTON', 28, 2, 'AUTH', '2042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (146, '提交', 'Submit', 'LINK', 28, 1, 'AUTH', '2039', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (147, '撤回/取消撤回', 'Reject', 'LINK', 28, 2, 'AUTH', '2040,2041', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (148, '列表', 'List', 'LIST', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (149, '导出', 'Export', 'BUTTON', 29, 1, 'AUTH', '1027', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (150, '查询条件', 'Condition', 'CONDITION', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (151, '查询', 'Select', 'BUTTON', 29, 2, 'AUTH', '1026, 1028', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (152, '撤回/取消撤回', 'Reject', 'LINK', 29, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (153, '列表', 'List', 'LIST', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (154, '新增', 'Add', 'BUTTON', 33, 1, 'AUTH', '2018,2027', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (155, '批量导入', 'BatchImport', 'BUTTON', 33, 2, 'AUTH', '2019', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (156, '查询条件', 'Condition', 'CONDITION', 33, 1, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (157, '查询', 'Select', 'BUTTON', 33, 5, 'AUTH', '2016,2017', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (158, '批量导出', 'BatchExport', 'BUTTON', 33, 3, 'AUTH', '2020', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (159, '作废', 'Cancel', 'BUTTON', 33, 4, 'AUTH', '2051', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (160, '修改', 'Update', 'LINK', 33, 1, 'AUTH', '2018', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (161, '列表', 'List', 'LIST', 34, 1, 'AUTH', '2021,2022,2023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (162, '查询条件', 'Condition', 'CONDITION', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (163, '查询', 'Select', 'BUTTON', 34, 1, 'AUTH', '2021,2022', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (164, '调配', 'Allocate', 'LINK', 34, 1, 'AUTH', '2024,2030,2031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (165, '发布SOP', 'Publish', 'LINK', 34, 2, 'AUTH', '2028', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (166, '撤销发布', 'UnPublish', 'LINK', 34, 3, 'AUTH', '2029', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (167, '列表', 'List', 'LIST', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (168, '查询条件', 'Condition', 'CONDITION', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (169, '查询', 'Select', 'BUTTON', 37, 1, 'AUTH', '2036', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (170, '列表', 'List', 'LIST', 38, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (171, '列表', 'List', 'LIST', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (172, '查询条件', 'Condition', 'CONDITION', 41, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (173, '作废', 'Cancel', 'BUTTON', 41, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (174, '查询', 'Select', 'BUTTON', 41, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (175, '填报', 'Fill', 'LINK', 41, 1, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (176, '计划变更报备', 'ProjectExchange', 'LINK', 41, 2, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (177, '新增SOP', 'AddSop', 'LINK', 41, 3, 'AUTH', '3041', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (178, '新增违规 ', 'AddViolation', 'LINK', 41, 4, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (179, '上报质量问题', 'ProblemSubmit', 'LINK', 41, 5, 'AUTH', '1031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (180, '列表', 'List', 'LIST', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (181, '查询条件', 'Condition', 'CONDITION', 42, 1, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (182, '作废', 'Cancel', 'BUTTON', 42, 1, 'AUTH', '3046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (183, '查询', 'Select', 'BUTTON', 42, 2, 'AUTH', '3034,3035,3043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (184, '填报', 'Fill', 'LINK', 42, 1, 'AUTH', '3042', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (185, '计划变更报备', 'ProjectExchange', 'LINK', 42, 2, 'AUTH', '3025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (186, '新增违规 ', 'AddViolation', 'LINK', 42, 3, 'AUTH', '1025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (187, '上报质量问题', 'ProblemSubmit', 'LINK', 42, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (188, '列表', 'List', 'LIST', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (189, '查询条件', 'Condition', 'CONDITION', 43, 1, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (190, '新增', 'Add', 'BUTTON', 43, 1, 'AUTH', '2033,2034,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (191, '作废', 'Cancel', 'BUTTON', 43, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (192, '查询', 'Select', 'BUTTON', 43, 3, 'AUTH', '2032', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (193, '列表', 'List', 'LIST', 44, 1, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (194, '查询条件', 'Condition', 'CONDITION', 44, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (196, '查询', 'Select', 'BUTTON', 44, 2, 'AUTH', '3026', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (197, '列表', 'List', 'LIST', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (198, '查询条件', 'Condition', 'CONDITION', 47, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (199, '查询', 'Select', 'BUTTON', 47, 1, 'AUTH', '1015', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (200, '跟进', 'Follow', 'LINK', 47, 1, 'AUTH', '1014,1017,1018', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (201, '关闭', 'Close', 'LINK', 47, 2, 'AUTH', '1013', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (202, '重启', 'Restart', 'LINK', 47, 3, 'AUTH', '1016', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (203, '列表', 'List', 'LIST', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (204, '查询条件', 'Condition', 'CONDITION', 48, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (205, '查询', 'Select', 'BUTTON', 48, 1, 'AUTH', '1023', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (206, '跟进', 'Follow', 'LINK', 48, 1, 'AUTH', '1019,1020,1022,1096', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (207, '关闭', 'Close', 'LINK', 48, 2, 'AUTH', '1021', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (208, '重启', 'Restart', 'LINK', 48, 3, 'AUTH', '1024', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (209, '列表', 'List', 'LIST', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (210, '查询条件', 'Condition', 'CONDITION', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (211, '新增', 'Add', 'BUTTON', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (212, '作废', 'Cancel', 'BUTTON', 51, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (213, '批量划定', 'BatchDelimit', 'BUTTON', 51, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (214, '查询', 'Select', 'BUTTON', 51, 4, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (215, '划定', 'Delimit', 'LINK', 51, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (216, '修改', 'Update', 'LINK', 51, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (217, '重新划定', 'ReDelimit', 'LINK', 51, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (218, '列表', 'List', 'LIST', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (219, '查询条件', 'Condition', 'CONDITION', 54, 1, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (220, '新增', 'Add', 'BUTTON', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (221, '查询', 'Select', 'BUTTON', 54, 2, 'AUTH', '2001', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (222, '修改', 'Update', 'LINK', 54, 1, 'AUTH', '2000', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (223, '作废', 'Cancel', 'LINK', 54, 2, 'AUTH', '2005', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (224, '设置考勤组', 'Set', 'LINK', 54, 3, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (225, '发布', 'Publish', 'LINK', 54, 4, 'AUTH', '2002', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (226, '重启', 'Restart', 'LINK', 54, 5, 'AUTH', '2004', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (227, '关闭', 'Close', 'LINK', 54, 6, 'AUTH', '2003', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (228, '列表', 'List', 'LIST', 55, 1, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (229, '查询条件', 'Condition', 'CONDITION', 55, 1, 'AUTH', '2006', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (230, '新增服务范围', 'Add', 'BUTTON', 55, 1, 'AUTH', '2008,2009', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (231, '查询', 'Select', 'BUTTON', 55, 2, 'AUTH', '2006,2007', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (232, '移除', 'Delete', 'LINK', 55, 1, 'AUTH', '2010', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (233, '列表', 'List', 'LIST', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (234, '查询条件', 'Condition', 'CONDITION', 56, 1, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (235, '新增', 'Add', 'BUTTON', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (236, '查询', 'Select', 'BUTTON', 56, 2, 'AUTH', '2011', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (237, '修改', 'Update', 'LINK', 56, 1, 'AUTH', '2012,2047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (238, '删除', 'Delete', 'LINK', 56, 2, 'AUTH', '2013', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (239, '任务管理', 'taskManage', 'MENU', 1, 3, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (240, '任务管理', 'task', 'MENU', 239, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (241, '列表', 'List', 'LIST', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (242, '查询条件', 'Condition', 'CONDITION', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (243, '查询', 'Select', 'BUTTON', 240, 2, 'AUTH', '72', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (244, '导出日志', 'Export', 'LINK', 240, 1, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (245, '下载文件', 'Download', 'LINK', 240, 2, 'AUTH', NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (246, '获取菜单权限', '/api/admin/sys/privilege/list', 'URL', 64, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (247, '获取用户权限', '/api/admin/common/get_menu', 'URL', 64, 8, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (248, '组织架构查询', '/api/admin/org/list', 'URL', 13, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (249, '组织架构保存/修改', '/api/admin/org/save', 'URL', 13, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (250, '组织架构启用/禁用', '/api/admin/org/enable', 'URL', 13, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (251, '获取机构下所有用户', '/api/admin/common/get_org_user_list', 'URL', 64, 9, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (252, '用户查询', '/api/admin/user/list', 'URL', 15, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (253, '服务档位删除', '/api/sys/level/delete', 'URL', 6, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (254, '服务档位', '/api/sys/level/get', 'URL', 6, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (255, '服务档位列表', '/api/sys/level/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (256, '服务档位列表', '/api/sys/level/query', 'URL', 6, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (257, '服务档位修改', '/api/sys/level/update', 'URL', 6, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (258, '客户新增', '/api/sys/custom/add', 'URL', 3, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (259, '客户删除', '/api/sys/custom/delete', 'URL', 3, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (260, '客户查询', '/api/sys/custom/get', 'URL', 3, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (261, '客户批量导入', '/api/sys/custom/import', 'URL', 3, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (262, '客户查询条件', '/api/sys/custom/query', 'URL', 3, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (263, '客户修改', '/api/sys/custom/update', 'URL', 3, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (264, '供应商新增', '/api/sys/supplier/add', 'URL', 4, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (265, '供应商启用/禁用', '/api/sys/supplier/enable', 'URL', 4, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (266, '供应商列表', '/api/sys/supplier/get', 'URL', 4, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (267, '供应商列表', '/api/sys/supplier/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (268, '供应商列表', '/api/sys/supplier/query', 'URL', 4, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (269, '供应商修改', '/api/sys/supplier/update', 'URL', 4, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (270, '设备新增修改', '/api/sys/device/save', 'URL', 5, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (271, '设备作废', '/api/sys/device/delete', 'URL', 5, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (272, '设备禁用', '/api/sys/device/disable', 'URL', 5, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (273, '设备查询条件', '/api/sys/device/get', 'URL', 5, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (274, '设备列表', '/api/sys/device/list', 'URL', 5, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (275, '设备查询条件', '/api/sys/device/query', 'URL', 5, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (276, '设备新增', '/api/sys/device/update', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (277, '考勤新增', '/api/sys/ding/group/add', 'URL', 7, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (278, '考勤删除', '/api/sys/ding/group/delete', 'URL', 7, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (279, '考勤查询', '/api/sys/ding/group/get', 'URL', 7, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (280, '考勤查询', '/api/sys/ding/group/list', 'URL', 7, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (281, '考勤查询条件', '/api/sys/ding/group/query', 'URL', 7, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (282, '考勤修改', '/api/sys/ding/group/update', 'URL', 7, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (283, '用户新增/修改', '/api/admin/user/save', 'URL', 15, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (284, '用户启用/禁用', '/api/admin/user/enable', 'URL', 15, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (285, '获取所有角色', '/api/admin/common/get_role_list', 'URL', 64, 10, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (286, '获取所有用户', '/api/admin/common/get_user_list', 'URL', 64, 11, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (287, '获取所有机构', '/api/admin/common/get_org_list', 'URL', 64, 12, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (288, '角色查询', '/api/admin/role/list', 'URL', 16, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (289, '角色新增/修改', '/api/admin/role/save', 'URL', 16, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (290, '角色删除', '/api/admin/role/delete', 'URL', 16, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (300, '服务档位新增', '/api/sys/level/add', 'URL', 6, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1000, '通知公告', '/api/sys/notice/delete', 'URL', 9, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1001, '通知公告', '/api/sys/notice/disable', 'URL', 9, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1002, '通知公告', '/api/sys/notice/get', 'URL', 9, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1003, '通知公告发布撤销', '/api/sys/notice/publishOrUnpublish', 'URL', 9, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1004, '通知公告列表', '/api/sys/notice/query', 'URL', 9, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1005, '通知公告新增修改', '/api/sys/notice/save', 'URL', 9, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1006, '回执查询统计', '/api/sys/message/count', 'URL', 9, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1007, '回执查询到出', '/api/sys/message/export', 'URL', 9, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1008, '回执查询', '/api/sys/message/query', 'URL', 9, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1009, '按多个类型统计当前用户未读消息', '/api/sys/message/countByTypes', 'URL', 59, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1010, '查看消息详细', '/api/sys/message/get', 'URL', 59, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1011, '按多个类型查询当前用户未读消息', '/api/sys/message/pageByTypes', 'URL', 59, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1012, '标记消息已阅读', '/api/sys/message/read', 'URL', 59, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1013, '延期预警关闭', '/api/admin/tb/delay/warn/close', 'URL', 47, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1014, '延期预警跟进', '/api/admin/tb/delay/warn/get', 'URL', 47, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1015, '延期预警查询', '/api/admin/tb/delay/warn/query', 'URL', 47, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1016, '延期预警重启', '/api/admin/tb/delay/warn/restart', 'URL', 47, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1017, '延期预警跟进', '/api/admin/tb/delay/warn/detail/list', 'URL', 47, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1018, '延期预警跟进', '/api/admin/tb/delay/warn/detail/save', 'URL', 47, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1019, '违规登记跟进', '/api/admin/tb/violation/detail/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1020, '违规登记跟进', '/api/admin/tb/violation/detail/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1021, '违规登记关闭', '/api/admin/tb/violation/close', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1022, '违规登记跟进', '/api/admin/tb/violation/get', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1023, '违规登记查询', '/api/admin/tb/violation/query', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1024, '违规登记重启', '/api/admin/tb/violation/restart', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1025, '违规登记保存', '/api/admin/tb/violation/save', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1026, '工时统计', '/api/admin/tb/ding/count', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1027, '工时统计', '/api/admin/tb/ding/export', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1028, '工时统计', '/api/admin/tb/ding/query', 'URL', 29, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1029, '质量问题反馈', '/api/admin/tb/quality/problem/apply/delete', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1030, '质量问题反馈', '/api/admin/tb/quality/problem/apply/query', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1031, '质量问题反馈', '/api/admin/tb/quality/problem/apply/save', 'URL', 21, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1032, '派单管理', '/api/admin/tb/crm/batchDisable', 'URL', 50, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1034, '派单管理', '/api/admin/tb/crm/countWaitZone', 'URL', 50, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1035, '派单管理', '/api/admin/tb/crm/delete', 'URL', 50, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1036, '派单管理', '/api/admin/tb/crm/get', 'URL', 50, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1037, '派单管理', '/api/admin/tb/crm/query', 'URL', 50, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1038, '派单管理', '/api/admin/tb/crm/save', 'URL', 50, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1039, '客户表查询', '/api/sys/custom/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1040, '获取产品信息列表接口', '/api/admin/tb/product/list', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1041, '考勤明细查询', '/api/admin/tb/ding/attendance/count', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1042, '考勤明细查询', '/api/admin/tb/ding/attendance/export', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1043, '考勤明细查询', '/api/admin/tb/ding/attendance/query', 'URL', 1044, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1044, '考勤明细查询', 'dingDetail', 'MENU', 25, 1, NULL, '1041,1042,1043', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1045, '设备品牌表列表接口', '/api/sys/device/brand/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1046, '设备启用', '/api/sys/device/enable', 'URL', 5, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1047, '设备导入', '/api/sys/device/import', 'URL', 5, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1048, '批量启用', 'BatchEnable', 'BUTTON', 5, 4, 'AUTH', '1045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1049, '批量禁用', 'BatchUnEnable', 'BUTTON', 5, 5, 'AUTH', '272', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1050, '批量导入', 'BatchImport', 'BUTTON', 5, 6, 'AUTH', '1047', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1051, '获取SOP用户', '/api/admin/sop/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1052, '修改', 'Update', 'LINK', 5, 1, 'AUTH', '276', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1053, '禁用启用', 'Enable', 'LINK', 5, 2, 'AUTH', '272,1046', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1054, '获取用户', '/api/admin/tb/violation/creator/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1055, '获取用户', '/api/admin/tb/violation/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1056, '获取用户', '/api/admin/tb/quality/problem/apply/user/list', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1057, '二期接口', '/api/sop/analyse/attendance', 'URL', 1103, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1058, '二期接口', '/api/sop/analyse/list', 'URL', 1103, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1059, '二期接口', '/api/sop/analyse/overview', 'URL', 1103, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1060, '二期接口', '/api/sop/analyse/processing', 'URL', 1103, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1061, '二期接口', '/api/sop/analyse/trend', 'URL', 1103, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1062, '二期接口', '/api/service/analyse/device/use', 'URL', 1102, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1063, '二期接口', '/api/service/analyse/list', 'URL', 1102, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1064, '二期接口', '/api/service/analyse/overview', 'URL', 1102, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1065, '二期接口', '/api/service/analyse/personnel/quota', 'URL', 1102, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1066, '二期接口', '/api/service/analyse/project/allocation', 'URL', 1102, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1067, '二期接口', '/api/service/analyse/region/personnel', 'URL', 1102, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1068, '二期接口', '/api/service/analyse/supplier', 'URL', 1102, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1069, '二期接口', '/api/service/analyse/supplier/detail', 'URL', 1102, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1070, '二期接口', '/api/service/analyse/supplier/region', 'URL', 1102, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1071, '二期接口', '/api/service/analyse/supplier/region/detail', 'URL', 1102, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1072, '二期接口', '/api/service/analyse/supplier/role', 'URL', 1102, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1073, '二期接口', '/api/admin/tb/crm/analyse/custom/type', 'URL', 1101, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1074, '二期接口', '/api/admin/tb/crm/analyse/custom/type/detail', 'URL', 1101, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1075, '二期接口', '/api/admin/tb/crm/analyse/monthly', 'URL', 1101, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1076, '二期接口', '/api/admin/tb/crm/analyse/monthly/detail', 'URL', 1101, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1077, '二期接口', '/api/admin/tb/crm/analyse/project', 'URL', 1101, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1078, '二期接口', '/api/admin/tb/crm/analyse/project/detail', 'URL', 1101, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1079, '二期接口', '/api/admin/tb/crm/analyse/region', 'URL', 1101, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1080, '二期接口', '/api/admin/tb/crm/analyse/region/detail', 'URL', 1101, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1081, '二期接口', '/api/admin/tb/crm/analyse/supplier', 'URL', 1101, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1082, '二期接口', '/api/admin/tb/crm/analyse/supplier/detail', 'URL', 1101, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1083, '二期接口', '/api/quality/analyse/influence', 'URL', 1106, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1084, '二期接口', '/api/quality/analyse/influence/detail', 'URL', 1106, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1085, '二期接口', '/api/quality/analyse/list', 'URL', 1106, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1086, '二期接口', '/api/quality/analyse/pie', 'URL', 1106, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1087, '二期接口', '/api/quality/analyse/progress', 'URL', 1106, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1088, '二期接口', '/api/quality/analyse/radar', 'URL', 1106, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1089, '二期接口', '/api/quality/analyse/supplier/detail', 'URL', 1106, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1090, '二期接口', '/api/sys/device/monitor/count', 'URL', 1105, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1091, '二期接口', '/api/sys/device/monitor/countByModel', 'URL', 1105, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1092, '二期接口', '/api/sys/device/monitor/countByServiceUnit', 'URL', 1105, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1093, '二期接口', '/api/sys/device/monitor/countByServiceUnitDetail', 'URL', 1105, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1094, '二期接口', '/api/sop/schedule/detail', 'URL', 1104, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1095, '二期接口', '/api/sop/schedule/progress', 'URL', 1104, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1096, '违规登记跟进', '/api/admin/tb/violation/getByCode', 'URL', 48, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1097, '二期接口', '/api/sop/schedule/projectProgress', 'URL', 1104, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1098, '考勤提交-明细', '/api/admin/tb/ding/detail', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1099, '二期接口', '/api/admin/tb/crm/analyse/project/count', 'URL', 1101, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1100, '数据驾驶舱', 'dataCockpit', 'MENU', NULL, 12, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1101, '派单分析', 'crmAnalyse', 'MENU', 1100, 1, NULL, '1073,1074,1075,1076,1077,1078,1079,1080,1081,1082,1099', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1102, '服务单元分析', 'serviceAnalyse', 'MENU', 1100, 2, NULL, '1062,1063,1064,1065,1066,1067,1068,1069,1070,1071,1072', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1103, 'SOP预警监控', 'sopAnalyse', 'MENU', 1100, 3, NULL, '1057,1058,1059,1060,1061', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1104, '项目进度监控', 'sopSchedule', 'MENU', 1100, 4, NULL, '1094,1095,1097', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1105, '设备保障监控', 'deviceMonitor', 'MENU', 1100, 5, NULL, '1090,1091,1092,1093', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (1106, '质量分析监控', 'qualityAnalyse', 'MENU', 1100, 6, NULL, '1083,1084,1085,1086,1087,1088,1089', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2000, '服务单元管理-编辑', '/api/service/service/unit/edit', 'URL', 54, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2001, '服务单元管理-分页查询', '/api/service/service/unit/page', 'URL', 54, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2002, '服务单元管理-发布', '/api/service/service/unit/publish', 'URL', 54, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2003, '服务单元管理-关闭', '/api/service/service/unit/cloze', 'URL', 54, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2004, '服务单元管理-重启', '/api/service/service/unit/restart', 'URL', 54, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2005, '服务单元管理-作废', '/api/service/service/unit/cancel', 'URL', 54, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2006, '服务范围管理-分页查询', '/api/service/service/scope/page', 'URL', 55, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2007, '服务范围管理-派单小计', '/api/service/service/scope/subTotal', 'URL', 55, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2008, '服务范围管理-新增服务范围-未绑定派单分页查询', '/api/service/service/scope/unbind/page', 'URL', 55, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2009, '服务范围管理-批量划定派单和服务单元关系', '/api/service/service/scope/bind_batch', 'URL', 55, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2010, '服务范围管理-移出', '/api/service/service/scope/unbind', 'URL', 55, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2011, '服务区域规划-分页查询', '/api/service/service/region/page', 'URL', 56, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2012, '服务区域规划-编辑', '/api/service/service/region/edit', 'URL', 56, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2013, '服务区域规划-删除', '/api/service/service/region/remove', 'URL', 56, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2014, '日志管理-查询', '/api/sys/log/page', 'URL', 10, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2015, '公共接口-查询服务单元列表', '/api/admin/common/query_service_unit', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2016, '人员档案管理-分页查询', '/api/admin/user/archives/page', 'URL', 33, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2017, '人员档案管理-小计', '/api/admin/user/archives/sub_total', 'URL', 33, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2018, '人员档案管理-编辑', '/api/admin/user/archives/edit', 'URL', 33, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2019, '人员档案管理-导入', '/api/admin/user/archives/import', 'URL', 33, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2020, '人员档案管理-导出', '/api/admin/user/archives/export', 'URL', 33, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2021, '人员调配管理-分页查询', '/api/admin/user/archives/allocation/page', 'URL', 34, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2022, '人员调配管理-小计查询', '/api/admin/user/archives/allocation/sub_total', 'URL', 34, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2023, '人员调配管理-根据派单号查询派单分配信息', '/api/admin/user/archives/allocation/detail_by_crm_no', 'URL', 34, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2024, '人员调配管理-调配', '/api/admin/user/archives/allocation/edit', 'URL', 34, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2025, '人员调配管理—自动调配', '/api/admin/user/archives/allocation/auto_edit', 'URL', 34, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2026, '批量调配', 'AllocateBatch', 'BUTTON', 34, 4, 'AUTH', '2025', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2027, '人员档案管理-生成编号', '/api/admin/user/archives/create_code', 'URL', 33, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2028, '人员调配管理-发布', '/api/admin/user/archives/allocation/publish', 'URL', 34, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2029, '人员调配管理-撤销发布', '/api/admin/user/archives/allocation/un_publish', 'URL', 34, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2030, '人员调配管理-查询空闲工程师', '/api/admin/user/archives/allocation/free_engineer', 'URL', 34, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2031, '人员调配管理-查询空闲区域协调人', '/api/admin/user/archives/allocation/free_coordinator', 'URL', 34, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2032, 'sop管理 - 设备出入库登记查询', '/api/admin/device/in/out/sop_page', 'URL', 43, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2033, '查询可出库的设备信息', '/api/admin/device/in/out/can_out_info', 'URL', 43, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2034, '查询可入库的设备信息', '/api/admin/device/in/out/can_in_info', 'URL', 43, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2035, '设备出入库提交', '/api/admin/device/in/out/submit', 'URL', 43, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2036, '资源保障-设备保障-出入库登记查询', '/api/admin/device/in/out/source_page', 'URL', 37, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2037, '考勤提交-分页查询', '/api/admin/tb/ding/submit/page', 'URL', 28, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2038, '考勤提交-小计', '/api/admin/tb/ding/submit/sub_total', 'URL', 28, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2039, '考勤提交-批量提交', '/api/admin/tb/ding/submit/submit_batch', 'URL', 28, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2040, '考勤提交-考勤撤回', '/api/admin/tb/ding/submit/apply_withdraw', 'URL', 28, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2041, '考勤提交-取消撤回', '/api/admin/tb/ding/submit/cancel_withdraw', 'URL', 28, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2042, '考勤提交-批量导出', '/api/admin/tb/ding/submit/export', 'URL', 28, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2043, '查询项目派单信息', '/api/admin/tb/crm/crm_project_info', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2044, '考勤打卡信息', '/api/admin/tb/ding/ding_info', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2045, '考勤结果统计', '/api/admin/tb/ding/ding_statistic', 'URL', 26, 2, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2046, '考勤打卡保存', '/api/admin/tb/ding/ding_save', 'URL', 26, 3, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2047, '服务区域规划-查询可选大区经理', '/api/service/service/region/find_leader', 'URL', 56, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2048, '考勤打卡-查询进行中的sop', '/api/admin/tb/ding/ding_find_running_sop', 'URL', 26, 4, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2049, '考勤打卡-查询所有sop', '/api/admin/tb/ding/ding_find_all_sop', 'URL', 26, 5, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2051, '人员档案管理-作废', '/api/admin/user/archives/cancel', 'URL', 33, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2052, '出入库登记查询-查询登记人列表', '/api/admin/device/in/out/registrant_list', 'URL', 64, 33, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2053, '智能客服', 'aiCustomerManage', 'MENU', NULL, 11, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2054, '智能客服', 'aiCustomerChildManage', 'MENU', 2053, 1, NULL, '2055', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2055, '软通智能客服单点登录', '/api/sso/rtzf/login', 'URL', 2054, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2056, '工时统计-同意撤回', '/api/admin/tb/ding/submit/agree_withdraw', 'URL', 29, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (2057, '同意撤回', 'Reject', 'LINK', 29, 5, 'AUTH', '2056', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3000, '修改密码', '/api/admin/common/update_password', 'URL', 64, 13, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3001, '流程审批接口', '/api/admin/flow/approve', 'URL', 64, 14, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3002, '创建流程接口', '/api/admin/flow/createDeployment', 'URL', 64, 15, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3003, '结束流程接口', '/api/admin/flow/delete', 'URL', 64, 16, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3004, '获取流程部署信息接口', '/api/admin/flow/deployment/data', 'URL', 64, 17, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3005, '获取流程表单属性接口', '/api/admin/flow/form_properties/get', 'URL', 64, 18, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3006, '获取流程待办', '/api/admin/flow/task/list', 'URL', 64, 19, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3007, '流程转办接口', '/api/admin/flow/task/approver/exchange', 'URL', 64, 20, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3008, '上传流程接口', '/api/admin/flow/upload', 'URL', 64, 21, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3009, '获取流程详细信息接口', '/api/admin/flow/view', 'URL', 64, 22, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3010, '清除缓存接口', '/api/admin/common/cache_clear', 'URL', 3012, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3011, '超管菜单', 'adminManage', 'MENU', NULL, 10, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3012, '清除缓存', 'cacheManage', 'MENU', 3011, 1, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3013, '消息提醒规则', 'messageRuleManage', 'MENU', 3011, 2, NULL, NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3014, '清除缓存', 'Clear', 'BUTTON', 3012, 1, 'AUTH', '3010', 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3015, '查询枚举接口', '/api/admin/common/query_enum', 'URL', 64, 23, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3016, '角色权限查询接口', '/api/admin/role/privilege/list', 'URL', 16, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3017, '考勤异常申请接口', '/api/admin/ding/exception/apply/save', 'URL', 26, 1, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3018, '根据角色类型获取所有用户', '/api/admin/common/get_user_list_by_role_type', 'URL', 64, 24, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3019, '考勤异常待审核接口', '/api/admin/ding/exception/apply/flow/task/un_done/list', 'URL', 27, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3020, '考勤异常已审核接口', '/api/admin/ding/exception/apply/flow/task/done/list', 'URL', 27, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3021, '考勤异常审核接口', '/api/admin/ding/exception/apply/flow/approve', 'URL', 27, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3022, '考勤异常批量审核接口', '/api/admin/ding/exception/apply/flow/batch/approve', 'URL', 27, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3023, '获取违规延期预警待办接口', '/api/admin/my/work_bench/warn/list', 'URL', 64, 25, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3024, '处理申请', 'Finish', 'LINK', 44, 1, 'AUTH', '3030,3031', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3025, '项目变更计划申请接口', '/api/admin/project/exchange/save', 'URL', 40, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3026, '项目变更计划列表接口', '/api/admin/project/exchange/list', 'URL', 40, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3027, '流程控件查询接口', '/api/admin/widget/flow/select', 'URL', 64, 26, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3028, '控件查询列表接口', '/api/admin/widget/list', 'URL', 64, 27, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3029, '批量结束流程接口', '/api/admin/flow/batch/delete', 'URL', 64, 28, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3030, '项目变更计划审核接口', '/api/admin/project/exchange/flow/approve', 'URL', 40, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3031, '项目变更计划详情接口', '/api/admin/project/exchange/view', 'URL', 40, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3032, '获取流程待办计数接口', '/api/admin/flow/task/list/count', 'URL', 64, 29, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3033, '获取违规/延期预警待办计数接口', '/api/admin/my/work_bench/warn/list/count', 'URL', 64, 30, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3034, 'sop元数据查询接口', '/api/admin/sop/metadata/list', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3035, 'sop元数据修改接口', '/api/admin/sop/metadata/save', 'URL', 40, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3036, '获取流程版本信息接口', '/api/admin/flow/version/list', 'URL', 64, 31, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3037, '编辑', 'Edit', 'LINK', 41, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3038, '作废', 'Cancel', 'LINK', 41, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3039, '编辑', 'Edit', 'LINK', 42, 6, 'AUTH', '3047,3048', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3040, '作废', 'Cancel', 'LINK', 42, 7, 'AUTH', '3045', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3041, 'sop申请接口', '/api/admin/sop/apply', 'URL', 40, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3042, 'sop填报接口', '/api/admin/sop/approve', 'URL', 40, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3043, 'sop列表接口', '/api/admin/sop/list', 'URL', 40, 8, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3044, '质量问题反馈审核', '/api/admin/tb/quality/problem/apply/approve', 'URL', 21, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3045, 'sop删除', '/api/admin/sop/cancel', 'URL', 40, 9, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3046, 'sop批量删除', '/api/admin/sop/batch/cancel', 'URL', 40, 10, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3047, 'sop编辑', '/api/admin/sop/edit', 'URL', 40, 11, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3048, 'sop保存 ', '/api/admin/sop/save', 'URL', 40, 12, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3049, '审核', 'Approve', 'LINK', 21, 1, 'AUTH', '1030,3044', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3050, '获取所有用户（动态表单用）接口', '/api/admin/common/get_user_list/form_widget', 'URL', 64, 32, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3051, '小程序签名预览', '/api/admin/common/file/preview/app', 'URL', 64, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3052, '考勤异常详情接口', '/api/admin/ding/exception/apply/view', 'URL', 27, 6, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3053, '设备发货管理', 'deviceDelivery', 'MENU', 36, 1, NULL, NULL, 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3054, '列表', 'List', 'LIST', 3053, 1, 'AUTH', '3067,3071', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3055, '查询条件', 'Condition', 'CONDITION', 3053, 1, 'AUTH', '3067,3071', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3056, '批量导入', 'BatchImport', 'BUTTON', 3053, 1, 'AUTH', '3069', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3057, '批量导出', 'BatchExport', 'BUTTON', 3053, 2, 'AUTH', '3070', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3058, '批量发货', 'BatchDeliver', 'BUTTON', 3053, 3, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3059, '批量签收', 'BatchReceive', 'BUTTON', 3053, 4, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3060, '批量作废', 'BatchCancel', 'BUTTON', 3053, 5, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3061, '修改', 'Update', 'LINK', 3053, 1, 'AUTH', '3065,3068', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3062, '作废', 'Cancel', 'LINK', 3053, 2, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3063, '发货', 'Deliver', 'LINK', 3053, 3, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3064, '签收', 'Receive', 'LINK', 3053, 4, 'AUTH', '3066', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3065, '设备发货保存接口', '/api/admin/device/delivery/save', 'URL', 3053, 1, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3066, '设备发货状态修改接口', '/api/admin/device/delivery/status/update', 'URL', 3053, 2, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3067, '设备发货列表分页查询', '/api/admin/device/delivery/page', 'URL', 3053, 3, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3068, '设备发货查询单个记录接口', '/api/admin/device/delivery/find/by/id', 'URL', 3053, 4, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3069, '设备发货导入', '/api/admin/device/delivery/import', 'URL', 3053, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3070, '设备发货导出', '/api/admin/device/delivery/export', 'URL', 3053, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3071, '设备发货人列表查询', '/api/admin/device/delivery/delivery_user_list', 'URL', 3053, 7, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3072, '绑定供应商', 'bandSupplier', 'LINK', 15, 4, 'AUTH', '3074,3075', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3073, '根据用户角色查询供应商信息接口', '/api/sys/supplier/list/by/user_role', 'URL', 4, 7, 'SYS', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3074, '查询绑定供应商', '/api/admin/user/query/supplier', 'URL', 15, 5, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3075, '绑定供应商', '/api/admin/user/band/supplier', 'URL', 15, 6, 'AUTH', NULL, 1, 1, 0);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3076, '批量返还', 'BatchReturn', 'BUTTON', 3053, 5, 'AUTH', '3078,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3077, '返还', 'Return', 'LINK', 3053, 6, 'AUTH', '3078,2035', 1, 0, 1);
INSERT INTO `sys_privilege` (`id`, `name`, `url`, `type`, `parent_id`, `sequence`, `property`, `related`, `enable`, `default_auth`, `front_display`) VALUES (3078, '设备发货可返还设备列表查询', '/api/admin/device/delivery/can_return_info', 'URL', 3053, 8, 'AUTH', NULL, 1, 1, 0);

INSERT INTO `sys_role` VALUES (1, NULL, '系统管理员', 1, 'ADMIN', NULL, NULL, NULL, NULL, 1, '系统角色');
INSERT INTO `sys_role` VALUES (2, NULL, '总负责人', 1, 'PMO', NULL, NULL, 443437441441857536, 1697425600601, 1, '系统角色');
INSERT INTO `sys_role` VALUES (3, NULL, '业务线负责人', 1, 'BUSSINESS', NULL, NULL, 443437441441857536, 1697425607705, 1, '系统角色');
INSERT INTO `sys_role` VALUES (4, NULL, '大区经理', 1, 'REGION_MANAGER', NULL, NULL, 1, 1702265926259, 1, '系统角色');
INSERT INTO `sys_role` VALUES (5, NULL, '区域协调人', 1, 'REGION_COORDINATOR', NULL, NULL, 443437441441857536, 1697425621388, 1, '外包角色');
INSERT INTO `sys_role` VALUES (6, NULL, '实施工程师', 1, 'EFFECT_ENGINEER', NULL, NULL, 1, 1699519055688, 1, '外包角色');
INSERT INTO `sys_role` VALUES (7, NULL, '助理工程师', 1, 'ASSISTANT_ENGINEER', NULL, NULL, 1, 1699518786207, 1, '外包角色');
INSERT INTO `sys_role` VALUES (8, NULL, 'QA', 1, 'QA', NULL, NULL, 443437441441857536, 1697785940307, 1, '系统角色');
INSERT INTO `sys_role` VALUES (9, NULL, '技术客服', 1, 'CUSTOM', NULL, NULL, 443437441441857536, 1697425672305, 1, '系统角色');
INSERT INTO `sys_role` VALUES (10, NULL, '客户经理', 1, 'ACCOUNT_MANAGER', NULL, NULL, 443437441441857536, 1697425679196, 1, '系统角色');
INSERT INTO `sys_role` VALUES (11, NULL, '乙方总负责人', 1, 'THIRD_PMO', NULL, NULL, 443437441441857536, 1697621767961, 1, '系统角色');
INSERT INTO `sys_role` VALUES (12, NULL, '设备发货员', 1, 'DEVICE_DELIVERY', NULL, NULL, 1, 1702266042618, 1, '系统角色');

INSERT INTO `sys_role_privilege` VALUES (446629625196445696, 2, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445697, 2, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445698, 2, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445699, 2, 2, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445700, 2, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445701, 2, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445702, 2, 2051, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445703, 2, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445704, 2, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445705, 2, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445706, 2, 7, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445707, 2, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445708, 2, 1032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445709, 2, 1033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445710, 2, 1034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445711, 2, 1035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445712, 2, 1036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445713, 2, 1037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445714, 2, 1038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445715, 2, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445716, 2, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445717, 2, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445718, 2, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445719, 2, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445720, 2, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445721, 2, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445722, 2, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445723, 2, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445724, 2, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445725, 2, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445726, 2, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445727, 2, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445728, 2, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445729, 2, 31, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445730, 2, 32, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445731, 2, 33, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445732, 2, 34, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445733, 2, 35, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445734, 2, 36, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445735, 2, 37, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445736, 2, 38, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445737, 2, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445738, 2, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445739, 2, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445740, 2, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445741, 2, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445742, 2, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445743, 2, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445744, 2, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445745, 2, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445746, 2, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445747, 2, 49, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445748, 2, 50, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445749, 2, 51, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445750, 2, 52, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445751, 2, 53, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445752, 2, 54, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445753, 2, 55, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445754, 2, 56, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445755, 2, 57, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445756, 2, 93, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445757, 2, 94, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445758, 2, 95, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445759, 2, 96, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445760, 2, 97, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445761, 2, 106, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445762, 2, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445763, 2, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445764, 2, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445765, 2, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445766, 2, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445767, 2, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445768, 2, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445769, 2, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445770, 2, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445771, 2, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445772, 2, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445773, 2, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445774, 2, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445775, 2, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445776, 2, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445777, 2, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445778, 2, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445779, 2, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445780, 2, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445781, 2, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445782, 2, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445783, 2, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445784, 2, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445785, 2, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445786, 2, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445787, 2, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445788, 2, 153, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445789, 2, 154, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445790, 2, 155, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445791, 2, 156, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445792, 2, 157, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445793, 2, 158, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445794, 2, 159, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445795, 2, 160, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445796, 2, 161, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445797, 2, 162, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445798, 2, 163, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445799, 2, 164, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445800, 2, 165, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445801, 2, 166, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445802, 2, 167, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445803, 2, 168, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445804, 2, 169, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445805, 2, 170, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445806, 2, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445807, 2, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445808, 2, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445809, 2, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445810, 2, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445811, 2, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445812, 2, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445813, 2, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445814, 2, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445815, 2, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445816, 2, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445817, 2, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445818, 2, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445819, 2, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445820, 2, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445821, 2, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445822, 2, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445823, 2, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445824, 2, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445825, 2, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445826, 2, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445827, 2, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445828, 2, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445829, 2, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445830, 2, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445831, 2, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445832, 2, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445833, 2, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445834, 2, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445835, 2, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445836, 2, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445837, 2, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445838, 2, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445839, 2, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445840, 2, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445841, 2, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445842, 2, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445843, 2, 209, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445844, 2, 210, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445845, 2, 211, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445846, 2, 212, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445847, 2, 213, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445848, 2, 214, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445849, 2, 215, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445850, 2, 216, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445851, 2, 217, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445852, 2, 218, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445853, 2, 219, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445854, 2, 220, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445855, 2, 221, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445856, 2, 222, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445857, 2, 223, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445858, 2, 224, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445859, 2, 225, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445860, 2, 226, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445861, 2, 227, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445862, 2, 228, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445863, 2, 229, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445864, 2, 230, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445865, 2, 231, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445866, 2, 232, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445867, 2, 233, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445868, 2, 234, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445869, 2, 235, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445870, 2, 236, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445871, 2, 237, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445872, 2, 238, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445873, 2, 277, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445874, 2, 278, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445875, 2, 281, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445876, 2, 282, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445877, 2, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445878, 2, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445879, 2, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445880, 2, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445881, 2, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445882, 2, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445883, 2, 2000, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445884, 2, 2001, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445885, 2, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445886, 2, 2002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445887, 2, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445888, 2, 2003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445889, 2, 2004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445890, 2, 2005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445891, 2, 2006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445892, 2, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445893, 2, 2007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445894, 2, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445895, 2, 2008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445896, 2, 2009, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445897, 2, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445898, 2, 2010, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445899, 2, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445900, 2, 2011, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445901, 2, 2012, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445902, 2, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445903, 2, 2013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445904, 2, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445905, 2, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445906, 2, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445907, 2, 2016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445908, 2, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445909, 2, 2017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445910, 2, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445911, 2, 2018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445912, 2, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445913, 2, 2019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445914, 2, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445915, 2, 2020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445916, 2, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445917, 2, 2021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445918, 2, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445919, 2, 2022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445920, 2, 2023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445921, 2, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445922, 2, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445923, 2, 2024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445924, 2, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445925, 2, 2025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445926, 2, 2026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445927, 2, 2027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445928, 2, 2028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445929, 2, 2029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445930, 2, 2030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445931, 2, 2031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445932, 2, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445933, 2, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445934, 2, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445935, 2, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445936, 2, 2036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445937, 2, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445938, 2, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445939, 2, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445940, 2, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445941, 2, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445942, 2, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445943, 2, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445944, 2, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445945, 2, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445946, 2, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445947, 2, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445948, 2, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445949, 2, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445950, 2, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445951, 2, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445952, 2, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445953, 2, 2047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629625196445954, 2, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169920, 3, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169921, 3, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169922, 3, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169923, 3, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169924, 3, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169925, 3, 2051, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169926, 3, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169927, 3, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169928, 3, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169929, 3, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169930, 3, 8, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169931, 3, 9, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169932, 3, 1034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169933, 3, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169934, 3, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169935, 3, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169936, 3, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169937, 3, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169938, 3, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169939, 3, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169940, 3, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169941, 3, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169942, 3, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169943, 3, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169944, 3, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169945, 3, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169946, 3, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169947, 3, 31, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169948, 3, 32, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169949, 3, 33, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169950, 3, 34, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169951, 3, 35, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169952, 3, 36, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169953, 3, 37, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169954, 3, 38, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169955, 3, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169956, 3, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169957, 3, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169958, 3, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169959, 3, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169960, 3, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169961, 3, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169962, 3, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169963, 3, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169964, 3, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169965, 3, 49, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169966, 3, 50, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169967, 3, 52, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169968, 3, 53, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169969, 3, 54, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169970, 3, 55, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169971, 3, 56, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169972, 3, 57, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169973, 3, 98, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169974, 3, 99, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169975, 3, 100, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169976, 3, 101, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169977, 3, 102, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169978, 3, 103, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169979, 3, 104, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169980, 3, 107, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169981, 3, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169982, 3, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169983, 3, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169984, 3, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169985, 3, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169986, 3, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169987, 3, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169988, 3, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169989, 3, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169990, 3, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169991, 3, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169992, 3, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169993, 3, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169994, 3, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169995, 3, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169996, 3, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169997, 3, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169998, 3, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001169999, 3, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170000, 3, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170001, 3, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170002, 3, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170003, 3, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170004, 3, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170005, 3, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170006, 3, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170007, 3, 153, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170008, 3, 154, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170009, 3, 155, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170010, 3, 156, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170011, 3, 157, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170012, 3, 158, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170013, 3, 159, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170014, 3, 160, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170015, 3, 161, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170016, 3, 162, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170017, 3, 163, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170018, 3, 164, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170019, 3, 165, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170020, 3, 166, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170021, 3, 167, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170022, 3, 168, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170023, 3, 169, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170024, 3, 170, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170025, 3, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170026, 3, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170027, 3, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170028, 3, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170029, 3, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170030, 3, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170031, 3, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170032, 3, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170033, 3, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170034, 3, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170035, 3, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170036, 3, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170037, 3, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170038, 3, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170039, 3, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170040, 3, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170041, 3, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170042, 3, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170043, 3, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170044, 3, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170045, 3, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170046, 3, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170047, 3, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170048, 3, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170049, 3, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170050, 3, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170051, 3, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170052, 3, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170053, 3, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170054, 3, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170055, 3, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170056, 3, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170057, 3, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170058, 3, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170059, 3, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170060, 3, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170061, 3, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170062, 3, 218, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170063, 3, 219, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170064, 3, 220, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170065, 3, 221, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170066, 3, 222, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170067, 3, 223, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170068, 3, 224, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170069, 3, 225, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170070, 3, 226, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170071, 3, 227, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170072, 3, 228, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170073, 3, 229, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170074, 3, 230, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170075, 3, 231, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170076, 3, 232, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170077, 3, 233, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170078, 3, 234, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170079, 3, 235, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170080, 3, 236, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170081, 3, 237, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170082, 3, 238, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170083, 3, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170084, 3, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170085, 3, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170086, 3, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170087, 3, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170088, 3, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170089, 3, 2000, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170090, 3, 2001, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170091, 3, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170092, 3, 2002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170093, 3, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170094, 3, 2003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170095, 3, 2004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170096, 3, 2005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170097, 3, 2006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170098, 3, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170099, 3, 2007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170100, 3, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170101, 3, 2008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170102, 3, 2009, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170103, 3, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170104, 3, 2010, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170105, 3, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170106, 3, 2011, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170107, 3, 2012, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170108, 3, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170109, 3, 2013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170110, 3, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170111, 3, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170112, 3, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170113, 3, 2016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170114, 3, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170115, 3, 2017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170116, 3, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170117, 3, 2018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170118, 3, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170119, 3, 2019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170120, 3, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170121, 3, 2020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170122, 3, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170123, 3, 2021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170124, 3, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170125, 3, 2022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170126, 3, 2023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170127, 3, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170128, 3, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170129, 3, 2024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170130, 3, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170131, 3, 2025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170132, 3, 2026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170133, 3, 1002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170134, 3, 2027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170135, 3, 1003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170136, 3, 2028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170137, 3, 1004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170138, 3, 2029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170139, 3, 1005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170140, 3, 2030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170141, 3, 1006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170142, 3, 2031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170143, 3, 1007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170144, 3, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170145, 3, 1008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170146, 3, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170147, 3, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170148, 3, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170149, 3, 2036, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170150, 3, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170151, 3, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170152, 3, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170153, 3, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170154, 3, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170155, 3, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170156, 3, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170157, 3, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170158, 3, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170159, 3, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170160, 3, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170161, 3, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170162, 3, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170163, 3, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170164, 3, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170165, 3, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170166, 3, 2047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629655001170167, 3, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025856, 5, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025857, 5, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025858, 5, 1026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025859, 5, 1027, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025860, 5, 1028, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025861, 5, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025862, 5, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025863, 5, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025864, 5, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025865, 5, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025866, 5, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025867, 5, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025868, 5, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025869, 5, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025870, 5, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025871, 5, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025872, 5, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025873, 5, 26, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025874, 5, 27, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025875, 5, 28, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025876, 5, 29, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025877, 5, 30, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025878, 5, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025879, 5, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025880, 5, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025881, 5, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025882, 5, 43, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025883, 5, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025884, 5, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025885, 5, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025886, 5, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025887, 5, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025888, 5, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025889, 5, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025890, 5, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025891, 5, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025892, 5, 131, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025893, 5, 132, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025894, 5, 133, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025895, 5, 134, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025896, 5, 135, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025897, 5, 136, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025898, 5, 137, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025899, 5, 138, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025900, 5, 139, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025901, 5, 140, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025902, 5, 141, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025903, 5, 142, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025904, 5, 143, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025905, 5, 144, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025906, 5, 145, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025907, 5, 146, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025908, 5, 147, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025909, 5, 148, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025910, 5, 149, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025911, 5, 150, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025912, 5, 151, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025913, 5, 152, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025914, 5, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025915, 5, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025916, 5, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025917, 5, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025918, 5, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025919, 5, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025920, 5, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025921, 5, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025922, 5, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025923, 5, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025924, 5, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025925, 5, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025926, 5, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025927, 5, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025928, 5, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025929, 5, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025930, 5, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025931, 5, 188, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025932, 5, 189, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025933, 5, 190, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025934, 5, 191, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025935, 5, 192, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025936, 5, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025937, 5, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025938, 5, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025939, 5, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025940, 5, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025941, 5, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025942, 5, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025943, 5, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025944, 5, 3017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025945, 5, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025946, 5, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025947, 5, 3019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025948, 5, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025949, 5, 3020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025950, 5, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025951, 5, 3021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025952, 5, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025953, 5, 3022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025954, 5, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025955, 5, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025956, 5, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025957, 5, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025958, 5, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025959, 5, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025960, 5, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025961, 5, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025962, 5, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025963, 5, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025964, 5, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025965, 5, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025966, 5, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025967, 5, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025968, 5, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025969, 5, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025970, 5, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025971, 5, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025972, 5, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025973, 5, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025974, 5, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025975, 5, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025976, 5, 2032, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025977, 5, 2033, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025978, 5, 2034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025979, 5, 2035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025980, 5, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025981, 5, 2037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025982, 5, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025983, 5, 2038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025984, 5, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025985, 5, 2039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025986, 5, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025987, 5, 2040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025988, 5, 2041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025989, 5, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025990, 5, 2042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025991, 5, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025992, 5, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025993, 5, 2044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025994, 5, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025995, 5, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025996, 5, 2045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025997, 5, 2046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025998, 5, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629712396025999, 5, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803968, 8, 1024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803969, 8, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803970, 8, 1025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803971, 8, 1029, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803972, 8, 1030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803973, 8, 1031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803974, 8, 8, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803975, 8, 9, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803976, 8, 10, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803977, 8, 1041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803978, 8, 1042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803979, 8, 19, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803980, 8, 1043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803981, 8, 1044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803982, 8, 20, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803983, 8, 21, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803984, 8, 23, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803985, 8, 24, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803986, 8, 25, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803987, 8, 39, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803988, 8, 40, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803989, 8, 41, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803990, 8, 42, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803991, 8, 44, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803992, 8, 45, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803993, 8, 46, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803994, 8, 47, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803995, 8, 48, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803996, 8, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803997, 8, 98, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803998, 8, 99, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254803999, 8, 100, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804000, 8, 101, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804001, 8, 102, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804002, 8, 103, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804003, 8, 104, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804004, 8, 107, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804005, 8, 108, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804006, 8, 109, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804007, 8, 110, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804008, 8, 127, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804009, 8, 128, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804010, 8, 129, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804011, 8, 130, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804012, 8, 171, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804013, 8, 172, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804014, 8, 173, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804015, 8, 174, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804016, 8, 175, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804017, 8, 176, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804018, 8, 177, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804019, 8, 178, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804020, 8, 179, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804021, 8, 180, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804022, 8, 181, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804023, 8, 182, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804024, 8, 183, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804025, 8, 184, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804026, 8, 185, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804027, 8, 186, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804028, 8, 187, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804029, 8, 193, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804030, 8, 194, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804031, 8, 196, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804032, 8, 197, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804033, 8, 198, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804034, 8, 199, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804035, 8, 200, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804036, 8, 201, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804037, 8, 202, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804038, 8, 203, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804039, 8, 204, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804040, 8, 205, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804041, 8, 206, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804042, 8, 207, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804043, 8, 3024, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804044, 8, 208, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804045, 8, 3025, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804046, 8, 3026, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804047, 8, 3030, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804048, 8, 3031, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804049, 8, 3034, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804050, 8, 3035, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804051, 8, 3037, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804052, 8, 3038, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804053, 8, 2014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804054, 8, 3039, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804055, 8, 3040, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804056, 8, 3041, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804057, 8, 3042, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804058, 8, 3043, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804059, 8, 3044, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804060, 8, 3045, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804061, 8, 3046, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804062, 8, 3047, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804063, 8, 3048, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804064, 8, 3049, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804065, 8, 1002, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804066, 8, 1003, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804067, 8, 1004, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804068, 8, 1005, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804069, 8, 1006, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804070, 8, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804071, 8, 1007, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804072, 8, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804073, 8, 1008, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804074, 8, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804075, 8, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804076, 8, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804077, 8, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804078, 8, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804079, 8, 1013, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804080, 8, 1014, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804081, 8, 1015, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804082, 8, 1016, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804083, 8, 1017, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804084, 8, 1018, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804085, 8, 1019, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804086, 8, 1020, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804087, 8, 1021, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804088, 8, 1022, 1);
INSERT INTO `sys_role_privilege` VALUES (446629888254804089, 8, 1023, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459584, 9, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459585, 9, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459586, 9, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459587, 9, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459588, 9, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459589, 9, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459590, 9, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459591, 9, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629925915459592, 9, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214144, 10, 240, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214145, 10, 1, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214146, 10, 241, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214147, 10, 242, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214148, 10, 243, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214149, 10, 244, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214150, 10, 245, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214151, 10, 72, 1);
INSERT INTO `sys_role_privilege` VALUES (446629954814214152, 10, 239, 1);
INSERT INTO `sys_role_privilege` VALUES (447452410772717568, 11, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927162363904, 12, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927162363905, 12, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558208, 12, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558209, 12, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558210, 12, 3056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558211, 12, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558212, 12, 3058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558213, 12, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558214, 12, 3060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558215, 12, 3061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558216, 12, 3062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558217, 12, 3063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558218, 12, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558219, 12, 3065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558220, 12, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558221, 12, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558222, 12, 3068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558223, 12, 3069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558224, 12, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558225, 12, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465601927166558226, 12, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060928, 9, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060929, 9, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060930, 9, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060931, 9, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060932, 9, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060933, 9, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060934, 9, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060935, 9, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (465602245640060936, 9, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247104, 5, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247105, 5, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247106, 5, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247107, 5, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247108, 5, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247109, 5, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247110, 5, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247111, 5, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247112, 5, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247113, 5, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247114, 5, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247115, 5, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247116, 5, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247117, 5, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247118, 5, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247119, 5, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247120, 5, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247121, 5, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247122, 5, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247123, 5, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247124, 5, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247125, 5, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247126, 5, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247127, 5, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247128, 5, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247129, 5, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247130, 5, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247131, 5, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247132, 5, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247133, 5, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247134, 5, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247135, 5, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247136, 5, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247137, 5, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247138, 5, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247139, 5, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247140, 5, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247141, 5, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247142, 5, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247143, 5, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247144, 5, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247145, 5, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247146, 5, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247147, 5, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247148, 5, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247149, 5, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247150, 5, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247151, 5, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247152, 5, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247153, 5, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247154, 5, 177, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247155, 5, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247156, 5, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247157, 5, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247158, 5, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247159, 5, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247160, 5, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247161, 5, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247162, 5, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247163, 5, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247164, 5, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247165, 5, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247166, 5, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247167, 5, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247168, 5, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247169, 5, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247170, 5, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247171, 5, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247172, 5, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247173, 5, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247174, 5, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247175, 5, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247176, 5, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247177, 5, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247178, 5, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247179, 5, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247180, 5, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247181, 5, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247182, 5, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247183, 5, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247184, 5, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247185, 5, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247186, 5, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247187, 5, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247188, 5, 3041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247189, 5, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247190, 5, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247191, 5, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247192, 5, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247193, 5, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247194, 5, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247195, 5, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247196, 5, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247197, 5, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247198, 5, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247199, 5, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247200, 5, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247201, 5, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247202, 5, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247203, 5, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247204, 5, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247205, 5, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247206, 5, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247207, 5, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471632752467247208, 5, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901888, 11, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901889, 11, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901890, 11, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901891, 11, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901892, 11, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901893, 11, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901894, 11, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901895, 11, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901896, 11, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471633834580901897, 11, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109138706432, 8, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109138706433, 8, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109138706434, 8, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109138706435, 8, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109138706436, 8, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900736, 8, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900737, 8, 2051, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900738, 8, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900739, 8, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900740, 8, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900741, 8, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900742, 8, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900743, 8, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900744, 8, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900745, 8, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900746, 8, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900747, 8, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900748, 8, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900749, 8, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900750, 8, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900751, 8, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900752, 8, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900753, 8, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900754, 8, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900755, 8, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900756, 8, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900757, 8, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900758, 8, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900759, 8, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900760, 8, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900761, 8, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900762, 8, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900763, 8, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900764, 8, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900765, 8, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900766, 8, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900767, 8, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900768, 8, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900769, 8, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900770, 8, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900771, 8, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900772, 8, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900773, 8, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900774, 8, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900775, 8, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900776, 8, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900777, 8, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900778, 8, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900779, 8, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900780, 8, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900781, 8, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900782, 8, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900783, 8, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900784, 8, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900785, 8, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900786, 8, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900787, 8, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900788, 8, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900789, 8, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900790, 8, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900791, 8, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900792, 8, 1073, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900793, 8, 1074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900794, 8, 1075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900795, 8, 1076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900796, 8, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900797, 8, 1077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900798, 8, 1078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900799, 8, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900800, 8, 1079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900801, 8, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900802, 8, 1080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900803, 8, 1081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900804, 8, 1082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900805, 8, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900806, 8, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900807, 8, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900808, 8, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900809, 8, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900810, 8, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900811, 8, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900812, 8, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900813, 8, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900814, 8, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900815, 8, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900816, 8, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900817, 8, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900818, 8, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900819, 8, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900820, 8, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900821, 8, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900822, 8, 1099, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900823, 8, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900824, 8, 1101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900825, 8, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900826, 8, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900827, 8, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900828, 8, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900829, 8, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900830, 8, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900831, 8, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900832, 8, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900833, 8, 99, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900834, 8, 100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900835, 8, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900836, 8, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900837, 8, 103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900838, 8, 104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900839, 8, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900840, 8, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900841, 8, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900842, 8, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900843, 8, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900844, 8, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900845, 8, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900846, 8, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900847, 8, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900848, 8, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900849, 8, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900850, 8, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900851, 8, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900852, 8, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900853, 8, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900854, 8, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900855, 8, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900856, 8, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900857, 8, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900858, 8, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900859, 8, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900860, 8, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900861, 8, 154, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900862, 8, 155, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900863, 8, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900864, 8, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900865, 8, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900866, 8, 159, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900867, 8, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900868, 8, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900869, 8, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900870, 8, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900871, 8, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900872, 8, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900873, 8, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900874, 8, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900875, 8, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900876, 8, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900877, 8, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900878, 8, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900879, 8, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900880, 8, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900881, 8, 177, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900882, 8, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900883, 8, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900884, 8, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900885, 8, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900886, 8, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900887, 8, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900888, 8, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900889, 8, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900890, 8, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900891, 8, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900892, 8, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900893, 8, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900894, 8, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900895, 8, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900896, 8, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900897, 8, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900898, 8, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900899, 8, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900900, 8, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900901, 8, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900902, 8, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900903, 8, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900904, 8, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900905, 8, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900906, 8, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900907, 8, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900908, 8, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900909, 8, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900910, 8, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900911, 8, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900912, 8, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900913, 8, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900914, 8, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900915, 8, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900916, 8, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900917, 8, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900918, 8, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900919, 8, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900920, 8, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900921, 8, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900922, 8, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900923, 8, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900924, 8, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900925, 8, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900926, 8, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900927, 8, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900928, 8, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900929, 8, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900930, 8, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900931, 8, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900932, 8, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900933, 8, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900934, 8, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900935, 8, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900936, 8, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900937, 8, 3041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900938, 8, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900939, 8, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900940, 8, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900941, 8, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900942, 8, 2019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900943, 8, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900944, 8, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900945, 8, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900946, 8, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900947, 8, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900948, 8, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900949, 8, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900950, 8, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900951, 8, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900952, 8, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900953, 8, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900954, 8, 2025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900955, 8, 2026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900956, 8, 1002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900957, 8, 2027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900958, 8, 1003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900959, 8, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900960, 8, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900961, 8, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900962, 8, 1005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900963, 8, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900964, 8, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900965, 8, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900966, 8, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900967, 8, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900968, 8, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900969, 8, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900970, 8, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900971, 8, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900972, 8, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900973, 8, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900974, 8, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900975, 8, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900976, 8, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900977, 8, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900978, 8, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900979, 8, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900980, 8, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900981, 8, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900982, 8, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900983, 8, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (471992109142900984, 8, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396544, 3, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396545, 3, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396546, 3, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396547, 3, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396548, 3, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396549, 3, 3, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396550, 3, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396551, 3, 3076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396552, 3, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396553, 3, 3077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396554, 3, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396555, 3, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396556, 3, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396557, 3, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396558, 3, 3078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396559, 3, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396560, 3, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396561, 3, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396562, 3, 1032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396563, 3, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396564, 3, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396565, 3, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396566, 3, 1035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396567, 3, 1036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396568, 3, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396569, 3, 1038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396570, 3, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396571, 3, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396572, 3, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396573, 3, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396574, 3, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396575, 3, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396576, 3, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396577, 3, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396578, 3, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396579, 3, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396580, 3, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396581, 3, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396582, 3, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396583, 3, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396584, 3, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396585, 3, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396586, 3, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396587, 3, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396588, 3, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396589, 3, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396590, 3, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396591, 3, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396592, 3, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396593, 3, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396594, 3, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396595, 3, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396596, 3, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396597, 3, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396598, 3, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396599, 3, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396600, 3, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396601, 3, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396602, 3, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396603, 3, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396604, 3, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396605, 3, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396606, 3, 43, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396607, 3, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396608, 3, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396609, 3, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396610, 3, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396611, 3, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396612, 3, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396613, 3, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396614, 3, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396615, 3, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396616, 3, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396617, 3, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396618, 3, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396619, 3, 1073, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396620, 3, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396621, 3, 1074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396622, 3, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396623, 3, 1075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396624, 3, 52, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396625, 3, 1076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396626, 3, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396627, 3, 1077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396628, 3, 54, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396629, 3, 1078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396630, 3, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396631, 3, 1079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396632, 3, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396633, 3, 1080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396634, 3, 57, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396635, 3, 1081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396636, 3, 1082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396637, 3, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396638, 3, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396639, 3, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396640, 3, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396641, 3, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396642, 3, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396643, 3, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396644, 3, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396645, 3, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396646, 3, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396647, 3, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396648, 3, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396649, 3, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396650, 3, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396651, 3, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396652, 3, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396653, 3, 74, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396654, 3, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396655, 3, 75, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396656, 3, 1099, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396657, 3, 76, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396658, 3, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396659, 3, 1101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396660, 3, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396661, 3, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396662, 3, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396663, 3, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396664, 3, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396665, 3, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396666, 3, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396667, 3, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396668, 3, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396669, 3, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396670, 3, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396671, 3, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396672, 3, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396673, 3, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396674, 3, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396675, 3, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396676, 3, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396677, 3, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396678, 3, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396679, 3, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396680, 3, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396681, 3, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396682, 3, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396683, 3, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396684, 3, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396685, 3, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396686, 3, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396687, 3, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396688, 3, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396689, 3, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396690, 3, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396691, 3, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396692, 3, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396693, 3, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396694, 3, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396695, 3, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396696, 3, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396697, 3, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396698, 3, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396699, 3, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396700, 3, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396701, 3, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396702, 3, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396703, 3, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396704, 3, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396705, 3, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396706, 3, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396707, 3, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396708, 3, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396709, 3, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396710, 3, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396711, 3, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396712, 3, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396713, 3, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396714, 3, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396715, 3, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396716, 3, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396717, 3, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396718, 3, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396719, 3, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396720, 3, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396721, 3, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396722, 3, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396723, 3, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396724, 3, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396725, 3, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396726, 3, 177, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396727, 3, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396728, 3, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396729, 3, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396730, 3, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396731, 3, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396732, 3, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396733, 3, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396734, 3, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396735, 3, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396736, 3, 188, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396737, 3, 189, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396738, 3, 192, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396739, 3, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396740, 3, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396741, 3, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396742, 3, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396743, 3, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396744, 3, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396745, 3, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396746, 3, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396747, 3, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396748, 3, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396749, 3, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396750, 3, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396751, 3, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396752, 3, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396753, 3, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396754, 3, 209, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396755, 3, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396756, 3, 214, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396757, 3, 218, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396758, 3, 219, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396759, 3, 221, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396760, 3, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396761, 3, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396762, 3, 230, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396763, 3, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396764, 3, 232, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396765, 3, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396766, 3, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396767, 3, 235, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396768, 3, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396769, 3, 237, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396770, 3, 238, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396771, 3, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396772, 3, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396773, 3, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396774, 3, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396775, 3, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396776, 3, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396777, 3, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396778, 3, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396779, 3, 262, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396780, 3, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396781, 3, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396782, 3, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396783, 3, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396784, 3, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396785, 3, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396786, 3, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396787, 3, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396788, 3, 2001, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396789, 3, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396790, 3, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396791, 3, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396792, 3, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396793, 3, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396794, 3, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396795, 3, 2008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396796, 3, 2009, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396797, 3, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396798, 3, 2010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396799, 3, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396800, 3, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396801, 3, 2012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396802, 3, 2013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396803, 3, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396804, 3, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396805, 3, 3041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396806, 3, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396807, 3, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396808, 3, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396809, 3, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396810, 3, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396811, 3, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396812, 3, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396813, 3, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396814, 3, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396815, 3, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396816, 3, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396817, 3, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396818, 3, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396819, 3, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396820, 3, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396821, 3, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396822, 3, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396823, 3, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396824, 3, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396825, 3, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396826, 3, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396827, 3, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396828, 3, 2032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396829, 3, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396830, 3, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396831, 3, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396832, 3, 2035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396833, 3, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396834, 3, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396835, 3, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396836, 3, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396837, 3, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396838, 3, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396839, 3, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396840, 3, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396841, 3, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396842, 3, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396843, 3, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396844, 3, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396845, 3, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396846, 3, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396847, 3, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396848, 3, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396849, 3, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396850, 3, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396851, 3, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396852, 3, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396853, 3, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396854, 3, 2047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396855, 3, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (475377100908396856, 3, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968540000256, 2, 3072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194560, 2, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194561, 2, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194562, 2, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194563, 2, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194564, 2, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194565, 2, 3074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194566, 2, 3, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194567, 2, 2051, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194568, 2, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194569, 2, 3075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194570, 2, 3076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194571, 2, 4, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194572, 2, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194573, 2, 3077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194574, 2, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194575, 2, 1029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194576, 2, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194577, 2, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194578, 2, 3078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194579, 2, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194580, 2, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194581, 2, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194582, 2, 2056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194583, 2, 1032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194584, 2, 2057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194585, 2, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194586, 2, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194587, 2, 1034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194588, 2, 11, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194589, 2, 1035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194590, 2, 12, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194591, 2, 1036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194592, 2, 13, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194593, 2, 1037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194594, 2, 14, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194595, 2, 1038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194596, 2, 15, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194597, 2, 16, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194598, 2, 17, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194599, 2, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194600, 2, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194601, 2, 18, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194602, 2, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194603, 2, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194604, 2, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194605, 2, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194606, 2, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194607, 2, 1045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194608, 2, 1046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194609, 2, 23, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194610, 2, 1047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194611, 2, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194612, 2, 1048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194613, 2, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194614, 2, 1049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194615, 2, 1050, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194616, 2, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194617, 2, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194618, 2, 1052, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194619, 2, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194620, 2, 1053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194621, 2, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194622, 2, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194623, 2, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194624, 2, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194625, 2, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194626, 2, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194627, 2, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194628, 2, 35, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194629, 2, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194630, 2, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194631, 2, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194632, 2, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194633, 2, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194634, 2, 38, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194635, 2, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194636, 2, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194637, 2, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194638, 2, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194639, 2, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194640, 2, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194641, 2, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194642, 2, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194643, 2, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194644, 2, 43, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194645, 2, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194646, 2, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194647, 2, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194648, 2, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194649, 2, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194650, 2, 46, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194651, 2, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194652, 2, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194653, 2, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194654, 2, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194655, 2, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194656, 2, 49, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194657, 2, 1073, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194658, 2, 50, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194659, 2, 1074, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194660, 2, 51, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194661, 2, 1075, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194662, 2, 52, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194663, 2, 1076, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194664, 2, 53, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194665, 2, 1077, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194666, 2, 54, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194667, 2, 1078, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194668, 2, 55, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194669, 2, 1079, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194670, 2, 56, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194671, 2, 1080, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194672, 2, 57, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194673, 2, 1081, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194674, 2, 1082, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194675, 2, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194676, 2, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194677, 2, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194678, 2, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194679, 2, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194680, 2, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194681, 2, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194682, 2, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194683, 2, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194684, 2, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194685, 2, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194686, 2, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194687, 2, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194688, 2, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194689, 2, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194690, 2, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194691, 2, 74, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194692, 2, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194693, 2, 75, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194694, 2, 1099, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194695, 2, 76, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194696, 2, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194697, 2, 77, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194698, 2, 1101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194699, 2, 78, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194700, 2, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194701, 2, 79, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194702, 2, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194703, 2, 80, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194704, 2, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194705, 2, 81, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194706, 2, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194707, 2, 82, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194708, 2, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194709, 2, 83, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194710, 2, 84, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194711, 2, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194712, 2, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194713, 2, 87, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194714, 2, 88, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194715, 2, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194716, 2, 90, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194717, 2, 91, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194718, 2, 92, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194719, 2, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194720, 2, 94, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194721, 2, 95, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194722, 2, 96, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194723, 2, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194724, 2, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194725, 2, 99, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194726, 2, 100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194727, 2, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194728, 2, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194729, 2, 103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194730, 2, 104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194731, 2, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194732, 2, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194733, 2, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194734, 2, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194735, 2, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194736, 2, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194737, 2, 111, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194738, 2, 112, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194739, 2, 113, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194740, 2, 114, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194741, 2, 115, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194742, 2, 116, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194743, 2, 117, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194744, 2, 118, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194745, 2, 119, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194746, 2, 120, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194747, 2, 121, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194748, 2, 122, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194749, 2, 123, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194750, 2, 124, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194751, 2, 125, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194752, 2, 126, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194753, 2, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194754, 2, 128, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194755, 2, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194756, 2, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194757, 2, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194758, 2, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194759, 2, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194760, 2, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194761, 2, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194762, 2, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194763, 2, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194764, 2, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194765, 2, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194766, 2, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194767, 2, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194768, 2, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194769, 2, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194770, 2, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194771, 2, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194772, 2, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194773, 2, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194774, 2, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194775, 2, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194776, 2, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194777, 2, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194778, 2, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194779, 2, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194780, 2, 154, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194781, 2, 155, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194782, 2, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194783, 2, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194784, 2, 158, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194785, 2, 159, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194786, 2, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194787, 2, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194788, 2, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194789, 2, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194790, 2, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194791, 2, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194792, 2, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194793, 2, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194794, 2, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194795, 2, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194796, 2, 170, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194797, 2, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194798, 2, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194799, 2, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194800, 2, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194801, 2, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194802, 2, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194803, 2, 177, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194804, 2, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194805, 2, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194806, 2, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194807, 2, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194808, 2, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194809, 2, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194810, 2, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194811, 2, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194812, 2, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194813, 2, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194814, 2, 188, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194815, 2, 189, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194816, 2, 190, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194817, 2, 191, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194818, 2, 192, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194819, 2, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194820, 2, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194821, 2, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194822, 2, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194823, 2, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194824, 2, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194825, 2, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194826, 2, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194827, 2, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194828, 2, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194829, 2, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194830, 2, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194831, 2, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194832, 2, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194833, 2, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194834, 2, 209, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194835, 2, 210, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194836, 2, 211, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194837, 2, 212, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194838, 2, 213, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194839, 2, 214, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194840, 2, 215, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194841, 2, 216, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194842, 2, 217, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194843, 2, 218, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194844, 2, 219, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194845, 2, 220, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194846, 2, 221, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194847, 2, 222, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194848, 2, 223, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194849, 2, 224, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194850, 2, 225, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194851, 2, 226, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194852, 2, 227, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194853, 2, 228, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194854, 2, 229, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194855, 2, 230, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194856, 2, 231, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194857, 2, 232, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194858, 2, 233, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194859, 2, 234, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194860, 2, 235, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194861, 2, 236, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194862, 2, 237, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194863, 2, 238, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194864, 2, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194865, 2, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194866, 2, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194867, 2, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194868, 2, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194869, 2, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194870, 2, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194871, 2, 248, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194872, 2, 249, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194873, 2, 250, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194874, 2, 252, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194875, 2, 253, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194876, 2, 254, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194877, 2, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194878, 2, 257, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194879, 2, 258, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194880, 2, 259, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194881, 2, 260, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194882, 2, 261, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194883, 2, 262, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194884, 2, 263, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194885, 2, 264, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194886, 2, 265, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194887, 2, 266, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194888, 2, 268, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194889, 2, 269, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194890, 2, 270, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194891, 2, 271, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194892, 2, 272, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194893, 2, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194894, 2, 276, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194895, 2, 277, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194896, 2, 278, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194897, 2, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194898, 2, 282, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194899, 2, 283, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194900, 2, 284, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194901, 2, 288, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194902, 2, 289, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194903, 2, 290, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194904, 2, 300, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194905, 2, 3016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194906, 2, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194907, 2, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194908, 2, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194909, 2, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194910, 2, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194911, 2, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194912, 2, 2000, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194913, 2, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194914, 2, 2001, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194915, 2, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194916, 2, 2002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194917, 2, 2003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194918, 2, 2004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194919, 2, 2005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194920, 2, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194921, 2, 2006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194922, 2, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194923, 2, 2007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194924, 2, 2008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194925, 2, 2009, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194926, 2, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194927, 2, 2010, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194928, 2, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194929, 2, 2011, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194930, 2, 2012, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194931, 2, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194932, 2, 2013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194933, 2, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194934, 2, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194935, 2, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194936, 2, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194937, 2, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194938, 2, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194939, 2, 3041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194940, 2, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194941, 2, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194942, 2, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194943, 2, 2019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194944, 2, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194945, 2, 2020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194946, 2, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194947, 2, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194948, 2, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194949, 2, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194950, 2, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194951, 2, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194952, 2, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194953, 2, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194954, 2, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194955, 2, 2025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194956, 2, 2026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194957, 2, 1002, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194958, 2, 1003, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194959, 2, 2027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194960, 2, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194961, 2, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194962, 2, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194963, 2, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194964, 2, 1005, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194965, 2, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194966, 2, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194967, 2, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194968, 2, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194969, 2, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194970, 2, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194971, 2, 3056, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194972, 2, 2032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194973, 2, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194974, 2, 3057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194975, 2, 2033, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194976, 2, 3058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194977, 2, 2034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194978, 2, 3059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194979, 2, 2035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194980, 2, 3060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194981, 2, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194982, 2, 3061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194983, 2, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194984, 2, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194985, 2, 3062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194986, 2, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194987, 2, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194988, 2, 3063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194989, 2, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194990, 2, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194991, 2, 3064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194992, 2, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194993, 2, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194994, 2, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194995, 2, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194996, 2, 3065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194997, 2, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194998, 2, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544194999, 2, 3066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195000, 2, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195001, 2, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195002, 2, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195003, 2, 3068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195004, 2, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195005, 2, 3069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195006, 2, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195007, 2, 3070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195008, 2, 2047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195009, 2, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (481047968544195010, 2, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886912, 10, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886913, 10, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886914, 10, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886915, 10, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886916, 10, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886917, 10, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886918, 10, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886919, 10, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886920, 10, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886921, 10, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886922, 10, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886923, 10, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886924, 10, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886925, 10, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886926, 10, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886927, 10, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886928, 10, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886929, 10, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886930, 10, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886931, 10, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886932, 10, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886933, 10, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886934, 10, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886935, 10, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886936, 10, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886937, 10, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886938, 10, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886939, 10, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886940, 10, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482178830446886941, 10, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191872, 4, 1024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191873, 4, 1, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191874, 4, 1025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191875, 4, 2, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191876, 4, 1026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191877, 4, 1027, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191878, 4, 1028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191879, 4, 5, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191880, 4, 6, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191881, 4, 1030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191882, 4, 7, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191883, 4, 1031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191884, 4, 8, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191885, 4, 9, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191886, 4, 10, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191887, 4, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191888, 4, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191889, 4, 19, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191890, 4, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191891, 4, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191892, 4, 20, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191893, 4, 21, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191894, 4, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191895, 4, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191896, 4, 27, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191897, 4, 28, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191898, 4, 29, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191899, 4, 30, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191900, 4, 31, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191901, 4, 32, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191902, 4, 33, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191903, 4, 1057, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191904, 4, 34, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191905, 4, 1058, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191906, 4, 1059, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191907, 4, 36, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191908, 4, 1060, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191909, 4, 37, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191910, 4, 1061, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191911, 4, 1062, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191912, 4, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191913, 4, 1063, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191914, 4, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191915, 4, 1064, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191916, 4, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191917, 4, 1065, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191918, 4, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191919, 4, 1066, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191920, 4, 43, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191921, 4, 1067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191922, 4, 44, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191923, 4, 1068, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191924, 4, 45, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191925, 4, 1069, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191926, 4, 1070, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191927, 4, 47, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191928, 4, 1071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191929, 4, 48, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191930, 4, 1072, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191931, 4, 1083, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191932, 4, 1084, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191933, 4, 1085, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191934, 4, 1086, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191935, 4, 1087, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191936, 4, 1088, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191937, 4, 1089, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191938, 4, 1090, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191939, 4, 1091, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191940, 4, 1092, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191941, 4, 1093, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191942, 4, 1094, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191943, 4, 1095, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191944, 4, 72, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191945, 4, 1096, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191946, 4, 1097, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191947, 4, 1098, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191948, 4, 1100, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191949, 4, 1102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191950, 4, 1103, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191951, 4, 1104, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191952, 4, 1105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191953, 4, 1106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191954, 4, 85, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191955, 4, 86, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191956, 4, 89, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191957, 4, 93, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191958, 4, 97, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191959, 4, 98, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191960, 4, 101, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191961, 4, 102, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191962, 4, 105, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191963, 4, 106, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191964, 4, 107, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191965, 4, 108, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191966, 4, 109, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191967, 4, 110, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191968, 4, 127, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191969, 4, 129, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191970, 4, 130, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191971, 4, 131, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191972, 4, 132, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191973, 4, 133, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191974, 4, 134, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191975, 4, 135, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191976, 4, 136, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191977, 4, 137, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191978, 4, 138, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191979, 4, 139, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191980, 4, 140, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191981, 4, 141, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191982, 4, 142, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191983, 4, 143, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191984, 4, 144, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191985, 4, 145, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191986, 4, 146, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191987, 4, 147, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191988, 4, 148, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191989, 4, 149, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191990, 4, 150, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191991, 4, 151, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191992, 4, 152, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191993, 4, 153, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191994, 4, 156, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191995, 4, 157, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191996, 4, 160, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191997, 4, 161, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191998, 4, 162, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818191999, 4, 163, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192000, 4, 164, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192001, 4, 165, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192002, 4, 166, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192003, 4, 167, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192004, 4, 168, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192005, 4, 169, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192006, 4, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192007, 4, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192008, 4, 173, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192009, 4, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192010, 4, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192011, 4, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192012, 4, 177, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192013, 4, 178, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192014, 4, 179, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192015, 4, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192016, 4, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192017, 4, 182, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192018, 4, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192019, 4, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192020, 4, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192021, 4, 186, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192022, 4, 187, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192023, 4, 188, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192024, 4, 189, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192025, 4, 192, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192026, 4, 193, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192027, 4, 194, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192028, 4, 196, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192029, 4, 197, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192030, 4, 198, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192031, 4, 199, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192032, 4, 200, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192033, 4, 201, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192034, 4, 202, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192035, 4, 203, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192036, 4, 204, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192037, 4, 205, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192038, 4, 206, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192039, 4, 207, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192040, 4, 208, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192041, 4, 239, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192042, 4, 240, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192043, 4, 241, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192044, 4, 242, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192045, 4, 243, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192046, 4, 244, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192047, 4, 245, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192048, 4, 256, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213818192049, 4, 275, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386176, 4, 281, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386177, 4, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386178, 4, 3019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386179, 4, 3020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386180, 4, 3021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386181, 4, 3022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386182, 4, 3024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386183, 4, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386184, 4, 3026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386185, 4, 3030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386186, 4, 3031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386187, 4, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386188, 4, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386189, 4, 3037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386190, 4, 3038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386191, 4, 2014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386192, 4, 3039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386193, 4, 3040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386194, 4, 2016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386195, 4, 3041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386196, 4, 2017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386197, 4, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386198, 4, 2018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386199, 4, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386200, 4, 3044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386201, 4, 3045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386202, 4, 2021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386203, 4, 3046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386204, 4, 2022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386205, 4, 2023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386206, 4, 3047, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386207, 4, 3048, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386208, 4, 2024, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386209, 4, 3049, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386210, 4, 2025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386211, 4, 2026, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386212, 4, 2028, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386213, 4, 1004, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386214, 4, 3053, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386215, 4, 2029, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386216, 4, 3054, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386217, 4, 2030, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386218, 4, 1006, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386219, 4, 3055, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386220, 4, 2031, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386221, 4, 1007, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386222, 4, 2032, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386223, 4, 1008, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386224, 4, 2036, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386225, 4, 1013, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386226, 4, 2037, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386227, 4, 2038, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386228, 4, 1014, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386229, 4, 1015, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386230, 4, 2039, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386231, 4, 1016, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386232, 4, 2040, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386233, 4, 2041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386234, 4, 1017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386235, 4, 2042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386236, 4, 1018, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386237, 4, 3067, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386238, 4, 1019, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386239, 4, 1020, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386240, 4, 1021, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386241, 4, 1022, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386242, 4, 1023, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (482181213822386243, 4, 3071, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364096, 6, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364097, 6, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364098, 6, 3025, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364099, 6, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364100, 6, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364101, 6, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364102, 6, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364103, 6, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364104, 6, 26, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364105, 6, 3034, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364106, 6, 3035, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364107, 6, 3042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364108, 6, 3043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364109, 6, 39, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364110, 6, 40, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364111, 6, 41, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364112, 6, 42, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364113, 6, 171, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364114, 6, 172, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364115, 6, 174, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364116, 6, 175, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364117, 6, 176, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364118, 6, 180, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364119, 6, 181, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364120, 6, 183, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364121, 6, 184, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364122, 6, 185, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364123, 6, 2044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364124, 6, 2045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (506123678799364125, 6, 2046, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223872, 7, 1041, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223873, 7, 1042, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223874, 7, 1043, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223875, 7, 1044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223876, 7, 24, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223877, 7, 25, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223878, 7, 3017, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223879, 7, 26, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223880, 7, 2044, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223881, 7, 2045, 1);
INSERT INTO `sys_role_privilege` (`id`, `role_id`, `privilege_id`, `enable`) VALUES (511196891942223882, 7, 2046, 1);

INSERT INTO `sys_user` (`id`, `login_name`, `real_name`, `code`, `password`, `mobile_number`, `org_id`, `gender`, `enable`, `pwd_count`, `pwd_update_time`, `remark`, `source`, `supplier_id`, `rtzf_sync_status`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, 'sysadmin', 'sysadmin', NULL, 'cW10aDIwMjMpKCo=', '0123456789*', NULL, NULL, 1, 1, 1629353956868, NULL, 'SYSTEM', NULL, NULL, NULL, NULL, 1, 1699606957631);

INSERT INTO `sys_user_role` VALUES (1, 1, 1, NULL, 1);

INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (1, 'TEXT', 'FORM', 'problem_point', 'problem_point', '问题简要', 'STRING', 1, 0, 1, 1, NULL, 30, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (2, 'TEXTAREA', 'FORM', 'problem_remark', 'problem_remark', '问题情况说明', 'STRING', 1, 0, 1, 1, NULL, 300, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (3, 'FILE', 'FORM', 'attachments_info', 'attachments_info', '附件说明', 'FILE', 0, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (4, 'TEXT', 'FORM', 'verify_point', 'verify_point', '核实情况简要', 'STRING', 1, 0, 1, 1, NULL, 50, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (5, 'TEXTAREA', 'FORM', 'verify_remark', 'verify_remark', '核实情况备注', 'STRING', 1, 0, 1, 1, NULL, 500, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (6, 'RADIO', 'FORM', 'problem_type', 'problem_type', '质量问题类型', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"UPDATE\",\"label\":\"修正类\"},{\"value\":\"OPTIMIZE\",\"label\":\"优化类\"},{\"value\":\"NO_PROBLEM\",\"label\":\"不是问题\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (7, 'RADIO', 'FORM', 'problem_reason_radio', 'problem_reason_radio', '问题归因', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"EXEC\",\"label\":\"执行类\"},{\"value\":\"MANAGER\",\"label\":\"管理协调类\"},{\"value\":\"PRODUCT_PROBLEM\",\"label\":\"产品缺陷类\"},{\"value\":\"PRODUCT_MANAGER\",\"label\":\"产品运维类\"},{\"value\":\"FLOW\",\"label\":\"流程制度类\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (8, 'RADIO', 'FORM', 'problem_degree_radio', 'problem_degree_radio', '影响度', 'ENUM', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"A\",\"label\":\"A级\"},{\"value\":\"B\",\"label\":\"B级\"},{\"value\":\"C\",\"label\":\"C级\"},{\"value\":\"D\",\"label\":\"D级\"}]', 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (9, 'MULTIPLE_SELECT', 'FORM', 'problem_users', 'problem_users', '责任人', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=ALL', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (10, 'MULTIPLE_SELECT', 'FORM', 'approve_check_users', 'approve_check_users', '甲方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (11, 'MULTIPLE_SELECT', 'FORM', 'approve_check_third_users', 'approve_check_third_users', '乙方复审人员', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, '/api/admin/common/get_user_list/form_widget?userSource=SYSTEM', NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (12, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (13, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (14, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (15, 'RADIO', 'FORM', 'approve_radio', 'approve_radio', '审批意见', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (16, 'SIGN', 'FORM', 'sign', 'sign', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (17, 'TEXTAREA', 'FORM', 'approve_remark', 'approve_remark', '备注', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'QUALITY_PROBLEM_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (18, 'RADIO', 'FORM', 'service_scope_radio', 'service_scope_radio', '服务范围', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"SCAN\",\"label\":\"仅扫描\"},{\"value\":\"SCAN_MARK\",\"label\":\"扫描+阅卷\"}]', 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (19, 'RADIO', 'FORM', 'deploy_way_radio', 'deploy_way_radio', '部署方式', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"ONLINE\",\"label\":\"线上部署\"},{\"value\":\"LOCAL\",\"label\":\"本地部署\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (20, 'RADIO', 'FORM', 'mark_way_radio', 'mark_way_radio', '阅卷方式', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"TOGETHER\",\"label\":\"集中阅卷\"},{\"value\":\"ANYTIME\",\"label\":\"分散阅卷\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (21, 'RADIO', 'FORM', 'scan_net_radio', 'scan_net_radio', '扫描场地网络情况', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"INTERNET\",\"label\":\"提供外网\"},{\"value\":\"LOCALHOST\",\"label\":\"不提供外网\"}]', 'OFFICE_SOP_FLOW', 1, 0, 1000, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (22, 'TEXT', 'FORM', 'contacts', 'contacts', '联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (23, 'TEXT', 'FORM', 'mobile_number', 'mobile_number', '电话', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (24, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (25, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (26, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '阅卷参数检查时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (27, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '阅卷收尾检查时间', 'LONG', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (28, 'MULTIPLE_SELECT', 'FORM', 'engineer_users_id', 'engineer_users_id', '工程师', 'ARRAY', 1, 1, 0, 1, NULL, NULL, NULL, '/api/admin/user/archives/allocation/can_choose?crmDetailId=#{id}&sopRoleType=ENGINEER', NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 11);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (29, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 1, 0, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (30, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (31, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (32, 'NUMBER', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (33, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (34, 'FILE', 'FORM', 'scan_set', 'scan_set', '扫描仪设置', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (35, 'FORM_GROUP_TITLE', 'FORM', 'scan_test_title', 'scan_test_title', '试扫检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (36, 'FILE', 'FORM', 'scan_test', 'scan_test', '试扫检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 2, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (37, 'FORM_GROUP_TITLE', 'FORM', 'scan_end_title', 'scan_end_title', '扫描收尾检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (38, 'FILE', 'FORM', 'scan_end', 'scan_end', '上传检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (39, 'FORM_GROUP_TITLE', 'FORM', 'not_upload_confirm_title', 'not_upload_confirm_title', '未上传确认', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (40, 'FILE', 'FORM', 'not_upload_confirm', 'not_upload_confirm', '未上传确认', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (41, 'FORM_GROUP_TITLE', 'FORM', 'identify_title', 'identify_title', '识别对照', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (42, 'FILE', 'FORM', 'identify', 'identify', '识别对照', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (43, 'FORM_GROUP_TITLE', 'FORM', 'absent_title', 'absent_title', '缺考识别检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (44, 'FILE', 'FORM', 'absent', 'absent', '缺考识别检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (45, 'FORM_GROUP_TITLE', 'FORM', 'manual_confirm_title', 'manual_confirm_title', '人工确认检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (46, 'FILE', 'FORM', 'manual_confirm', 'manual_confirm', '人工确认检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 3, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (47, 'FORM_GROUP_TITLE', 'FORM', 'mark_paper_title', 'mark_paper_title', '阅卷参数检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (48, 'FILE', 'FORM', 'mark_paper', 'mark_paper', '阅卷参数检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (49, 'FORM_GROUP_TITLE', 'FORM', 'mark_paper_progress_title', 'mark_paper_progress_title', '阅卷进度检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (50, 'FILE', 'FORM', 'mark_paper_progress', 'mark_paper_progress', '阅卷进度检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (51, 'FORM_GROUP_TITLE', 'FORM', 'objective_check_title', 'objective_check_title', '实考客观题0分检查', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (52, 'FILE', 'FORM', 'objective_check', 'objective_check', '实考客观题0分检查', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (53, 'FORM_GROUP_TITLE', 'FORM', 'subjective_check_title', 'subjective_check_title', '实考主观题0分，客观题有分', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (54, 'FILE', 'FORM', 'subjective_check', 'subjective_check', '实考主观题0分，客观题有分', 'ARRAY', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'OFFICE_SOP_FLOW', 5, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (92, 'FORM_GROUP_TITLE', 'FORM', 'project_basic_info_title', 'project_basic_info_title', '项目基本信息', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (93, 'NUMBER', 'FORM', 'course_count', 'course_count', '科目数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 2);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (94, 'NUMBER', 'FORM', 'exam_student_course_count', 'exam_student_course_count', '考生科次', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 3);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (95, 'RADIO', 'FORM', 'bar_code_paper_count', 'bar_code_paper_count', '条码标准张数', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"3\",\"label\":\"3张\"},{\"value\":\"4\",\"label\":\"4张\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 4);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (96, 'RADIO_WITH_INPUT', 'FORM', 'bar_code_paste_way', 'bar_code_paste_way', '条码粘贴方式', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"EXAM_STUDENT\",\"label\":\"考生自贴/数码印刷\"},{\"value\":\"PAPER\",\"label\":\"回卷后粘贴\"},{\"value\":\"OTHER\",\"label\":\"其它\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 5);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (97, 'RADIO', 'FORM', 'has_objective_card', 'has_objective_card', '有无客观题卡', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"HAVA\",\"label\":\"有\"},{\"value\":\"NOT_HAVA\",\"label\":\"无\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 6);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (98, 'NUMBER', 'FORM', 'scan_count', 'scan_count', '扫描仪数量(台)', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 7);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (99, 'DATE', 'FORM', 'exam_student_Internal_date', 'exam_student_Internal_date', '考生数据内部交接时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 8);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (100, 'NUMBER', 'FORM', 'week_people_day', 'week_people_day', '现场标准服务周期（人天）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 9);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (101, 'DATE', 'FORM', 'project_key_info_date', 'project_key_info_date', '项目关键信息提交截止时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 10);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (102, 'DATE', 'FORM', 'service_finish_plan_date', 'service_finish_plan_date', '现场服务完成撤场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 12);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (103, 'FORM_GROUP_TITLE', 'FORM', 'project_people_plan_title', 'project_people_plan_title', '项目人员安排及风险预估', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 13);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (104, 'SINGLE_SELECT', 'FORM', 'region_user_id', 'region_user_id', '区域协调人', 'STRING', 1, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 14);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (105, 'SINGLE_SELECT', 'FORM', 'engineer_user_id', 'engineer_user_id', '实施工程师', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 15);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (106, 'MULTIPLE_SELECT', 'FORM', 'assistant_engineer_user_id', 'assistant_engineer_user_id', '助理实施工程师', 'ARRAY', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 4, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 16);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (107, 'ONLY_TITLE', 'FORM', 'project_rish_title', 'project_rish_title', '项目风险预估（仅供参考）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 17);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (108, 'RADIO', 'FORM', 'delay_rish_cb', 'delay_rish_cb', '延期风险', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 18);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (109, 'RADIO', 'FORM', 'engineer_rish_cb', 'engineer_rish_cb', '实施难度', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"LOW\",\"label\":\"低\"},{\"value\":\"MIDDLE\",\"label\":\"中\"},{\"value\":\"HIGH\",\"label\":\"高\"}]', 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 19);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (110, 'TEXTAREA', 'FORM', 'other_remark', 'other_remark', '其它备注（建议关注的其它方面）', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 20);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (111, 'FORM_GROUP_TITLE', 'FORM', 'project_contacts_title', 'project_contacts_title', '项目联系人', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 21);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (112, 'TABLE', 'FORM', 'project_contacts_table', 'project_contacts_table', '项目联系人表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 22);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (113, 'TEXT', 'TABLE', 'school_college', 'school_college', '项目联系人表格_机构输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 23);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (114, 'TEXT', 'TABLE', 'school_name', 'school_name', '项目联系人表格_姓名输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 24);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (115, 'TEXT', 'TABLE', 'school_job', 'school_job', '项目联系人表格_职务输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 25);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (116, 'TEXT', 'TABLE', 'school_tel', 'school_tel', '项目联系人表格_电话输入框', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 3, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 0, NULL, NULL, 1, 26);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (117, 'FORM_GROUP_TITLE', 'FORM', 'cloud_mark_title', 'cloud_mark_title', '云阅卷（填写前请先认真查阅移交的项目初审及项目基本信息）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (118, 'RADIO', 'FORM', 'run_env_accord', 'run_env_accord', '运行环境调研是否符合要求', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"ACCORD\",\"label\":\"符合\"},{\"value\":\"UN_ACCORD\",\"label\":\"不符合\"}]', 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (119, 'DATE', 'FORM', 'env_deployment_time', 'env_deployment_time', '环境部署时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (120, 'LABEL', 'FORM', 'env_list_label', 'env_list_label', '[{\"label\":\"参照《软硬件准备检查清单》对学校进行调研\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (121, 'DATE', 'FORM', 'scan_start_time', 'scan_start_time', '扫描开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (122, 'DATE', 'FORM', 'scan_end_time', 'scan_end_time', '扫描结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (123, 'LABEL', 'FORM', 'unite_label', 'unite_label', '[{\"label\":\"统一元旦后安排进场\",\"color\":\"blue\"}]', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (124, 'DATE', 'FORM', 'mark_room_check_time', 'mark_room_check_time', '评卷机房检查时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (125, 'TEXT', 'FORM', 'mark_room_count', 'mark_room_count', '评卷机房数量（个）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (126, 'TEXT', 'FORM', 'max_mark_teacher_count', 'max_mark_teacher_count', '最大评卷教师数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (127, 'DATE', 'FORM', 'mark_param_time', 'mark_param_time', '评卷参数提供时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (128, 'DATE', 'FORM', 'mark_start_time', 'mark_start_time', '评卷开始时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (129, 'DATE', 'FORM', 'mark_end_time', 'mark_end_time', '评卷结束时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 1, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (130, 'TEXT', 'FORM', 'choose_course_count', 'choose_course_count', '有选做题的科目数量（门）', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (131, 'TEXTAREA', 'FORM', 'ramark', 'ramark', '其它特殊要求', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 2, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (133, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_region', 'inside_title_region', '大区经理项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (134, 'RADIO', 'FORM', 'approve_radio_region', 'approve_radio_region', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (135, 'SIGN', 'FORM', 'sign_region', 'sign_region', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (136, 'TEXTAREA', 'FORM', 'approve_ramark_region', 'approve_ramark_region', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 1, 'REGION_MANAGER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (137, 'FORM_GROUP_TITLE', 'FORM', 'inside_title_engineer', 'inside_title_engineer', '工程师项目内审（请于24小时内完成内审）', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (138, 'RADIO', 'FORM', 'approve_radio_engineer', 'approve_radio_engineer', '我对以上项目关键信息已审核，确认内容无误：', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"同意\"},{\"value\":\"0\",\"label\":\"不同意\"}]', 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (139, 'SIGN', 'FORM', 'sign_engineer', 'sign_engineer', '添加签名', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (140, 'TEXTAREA', 'FORM', 'approve_ramark_engineer', 'approve_ramark_engineer', '审批意见', 'STRING', 0, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 3, 0, 2, 'EFFECT_ENGINEER', 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (141, 'FORM_GROUP_TITLE', 'FORM', 'upload_list_title', 'upload_list_title', '上传清单和报告', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (144, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备出库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (145, 'DATE', 'FORM', 'device_out_time', 'device_out_time', '设备出库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (146, 'DEVICE_OUT_TABLE', 'FORM', 'device_out_table', 'device_out_table', '设备出库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 4, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (147, 'FORM_GROUP_TITLE', 'FORM', 'import_data_title', 'import_data_title', '导入数据', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (148, 'CHECKBOX', 'FORM', 'data_same', 'data_same', '导入数据', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"true\",\"label\":\"导入数据与原始数据是否一致\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (149, 'NUMBER', 'FORM', 'import_exam_student_count', 'import_exam_student_count', '导入考生数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (150, 'FORM_GROUP_TITLE', 'FORM', 'scan_set_title', 'scan_set_title', '扫描仪设置', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (151, 'CHECKBOX', 'FORM', 'scan_set_cb', 'scan_set_cb', '扫描仪设置', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、扫描仪分辨率设置是150\"},{\"value\":\"2\",\"label\":\"2、是否勾选“自动剪切”，“纠正歪斜”\"},{\"value\":\"3\",\"label\":\"3、是否勾选“边界去除”\"},{\"value\":\"4\",\"label\":\"4、勾选“双张进纸检测”-“有效”-动作“停止”\"},{\"value\":\"5\",\"label\":\"5、爱普生型号，选择“扫描模式”-“设置”-“装订边缘”-“上”\"},{\"value\":\"6\",\"label\":\"6、华高扫描仪型号，检查背面图像；如背面图像倒转，则勾选“背面旋转180°”\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (152, 'FORM_GROUP_TITLE', 'FORM', 'card_format_title', 'card_format_title', '卡格式制作', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (153, 'CHECKBOX', 'FORM', 'card_format_cb', 'card_format_cb', '卡格式制作', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、定位线：上方紧挨装钉“线”字，但是不超过“线”字；考生信息遮挡区需按要求框选\"},{\"value\":\"2\",\"label\":\"2、扫描原图正反面检查是否正确\"},{\"value\":\"3\",\"label\":\"3、扫描裁切图屏蔽区检查是否正确\"}]', 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (154, 'FORM_GROUP_TITLE', 'FORM', 'paper_list_title', 'paper_list_title', '纸质检查表上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (155, 'FILE', 'FORM', 'paper_check_photos', 'paper_check_photos', '上传关键节点检查(纸质)表单拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 6, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (156, 'FORM_GROUP_TITLE', 'FORM', 'manual_bar_code_title', 'manual_bar_code_title', '手工输入条码确认', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (157, 'CHECKBOX', 'FORM', 'manual_bar_code_cb', 'manual_bar_code_cb', '手工输入条码确认', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, '[{\"value\":\"1\",\"label\":\"“数据检查”-“人工确认”，”确认已完成”\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (158, 'NUMBER', 'FORM', 'complete_count', 'complete_count', '完成数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (159, 'FORM_GROUP_TITLE', 'FORM', 'bar_code_full_title', 'bar_code_full_title', '条码数量完整性检查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (160, 'CHECKBOX', 'FORM', 'bar_code_full_cb', 'bar_code_full_cb', '条码数量完整性检查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"“缺页异常”查询需无数据“\"},{\"value\":\"2\",\"label\":\"“无条码考生”查询需无数据“\"},{\"value\":\"3\",\"label\":\"“追加题卡”需和学校记录一致“\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (161, 'FORM_GROUP_TITLE', 'FORM', 'missing_list_title', 'missing_list_title', '缺考名单比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (162, 'CHECKBOX', 'FORM', 'missing_list_cb', 'missing_list_cb', '缺考名单比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"核对“无答题卡考生”的考生是否与学校提供的缺考名单吻合，人工指定为缺考（缺考一致性>导入缺考>导入缺考缺页）；需核对平台“考生管理”中状态为“未上传”的考生是否全部为缺考、是否全部变为“人工指定”。\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (163, 'NUMBER', 'FORM', 'missing_course_count', 'missing_course_count', '缺考科次数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (164, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (165, 'RADIO', 'FORM', 'school_sign_radio', 'school_sign_radio', '学校是否确认签字', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"是\"},{\"value\":\"NO\",\"label\":\"否\"}]', 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (166, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 7, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (167, 'FORM_GROUP_TITLE', 'FORM', 'mark_param_check_title', 'mark_param_check_title', '评卷参数核对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (168, 'CHECKBOX', 'FORM', 'mark_param_check_cb', 'mark_param_check_cb', '评卷参数核对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核查每个科目的满分是否正确(100分、150分、300分)\"},{\"value\":\"2\",\"label\":\"2、检查所有科目结构和分组是否全部导入。（评卷管理>评卷进度查看分组状态）\"}]', 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (169, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (170, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 8, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (171, 'FORM_GROUP_TITLE', 'FORM', 'score_check_title', 'score_check_title', '分数复核', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (172, 'CHECKBOX', 'FORM', 'score_check_cb', 'score_check_cb', '分数复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校复核小题0分，大题0分，全卷0分的情况\"}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (173, 'RADIO', 'FORM', 'school_score_check_radio', 'school_score_check_radio', '学校是否选择复核', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"YES\",\"label\":\"复核(推荐)\"},{\"value\":\"NO\",\"label\":\"不复核\",\"remark\":[{\"content\":\"若选择该项，则需要学校签字确认，签字随关键节点检查表上传\",\"color\":\"red\"}]}]', 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (175, 'FORM_GROUP_TITLE', 'FORM', 'paper_form_upload_title', 'paper_form_upload_title', '纸质表单上传', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (176, 'FILE', 'FORM', 'paper_form_photos', 'paper_form_photos', '上传关键节点检查表(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 9, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (177, 'FORM_GROUP_TITLE', 'FORM', 'mark_progress_title', 'mark_progress_title', '评卷进度核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (178, 'CHECKBOX', 'FORM', 'mark_progress_cb', 'mark_progress_cb', '评卷进度核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对阅卷总进度是否为100%\"},{\"value\":\"2\",\"label\":\"2、核对科目阅卷进度是否为100%\"},{\"value\":\"3\",\"label\":\"3、核对问题卷是否处理完成\"},{\"value\":\"4\",\"label\":\"4、核对所有科目状态点击“结束”\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (179, 'FORM_GROUP_TITLE', 'FORM', 'cross_check_title', 'cross_check_title', '交叉比对', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (180, 'CHECKBOX', 'FORM', 'cross_check_cb', 'cross_check_cb', '交叉比对', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"是否建议学校进行同一考生多科目分数交叉比对（成绩复核-成绩校验），需所有科目全部改完后管理员账号才能查看。\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (181, 'FORM_GROUP_TITLE', 'FORM', 'mark_submit_title', 'mark_submit_title', '成绩提交核查', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (182, 'CHECKBOX', 'FORM', 'mark_submit_cb', 'mark_submit_cb', '成绩提交核查', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, '[{\"value\":\"1\",\"label\":\"1、核对导出成绩表数据条数是否与考生表数量一致，不能带有“未导出全量数据”字样\"},{\"value\":\"2\",\"label\":\"2、导出验收报告（考试管理-详情-验收报告），和学校签字确认盖章，上报验收报告至质控平台\"},{\"value\":\"3\",\"label\":\"3、核对导出图片工具是否正常运行\"}]', 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (183, 'NUMBER', 'FORM', 'export_score_count', 'export_score_count', '导出成绩数量', 'INT', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (184, 'FILE', 'FORM', 'upload_check_report_photos', 'upload_check_report_photos', '上传验收报告(纸质)拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 10, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (185, 'ONLY_TITLE', 'FORM', 'device_in_out_label', 'device_in_out_label', '设备入库', 'STRING', 0, 1, 0, 1, NULL, NULL, NULL, NULL, NULL, NULL, 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (186, 'DATE', 'FORM', 'device_in_time', 'device_in_time', '设备入库时间', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (187, 'DEVICE_IN_TABLE', 'FORM', 'device_in_table', 'device_in_table', '设备入库表格', 'STRING', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 11, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (188, 'FILE', 'FORM', 'upload_device_version_photos', 'upload_device_version_photos', '上传扫描仪固件版本号拍照', 'ARRAY', 1, 0, 1, 1, NULL, 5, NULL, NULL, NULL, NULL, 12, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 5, 0, NULL, NULL, 1, 1);
INSERT INTO `t_d_form_widget` (`id`, `code`, `type`, `form_id`, `form_name`, `title`, `input_type`, `required`, `readable`, `writable`, `visable`, `scale`, `length`, `binding`, `data_grid`, `tips`, `format`, `span`, `sub_title`, `options`, `flow_type`, `form_setup`, `handle`, `form_group`, `role_type`, `create_id`, `create_time`) VALUES (190, 'DATE', 'FORM', 'service_finish_plan_begin_date', 'service_finish_plan_begin_date', '现场服务完成进场计划时间', 'LONG', 1, 0, 1, 1, NULL, NULL, NULL, NULL, NULL, 'yyyy-MM-dd', 6, NULL, NULL, 'CLOUD_MARK_SOP_FLOW', 1, 1, NULL, NULL, 1, 11);

INSERT INTO `t_d_table_prop` VALUES (5, 112, 1, 'college', 'college', '学院/分（子）机构', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 113, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (6, 112, 2, 'name', 'name', '姓名', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 114, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (7, 112, 3, 'job', 'job', '职务', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 115, NULL, 1, 1);
INSERT INTO `t_d_table_prop` VALUES (8, 112, 4, 'mobileNumber', 'mobileNumber', '电话', NULL, NULL, NULL, NULL, NULL, NULL, NULL, 1, 1, 116, NULL, 1, 1);

INSERT INTO `sys_device_brand` (`id`, `brand`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, '佳能', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` (`id`, `brand`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, '东芝', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` (`id`, `brand`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, '爱普生', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` (`id`, `brand`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, '华高', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_brand` (`id`, `brand`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, '学校自购', NULL, NULL, NULL, NULL);

INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (1, 1, '3300', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (2, 2, '3400', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (3, 3, 'DS-31100', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (4, 3, 'DS-31200', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (5, 4, '1080', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (6, 4, '5100', NULL, NULL, NULL, NULL);
INSERT INTO `sys_device_model` (`id`, `brand_id`, `model`, `create_id`, `create_time`, `update_id`, `update_time`) VALUES (7, 5, '虚拟型号', NULL, NULL, NULL, NULL);

INSERT INTO `sys_ding_date` VALUES (1, 2023, 9, 29, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (2, 2023, 9, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (3, 2023, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (4, 2023, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (5, 2023, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (6, 2023, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (7, 2023, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (8, 2023, 10, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (9, 2023, 10, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (10, 2023, 10, 8, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (11, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (12, 2023, 12, 30, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (13, 2024, 1, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (14, 2024, 2, 4, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (15, 2024, 2, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (16, 2024, 2, 11, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (17, 2024, 2, 12, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (18, 2024, 2, 13, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (19, 2024, 2, 14, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (20, 2024, 2, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (21, 2024, 2, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (22, 2024, 2, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (23, 2024, 2, 18, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (24, 2024, 4, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (25, 2024, 4, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (26, 2024, 4, 6, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (27, 2024, 4, 7, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (28, 2024, 5, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (29, 2024, 5, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (30, 2024, 5, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (31, 2024, 5, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (32, 2024, 5, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (33, 2024, 5, 11, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (34, 2024, 6, 8, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (35, 2024, 6, 9, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (36, 2024, 6, 10, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (37, 2024, 9, 14, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (38, 2024, 9, 29, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (39, 2024, 9, 15, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (40, 2024, 9, 16, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (41, 2024, 9, 17, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (42, 2024, 10, 12, 'WEEKDAY');
INSERT INTO `sys_ding_date` VALUES (43, 2024, 10, 1, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (44, 2024, 10, 2, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (45, 2024, 10, 3, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (46, 2024, 10, 4, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (47, 2024, 10, 5, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (48, 2024, 10, 7, 'LEGAL_HOLIDAYS');
INSERT INTO `sys_ding_date` VALUES (49, 2024, 4, 28, 'WEEKDAY');

INSERT INTO `t_f_custom_flow_allocation` VALUES (1, 'PROJECT_EXCHANGE_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);
INSERT INTO `t_f_custom_flow_allocation` VALUES (2, 'QUALITY_PROBLEM_FLOW', NULL, NULL, '{\"approveConfig\":[{\"setup\":2,\"roleTypes\":[{\"role\":\"QA\"}]}]}', NULL);

INSERT INTO `sys_config` VALUES (1, NULL, 'sys.user.initPassword', '初始密码', '123456', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (2, NULL, 'sys.warning.days', '预警天数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (3, NULL, 'sys.code.enable', '是否启用短信验证码', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (4, NULL, 'sys.message.enable', '是否启用短信消息提示', 'true', 'true-启用，false-禁用', 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (5, NULL, 'sys.message.resendCount', '失败短信重试次数', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (6, NULL, 'attachment.type', '附件类型', '[.xlsx,.xls,.doc,.docx,.pdf,.jpg,.jpeg,.png,.html,.zip,.mp3,.wav,.dll,.exe,.ftl,.bpmn,.xml]', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (7, NULL, 'attachment.length', '附件名称长度', '100', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (8, NULL, 'attachment.size', '附件大小 ', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (9, NULL, 'thread.pool.core.size', '自定义线程池大小', '200', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (10, NULL, 'custom.thread.pool.core.size', '是否自定义线程池大小', 'true', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (11, NULL, 'session.active', 'session会话时长', '6h', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (12, NULL, 'sms.normal.code', '万能短信验证码', 'qmth', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (13, NULL, 'code.expired.time', '短信验证码有效时间', '2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (14, NULL, 'code.send.interval', '短信验证码发送间隔', '60', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (15, NULL, 'sys.txt.charset', 'txt文件编码', 'UTF-8', NULL, 1, 1, 1, NULL, NULL, 1700209205546);
INSERT INTO `sys_config` VALUES (16, NULL, 'admin.logo.url', '超级管理员logo', 'http://qmth-test.oss-cn-shenzhen.aliyuncs.com/frontend/wx_logo.png', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (17, NULL, 'sms.sign.name', '短信签名', '启明泰和', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (18, NULL, 'sms.tpl.code', '登录短信验证码', 'SMS_147416565', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (19, NULL, 'flow.code.length', '流程编码长度', '3', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (20, NULL, 'flow.job.before.time', '流程提前提醒任务时间（单位：分）', '720', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (21, NULL, 'flow.job.after.time', '流程超时提醒任务时间（单位：分）', '1440', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (22, NULL, 'notice.job.after.time', '公告超时提醒任务时间（单位：分）', '30', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (23, NULL, 'remind.job.poll.time', '任务提醒轮询时间', '0 0/5 * * * ?', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (24, NULL, 'remind.job.db.limit', '任务提醒查询条数', 'limit 0,100', NULL, 1, 2, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (25, NULL, 'sms.notice.code', '系统通知公告', 'SMS_462685325', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (26, NULL, 'sms.sop.before.remind.code', '教务处/研究生SOP提前提醒通知', 'SMS_462805277', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (27, NULL, 'sms.sop.done.code', '教务处/研究生SOP节点负责人通知', 'SMS_462730333', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (28, NULL, 'sms.flow.project.exchange.finish.code', '系统计划变更审核通知', 'SMS_462680680', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (29, NULL, 'sms.flow.ding.exception.finish.code', '异常审核通知', 'SMS_462700329', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (30, NULL, 'sms.sop.after.remind.code', '延期提醒通知', 'SMS_462700328', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (31, NULL, 'sms.violation.remind.code', '违规提醒通知', 'SMS_462805303', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (32, NULL, 'baidu.face.compare.url', '百度人脸识别人脸比对接口', '/face-api/v3/face/match', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (33, NULL, 'baidu.face.compare.score', '百度人脸识别人脸比对阈值', '70', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (34, NULL, 'fxxk.app.auth.url', '纷享销客应用级授权接口', 'https://open.fxiaoke.com/cgi/corpAccessToken/get/V2', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (35, NULL, 'fxxk.user.by.mobile.url', '纷享销客根据手机号查询员工接口', 'https://open.fxiaoke.com/cgi/user/getByMobile', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (36, NULL, 'fxxk.custom.query.url', '纷享销客查询自定义对象接口', 'https://open.fxiaoke.com/cgi/crm/custom/v2/data/query', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (37, NULL, 'fxxk.user.query.url', '纷享销客查询人员对象的列表', 'https://open.fxiaoke.com/cgi/crm/v2/data/query', NULL, 1, 1, 1, NULL, NULL, NULL);
INSERT INTO `sys_config` VALUES (38, NULL, 'rtzf.oss.login.url', '软通智服单点登录接口', 'https://qmth.echo-isoftstone.com/login', NULL, 1, 1, 1, NULL, NULL, NULL);